#include "framecpp/config.h"

#include "general/AtExit.hh"

#include "framecpp/DetectorMap.hh"

SINGLETON_TS_INST( FrameCPP::DETECTOR_MAP );

FrameCPP::DETECTOR_MAP::detector_map_type
FrameCPP::DETECTOR_MAP::m_detector_to_prefix;

FrameCPP::DETECTOR_MAP::
DETECTOR_MAP( )
{
  //---------------------------------------------------------------------
  // Clean up memory usage so memory checkers do not complain
  //---------------------------------------------------------------------
  General::AtExit::Append( singleton_suicide );
  //---------------------------------------------------------------------
  // Initialize things that will be needed
  //---------------------------------------------------------------------
  m_detector_to_prefix[ "TAMA_300" ] = "T1";
  m_detector_to_prefix[ "Virgo_CITF" ] = "V1";
  m_detector_to_prefix[ "Virgo" ] = "V2";
  m_detector_to_prefix[ "GEO_600" ] = "G1";
  m_detector_to_prefix[ "LHO_2k" ] = "H2";
  m_detector_to_prefix[ "LHO_4k" ] = "H1";
  m_detector_to_prefix[ "LLO_4k" ] = "L1";
  m_detector_to_prefix[ "CIT_40" ] = "P1";
  m_detector_to_prefix[ "ALLEGRO_" ] = "A1";
  m_detector_to_prefix[ "AURIGA" ] = "O1";
  m_detector_to_prefix[ "EXPLORER" ] = "E1";
  m_detector_to_prefix[ "NIOBE" ] = "B1";
  m_detector_to_prefix[ "Nautilus" ] = "N1";
  m_detector_to_prefix[ "ACIGA" ] = "K1";
}

const char* FrameCPP::DETECTOR_MAP::
DetectorToPrefix( const std::string& Detector )
{
  detector_map_type::const_iterator	last = m_detector_to_prefix.end( );
  detector_map_type::const_iterator
    cur = m_detector_to_prefix.find( Detector.c_str( ) );

  if ( ( cur == last ) &&
       ( Detector.substr(0, 8) == "ALLEGRO_" ) )
  {
    cur = m_detector_to_prefix.find( "ALLEGRO_" );
  }
  if ( cur == last )
  {
    return "";
  }
  else
  {
    return (*cur).second;
  }
}

