#ifndef FrameCppClassInfoHH
#define FrameCppClassInfoHH


#include "base.hh"
#include "sh.hh"


namespace FrameCPP {

namespace Version_3_4_5 {


//-----------------------------------------------------------------------------
//
//: Frame Class Information.
//
// This class stores information about the Frame classes which is needed during
// the write and read procedures.  This information is: <p><ol>
//
// <li> Structure Header - This is used by the Dictionary class to identify
//    what type of object is being read from the stream.  It is also used by
//    the Output class to write the header for an object before its first
//    appearance. </li>
//
// <li> Pointer to read function - This is used by the Dictionary class to read
//    a frame object into memory once it is identified on the stream.</li></ol>
//
//!todo: Should there be a default constructor?
//
class ClassInfo
{
public:

    /* typedef's */
    typedef Base* (*ReadPointer)( Input& );

    /* Constructors */
    //: Default Constructor.
    ClassInfo();
    //: Copy constructor.
    ClassInfo( const ClassInfo& info );
    ClassInfo( const SH& sh, ReadPointer rp );
    
    /* Operator Overloads */

    //-----------------------------------------------------------------------------
    //
    //: Assignment Operator.
    //
    //!param: const ClassInfo& info - Object to be assigned.
    //
    //!return: const ClassInfo& -- This object.
    //
    //!exc: None.
    //
    const ClassInfo& operator=( const ClassInfo& info );
    
    /* Accessors */
    const SH& getSH() const;
    ClassInfo::ReadPointer getReadFunction() const;
    
private:

    SH mSH;
    ReadPointer mRead;
};


//-----------------------------------------------------------------------------
//
//: Get the Structure Header.
//
//!return const SH& -- Get structure header.
//
//!exc: None.
//   
inline const SH& ClassInfo::getSH() const
{
    return mSH;
}


//: Get the pointer to the read function.
//
//!exc: None.
//      
inline ClassInfo::ReadPointer
ClassInfo::getReadFunction() const
{
    return mRead;
}


} // namespace - Version_3_4_5

} // namespace - FrameCPP


#endif // FrameCppClassInfoHH
