#include "framecpp/config.h"

#include <memory>

#include "daqreader.hh"

using namespace FrameCPP::Version_3_4_5;

   
//: Destructor
//
//!exc: None.
//   
DaqReader::~DaqReader()
{}
   
   
   
//-----------------------------------------------------------------------------
//
//: Read a reference DaqFrame and build an update map along the way.
//
//!return: Frame*
//
//!exc: std::bad_alloc - Memory allocation failed.
//!exc: read_failure - Read failure.
//!exc: cannot_update   
//
DaqFrame* DaqReader::readFrame()
{
  // Read the frame in fully and construct the map in the frame
  std::auto_ptr<FrameCPP::Version_3_4_5::Frame>
    frame( FrameReaderBase::readFrame() );
  std::auto_ptr<FrameCPP::Version_3_4_5::DaqFrame>
    daq_frame( new FrameCPP::Version_3_4_5::DaqFrame( *(frame.get( )) ) );

  // FIXME: avoid map copying here, use the maps in the newly allocated frame
  // instead, reference them inside the frame.

  daq_frame->setDaqMaps(this->instanceOffsetMap, this->classInstanceMultimap);
  daq_frame->setFileHeader(this->mFileHeader);
  return daq_frame.release( );
}

//: Overrides object header reader in the superclass.
//
//!exc: read_failure
//   
FrameCPP::Version_3_4_5::ObjectHeader DaqReader::readNextObjectHeader()
{
  FrameCPP::Version_3_4_5::ObjectHeader hdr =
    FrameReaderBase::readNextObjectHeader();
  INT_2U instance = hdr.getInstance();
  INT_2U classId =  hdr.getFileClassId();
  INT_2U localClassId =  hdr.getLocalClassId();

  if (classId != ID_SH && classId != ID_SE)
    {
      unsigned long cur_pos = mStream->tellg();
      classInstanceMultimap.insert (std::pair<INT_4U, INT_4U> (localClassId, instance));
      adc_map_info_struct ami = { cur_pos,
				  (localClassId == ID_ADCDATA? adcDataVectorPtr(cur_pos): 0),
				  false };
      instanceOffsetMap.insert (std::pair<INT_4U, adc_map_info_struct>
				(localClassId << 16 | instance, ami ));
      mStream->seekg( cur_pos );
    }

  //  cout << classId << '\t' << instance  << '\t' << mStream->tellg () << endl;

  return hdr;
}
