#ifndef FrameAppendWriterTOCHH
#define FrameAppendWriterTOCHH

#include "framewritertoc.hh"
#include "tocreader.hh"
#include "output.hh"


namespace FrameCPP {

namespace Version_3_4_5 {

//-----------------------------------------------------------------------------
//
//: FrameAppendWriterTOC
//
// Allows appending new frames to existing frame file, which has a TOC.
//
class FrameAppendWriterTOC : public TOCReader, public FrameWriterTOC
{
public:

  enum ErrorCode
  {
    BADSTREAM = TOCReader::BADSTREAM,
    SKIPFAILED = TOCReader::SKIPFAILED,
    ENDOFFILE = TOCReader::ENDOFFILE,
    NOTOC = TOCReader::NOTOC,
    BADTOC = TOCReader::BADTOC,
    BADVERSION
  };


  //: Constructor
  //
  //!exc: write_failure - Write failed.   
  //!exc: read_failure - Read failed.
  //         
  FrameAppendWriterTOC( std::iostream& out );

  //: Destructor
  //
  //!exc: write_failure - Write failed.      
  virtual ~FrameAppendWriterTOC();

  //: Append frame
  //
  //!exc: write_failure - Write failed.   
  //!exc: read_failure - Read failed.
  //            
  void appendFrame( const Frame& frame );

  //: Append frame (overrides Output::writeFrame())
  //
  //!exc: write_failure - Write failed.   
  //            
  void writeFrame( const Frame& frame );
};
 
} // namespace - Version_3_4_5

} // namespace - FrameCPP


#endif // FrameAppendWriterTOCHH
