#ifndef FrameCppFrameReaderHH
#define FrameCppFrameReaderHH


#include "framereaderbase.hh"
#include "time.hh"
#include "location.hh"


namespace FrameCPP {

namespace Version_3_4_5 {

class Time;
class Location;


//-----------------------------------------------------------------------------
//
//: FrameReader
//
// This class is used to hide all of the functions which are not needed by a
// user just reading a frame.  Most of the functionality of FrameReaderBase
// is used by the frame objects themselves.  This functionality should not be
// used by the user.
//
class FrameReader : private FrameReaderBase, public VFrameReader
{
public:

    enum ErrorCode
    {
        BADSTREAM = FrameReaderBase::BADSTREAM,
        SKIPFAILED = FrameReaderBase::SKIPFAILED,
        ENDOFFILE = FrameReaderBase::ENDOFFILE
    };
    
    //: Constructor
    //!exc: read_failure - Read failed.
    FrameReader( std::istream& in, bool autoReset = false );

    //: Destructor
    //!exc: None.   
    virtual ~FrameReader();

    FrameReaderBase::getError;
    FrameReaderBase::errorBegin;
    FrameReaderBase::errorEnd;
    FrameReaderBase::errorQuantity;
    FrameReaderBase::errorFirst;
    FrameReaderBase::errorLast;
    FrameReaderBase::clearErrors;
    FrameReaderBase::setErrorWatch;
    FrameReaderBase::getNBytes;
    FrameReaderBase::getFrameNumber;
    
    //!exc: std::bad_alloc - Memory allocation failed.
    //!exc: read_failure - Read failed.
    Frame* readFrame();

    // Accessors
    FrameReaderBase::getFrames;
    FrameReaderBase::isAutoReset;

    FrameReaderBase::setAutoReset;

    FrameReaderBase::reset;

    //!exc: None.      
    bool isEndOfFile() const;
};


//: Constructor
//
//!exc: read_failure - Read failed.   
//   
inline FrameReader::FrameReader( std::istream& in, bool autoReset )
        : FrameReaderBase( in, autoReset )
{
}



//-----------------------------------------------------------------------------
//
//: See if FrEndOfFile structure was read, ie. the end of frame file reached
//
//!return: bool - True if FrEndOfFile was read.
//
//!exc: None 
//     
inline bool FrameReader::isEndOfFile( ) const
{
  return mEndOfFile;
}

//-----------------------------------------------------------------------------
//
//: Read a Frame
//
//!return: Frame* - Newly allocated.
//
//!exc: std::bad_alloc - Out of memory.
//!exc: read_failure - Read failed.
//
inline Frame* FrameReader::readFrame()
{
  Frame *frame = FrameReaderBase::readFrame();

  // The following steps are here because they are not needed 
  // in the TOCReader. So they were transferred over here
  // from FrameReaderBase::readFrame()
  //

  // optional table of contents
  if ( readNextObjectHeader().getLocalClassId() == ID_TOC )
    {
      // Do TOC pseudo read (skip actually)
      dictionary->readElement( *this, ID_TOC );
    }
  
  ObjectHeader oh = readNextObjectHeader();
  if ( oh.getLocalClassId() == ID_ENDOFFILE )
    {
      readEndOfFile();
    }

  return frame;
}

 
} // namespace - Version_3_4_5

} // namespace - FrameCPP


#endif // FrameCppFrameReaderHH
