#ifndef FrameCPPFrameReadPlanHH
#define FrameCPPFrameReadPlanHH

#include "frame.hh"
#include "tocreader.hh"
#include "toc.hh"

namespace FrameCPP {

namespace Version_3_4_5 {

//-----------------------------------------------------------------------------
//
//: Frame Reading Plan
//
class FrameReadPlan : public TOCReader
{
public:

  enum ErrorCode
  {
    BADSTREAM = FrameReaderBase::BADSTREAM,
    SKIPFAILED = FrameReaderBase::SKIPFAILED,
    ENDOFFILE = FrameReaderBase::ENDOFFILE,
    NOTOC,
    BADTOC
  };

  /* Error Methods */
  Input::getError;
  Input::errorBegin;
  Input::errorEnd;
  Input::errorQuantity;
  Input::errorFirst;
  Input::errorLast;
  Input::clearErrors;
  Input::setErrorWatch;

  //: Constructor
  //   
  //!exc: std::bad_alloc - Out of memory.
  //!exc: read_failure - Read failed.
  //   
  FrameReadPlan( std::istream& in );

  //: Destructor
  //
  //!exc: None.
  ~FrameReadPlan();

  //: Read frame data
  //   
  //!exc: read_failure - Read failed.
  //!exc: cannot_update    
  //      
  Frame& readFrame( std::istream& in, INT_4U frameNumber );

  //: Cat frame data
  //   
  //!exc: cannot_cat - 
  //!exc: read_failure - Read failed.
  //      
  Frame& catFrame( std::istream& in, INT_4U frameNumber, bool newFrame );
  
  //: Include ADCs in the frame
  //   
  //!exc: std::bad_alloc - Out of memory.
  //!exc: read_failure - Read failed.
  //!exc: not_found_error - Data is not found.   
  //      
  void daqTriggerADC( const std::vector<std::string> &adcNames );
 
private:
  //: Called when next object is read by FrameReaderBase::readBase()
  //!exc: std::bad_alloc - Out of memory.
  void readBaseHook( Base *base, INT_2U classId );

  //: Skeleton frame object
  Frame frame;

  //: Skeleton raw data object
  FrameCPP::Version_3_4_5::RawData raw_data;

  //: Adc structure offsets.
  // Internal vector is over the frames.
  // Outer vector is over the triggered adc names.
  std::vector< std::vector< INT_8U > > adcDataOffsetVector;

  inline int daqNameLength ( INT_8U );

#if 0
  inline INT_4S daqFrameRun ( INT_8U );
  inline INT_4U daqFrameNumber ( INT_8U );
  inline INT_4U daqFrameGTimeS ( INT_8U );
  inline INT_4U daqFrameGTimeN ( INT_8U );
  inline INT_2U daqFrameULeapS ( INT_8U );
  inline INT_4S daqFrameLocalTime ( INT_8U );
#endif


}; // class FrameReadPlan

} // namespace - Version_3_4_5

} // namespace - FrameCPP

#endif // FrameCPPFrameReadPlanHH
