#ifndef FrameWriterTOCHH
#define FrameWriterTOCHH


#include "toc.hh"
#include "output.hh"


namespace FrameCPP {

namespace Version_3_4_5 {

class Base;
class ObjectInfo;
class ContainerBase;


//-----------------------------------------------------------------------------
//
//: FrameWriterTOC
//
class FrameWriterTOC : public virtual TOC, public Output
{
public:

  //: Constructs an object and writes the fileheader.
  //!exc: write_failure 
  FrameWriterTOC( std::ostream& out );

  //: Destructor
  //!exc: write_failure    
  virtual ~FrameWriterTOC();

  /* I/O overload */
  //-----------------------------------------------------------------------------
  //
  //: Begins a record.
  //
  // This writes out the length, class and instance id of the
  // object and also initializes the ObjectInfo structure.
  //
  // Overloaded from Output class. TOC hosekeeping code.
  //
  //!exc: write_failure    
  //
  virtual void startRecord( INT_2U classId, INT_4U length, const FrameCPP::Version_3_4_5::Base* b );
 
  //: Write the TOC and close the file
  void close( unsigned long dummy = 0 );

protected:
  //: File was already closed.
  Output::mClosed;

private:
  //: Offset to the start of TOC in file.
  unsigned long tocSize;
  //: RawData was written out.
  bool raw_seen;
};
 
} // namespace - Version_3_4_5

} // namespace - FrameCPP


#endif // FrameWriterTOCHH
