#include "framecpp/config.h"

#include "location.hh"
#include "input.hh"
#include "output.hh"

using namespace FrameCPP::Version_3_4_5;


//-----------------------------------------------------------------------------
// Operator Overloads
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//
//: Assignment Operator.
//
//!param: const Location& loc - Obejct to be assigned.
//
//!return: const Location& - This object.
//
//!exc: None.
//   
const Location& Location::operator=( const Location& loc )
{
    if ( this != &loc )
    {
        mLatD = loc.mLatD;
        mLatM = loc.mLatM;
        mLatS = loc.mLatS;
        mLonD = loc.mLonD;
        mLonM = loc.mLonM;
        mLonS = loc.mLonS;
    }

    return *this;
}


//-----------------------------------------------------------------------------
//
//: Equal comparison.
//
// Determines whether two Location objects are equal.
//
//!param: const Location& loc - The object to compare with.
//
//!return: bool - true if the objects are equal.
//
//!kw: const
//
//!exc: None.
//      
bool Location::operator==( const Location& loc ) const
{
    return ( ( mLatD == loc.mLatD ) && ( mLatM == loc.mLatM )
             && ( mLatS == loc.mLatS ) && ( mLonD == loc.mLonD )
             && ( mLonM == loc.mLonM ) && ( mLonS == loc.mLonS ) );
}


//-----------------------------------------------------------------------------
//
//: Input extraction operator.
//
// This reads this object from an Input object.
//
//!param: Input& in - Input data stream object.
//!param: Location& loc - Data target.
//
//!return: Input& - Input data stream object.
//
//!exc: read_failure - Read failure occured.
//
Input& FrameCPP::Version_3_4_5::operator>>( Input& in, Location& loc )
{
    in >> loc.mLatD >> loc.mLatM >> loc.mLatS
       >> loc.mLonD >> loc.mLonM >> loc.mLonS;

    return in;
}


//-----------------------------------------------------------------------------
//
//: Output insertion operator.
//
// This writes this object to an Output object.
//
//!param: Output& out - Output data stream object.
//!param: const Location& loc - Data source.
//
//!return: Output& - Output data stream object.
//
//!exc: write_failure - Write failure occured.
//
Output& FrameCPP::Version_3_4_5::operator<<( Output& out, const Location& loc )
{
    out << loc.getLatD() << loc.getLatM() << loc.getLatS()
       << loc.getLonD() << loc.getLonM() << loc.getLonS();
    return out;
}
