#include "framecpp/config.h"

#include "toc.hh"
#include "input.hh"
#include "output.hh"

#include <vector>

using FrameCPP::Version_3_4_5::TOC;
using FrameCPP::Version_3_4_5::Time;

using namespace std;


//-----------------------------------------------------------------------------
// Constructors
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//
//: Default Constructor.
//
//!exc: None.      
TOC::TOC() 
        : Base(),
	  ULeapS( 0 ), dataQuality(),
	  GTime(), dt(), runs(), frame(),
	  position(),
	  positionADC(), positionSer(), positionTable(), positionMsg(),

	  SHid(), SHname(),

	  detectorNamePositionMap(),
	  statNamePositionMap(),
	  adcNamePositionMap(),
	  procNamePositionMap(),
	  simNamePositionMap(),
	  serNamePositionMap(),
	  sumNamePositionMap(),
	  eventNamePositionMap(),
	  simEventNamePositionMap()
{
}


//-----------------------------------------------------------------------------
//
//: Copy Constructor.
//
// This performs a deep copy of the object.
//
//!param: const TOC& toc - Object to be copied.
//
//!exc: std::bad_alloc - Memory allocation failed.
//
TOC::TOC( const TOC& toc ) 
  : Base( toc ),

    /* frame header */
    ULeapS( toc.ULeapS ),
    dataQuality( toc.dataQuality ),
    GTime( toc.GTime ),
    dt( toc.dt ), runs( toc.runs ),
    frame( toc.frame ),
    position( toc.position ),
    positionADC( toc.positionADC ), positionSer( toc.positionSer),
    positionTable( toc.positionTable ), positionMsg( toc.positionMsg ),
  
    /* Structure Header */
    SHid( toc.SHid ), SHname( toc.SHname ),

    /* Detector */
    detectorNamePositionMap( toc.detectorNamePositionMap ),

    /* Static data */
    statNamePositionMap( toc.statNamePositionMap ),

    /* ADC data */
    adcNamePositionMap( toc.adcNamePositionMap ),

    /* Proc data */
    procNamePositionMap( toc.procNamePositionMap ),

    /* Sim data */
    simNamePositionMap( toc.simNamePositionMap ),

    /* Ser data */
    serNamePositionMap( toc.serNamePositionMap ),

    /* Sum data */
    sumNamePositionMap( toc.sumNamePositionMap ),

    /* Event data */
    eventNamePositionMap( toc.eventNamePositionMap ),

    /* SimEvt data */
    simEventNamePositionMap( toc.simEventNamePositionMap )

{
}

   
//-----------------------------------------------------------------------------
//
//: Destructor.
//
// This method currently does nothing.
//
TOC::~TOC()
{
}
   
   
//-----------------------------------------------------------------------------
// Operator Overloads
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//
//: Assignment Operator.
//
//!param: const TOC& toc - Object to be assigned.
//
//!return: const TOC& - This TOC object.
//
//!exc: std::bad_alloc - Memory allocation failed.
//
const TOC& TOC::operator=( const TOC& toc )
{
  if ( this != &toc )
  {
    /* frame header */
    ULeapS = toc.ULeapS;
    dataQuality = toc.dataQuality;
    GTime = toc.GTime;
    dt = toc.dt; runs = toc.runs;
    frame = toc.frame;
    position = toc.position;
    positionADC = toc.positionADC;
    positionSer = toc.positionSer;
    positionTable = toc.positionTable;
    positionMsg = toc.positionMsg;

    /* Structure Header */
    SHid = toc.SHid; SHname = toc.SHname;

    /* Detector */
    detectorNamePositionMap = toc.detectorNamePositionMap;

    /* Static position */
    statNamePositionMap = toc.statNamePositionMap;

    /* ADC data */
    adcNamePositionMap = toc.adcNamePositionMap;

    /* Proc data */
    procNamePositionMap = toc.procNamePositionMap;    

    /* Sim data */
    simNamePositionMap = toc.simNamePositionMap;    

    /* Ser data */
    serNamePositionMap = toc.serNamePositionMap;    

    /* Summary data */
    sumNamePositionMap = toc.sumNamePositionMap;    

    /* Event data */
    eventNamePositionMap = toc.eventNamePositionMap;    

    /* SimEvt data */
    simEventNamePositionMap = toc.simEventNamePositionMap;    

  }
  return *this;
}



//-----------------------------------------------------------------------------
// Required Methods
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//
//: Constructs the Structure Header for this class.
//
//!return: SH - TOC structure header.
//
//!exc: std::bad_alloc - Memory allocation failed.
//
FrameCPP::Version_3_4_5::SH TOC::getSH()
{
  SH sh( "FrTOC", ID_TOC, "Table of Contents" );

  // Frame header
  sh.appendSE( SE( "ULeapS", "INT_2S", "" ) );
  sh.appendSE( SE( "nFrame", "INT_4U", "" ) );
  sh.appendSE( SE( "dataQuality", "INT_4U *", "" ) );
  sh.appendSE( SE( "GTimeS", "INT_4U *", "" ) );
  sh.appendSE( SE( "GTimeN", "INT_4U *", "" ) );
  sh.appendSE( SE( "dt", "REAL_8 *", "" ) );
  sh.appendSE( SE( "runs", "INT_4S *", "" ) );
  sh.appendSE( SE( "frame", "INT_4U *", "" ) );
  sh.appendSE( SE( "positionH", "INT_8U *", "" ) );
  sh.appendSE( SE( "nFirstADC", "INT_8U *", "" ) );
  sh.appendSE( SE( "nFirstSer", "INT_8U *", "" ) );
  sh.appendSE( SE( "nFirstTable", "INT_8U *", "" ) );
  sh.appendSE( SE( "nFirstMsg", "INT_8U *", "" ) );

  // Structure header
  sh.appendSE( SE( "nSH", "INT_4U", "" ) );
  sh.appendSE( SE( "SHid", "INT_2U *", "" ) );
  sh.appendSE( SE( "SHname", "STRING *", "" ) );

  // Detector
  sh.appendSE( SE( "nDetector", "INT_4U", "" ) );
  sh.appendSE( SE( "nameDetector", "STRING *", "" ) );
  sh.appendSE( SE( "positionDetector", "INT_8U *", "" ) );

  // Stat data
  sh.appendSE( SE( "nStatType", "INT_4U", "" ) );
  //
  // For each FrStatData unique name:
  //
  sh.appendSE( SE( "nameStat", "STRING", "" ) );
  sh.appendSE( SE( "detector", "STRING", "" ) );
  sh.appendSE( SE( "nStatInstance", "INT_4U", "" ) );
  //
  // For each stat name instance:
  //
  sh.appendSE( SE( "tStart", "INT_4U *", "" ) );
  sh.appendSE( SE( "tEnd", "INT_4U *", "" ) );
  sh.appendSE( SE( "version", "INT_4U *", "" ) );
  sh.appendSE( SE( "positionStat", "INT_8U *", "" ) );
  
  // Adc data
  sh.appendSE( SE( "nADC", "INT_4U", "" ) );
  sh.appendSE( SE( "name", "STRING *", "" ) );
  sh.appendSE( SE( "channelID", "INT_4U *", "" ) );
  sh.appendSE( SE( "groupID", "INT_4U *", "" ) );
  sh.appendSE( SE( "positionADC", "INT_8U *", "" ) ); // 2D

  // Proc data
  sh.appendSE( SE( "nProc", "INT_4U", "" ) );
  sh.appendSE( SE( "nameProc", "STRING *", "" ) );
  sh.appendSE( SE( "positionProc", "INT_8U *", "" ) ); // 2D

  // Sim data
  sh.appendSE( SE( "nSim", "INT_4U", "" ) );
  sh.appendSE( SE( "nameSim", "STRING *", "" ) );
  sh.appendSE( SE( "positionSim", "INT_8U *", "" ) ); // 2D

  // Serial data
  sh.appendSE( SE( "nSer", "INT_4U", "" ) );
  sh.appendSE( SE( "nameSer", "STRING *", "" ) );
  sh.appendSE( SE( "positionSer", "INT_8U *", "" ) ); // 2D

  // Summary data
  sh.appendSE( SE( "nSummary", "INT_4U", "" ) );
  sh.appendSE( SE( "nameSum", "STRING *", "" ) );
  sh.appendSE( SE( "positionSum", "INT_8U *", "" ) ); // 2D

  // Event data
  sh.appendSE( SE( "nEventType", "INT_4U", "" ) );
  sh.appendSE( SE( "nameEvent", "STRING *", "" ) );
  sh.appendSE( SE( "nEvent", "INT_4U *", "" ) );
  sh.appendSE( SE( "GTimeSEvent", "INT_4U *", "" ) );
  sh.appendSE( SE( "GTimeNEvent", "INT_4U *", "" ) );
  sh.appendSE( SE( "amplitudeEvent", "REAL_4 *", "" ) );
  sh.appendSE( SE( "positionEvent", "INT_8U *", "" ) );

  // Sim Event data 
  sh.appendSE( SE( "nSimEvent", "INT_4U", "" ) );
  sh.appendSE( SE( "nameSimEvent", "STRING *", "" ) );
  sh.appendSE( SE( "nSimEvent", "INT_4U *", "" ) );
  sh.appendSE( SE( "GTimeSSim", "INT_4U *", "" ) );
  sh.appendSE( SE( "GTimeNSim", "INT_4U *", "" ) );
  sh.appendSE( SE( "amplitudeSimEvent", "REAL_4 *", "" ) );
  sh.appendSE( SE( "positionSimEvt", "INT_8U *", "" ) );

  return sh;
}



//-----------------------------------------------------------------------------
//
//: Constructs frame data version 4 Structure Header for this class
//
//!return: SH - TOC structure header.
//
//!exc: std::bad_alloc - Memory allocation failed.
//
FrameCPP::Version_3_4_5::SH TOC::getSH4()
{
  SH sh( "FrTOC", ID_TOC, "Table of Contents" );

  // Frame header
  sh.appendSE( SE( "ULeapS", "INT_2S", "" ) );
  sh.appendSE( SE( "localTime", "INT_4S", "" ) );
  sh.appendSE( SE( "nFrame", "INT_4U", "" ) );
  sh.appendSE( SE( "GTimeS", "INT_4U *", "" ) );
  sh.appendSE( SE( "GTimeN", "INT_4U *", "" ) );
  sh.appendSE( SE( "dt", "REAL_8 *", "" ) );
  sh.appendSE( SE( "runs", "INT_4S *", "" ) );
  sh.appendSE( SE( "frame", "INT_4U *", "" ) );
  sh.appendSE( SE( "positionH", "INT_8U *", "" ) );
  sh.appendSE( SE( "nFirstADC", "INT_8U *", "" ) );
  sh.appendSE( SE( "nFirstSer", "INT_8U *", "" ) );
  sh.appendSE( SE( "nFirstTable", "INT_8U *", "" ) );
  sh.appendSE( SE( "nFirstMsg", "INT_8U *", "" ) );

  // Structure header
  sh.appendSE( SE( "nSH", "INT_4U", "" ) );
  sh.appendSE( SE( "SHid", "INT_2U *", "" ) );
  sh.appendSE( SE( "SHname", "STRING *", "" ) );

  // Stat data
  sh.appendSE( SE( "nStatType", "INT_4U", "" ) );
  //
  // For each FrStatData unique name:
  //
  sh.appendSE( SE( "nameStat", "STRING", "" ) );
  sh.appendSE( SE( "detector", "STRING", "" ) );
  sh.appendSE( SE( "nStatInstance", "INT_4U", "" ) );
  //
  // For each stat name instance:
  //
  sh.appendSE( SE( "tStart", "INT_4U *", "" ) );
  sh.appendSE( SE( "tEnd", "INT_4U *", "" ) );
  sh.appendSE( SE( "version", "INT_4U *", "" ) );
  sh.appendSE( SE( "positionStat", "INT_8U *", "" ) );
  
  // Adc data
  sh.appendSE( SE( "nADC", "INT_4U", "" ) );
  sh.appendSE( SE( "name", "STRING *", "" ) );
  sh.appendSE( SE( "channelID", "INT_4U *", "" ) );
  sh.appendSE( SE( "groupID", "INT_4U *", "" ) );
  sh.appendSE( SE( "positionADC", "INT_8U *", "" ) ); // 2D

  // Proc data
  sh.appendSE( SE( "nProc", "INT_4U", "" ) );
  sh.appendSE( SE( "nameProc", "STRING *", "" ) );
  sh.appendSE( SE( "positionProc", "INT_8U *", "" ) ); // 2D

  // Sim data
  sh.appendSE( SE( "nSim", "INT_4U", "" ) );
  sh.appendSE( SE( "nameSim", "STRING *", "" ) );
  sh.appendSE( SE( "positionSim", "INT_8U *", "" ) ); // 2D

  // Serial data
  sh.appendSE( SE( "nSer", "INT_4U", "" ) );
  sh.appendSE( SE( "nameSer", "STRING *", "" ) );
  sh.appendSE( SE( "positionSer", "INT_8U *", "" ) ); // 2D

  // Summary data
  sh.appendSE( SE( "nSummary", "INT_4U", "" ) );
  sh.appendSE( SE( "nameSum", "STRING *", "" ) );
  sh.appendSE( SE( "positionSum", "INT_8U *", "" ) ); // 2D

  // Trigger data
  sh.appendSE( SE( "nTrig", "INT_4U", "" ) );
  sh.appendSE( SE( "nameTrig", "STRING *", "" ) );
  sh.appendSE( SE( "GTimeSTrig", "INT_4U *", "" ) );
  sh.appendSE( SE( "GTimeNTrig", "INT_4U *", "" ) );
  sh.appendSE( SE( "positionTrig", "INT_8U *", "" ) ); // 1D

  // Sim Event data 
  sh.appendSE( SE( "nSimEvt", "INT_4U", "" ) );
  sh.appendSE( SE( "nameSimEvt", "STRING *", "" ) );
  sh.appendSE( SE( "GTimeSSim", "INT_4U *", "" ) );
  sh.appendSE( SE( "GTimeNSim", "INT_4U *", "" ) );
  sh.appendSE( SE( "positionSimEvt", "INT_8U *", "" ) ); // 1D

  return sh;
}


//-----------------------------------------------------------------------------
//
//: Skips TOC structure in an Input object.
//
// This method skips over the TOC. It is not read here.
//
//!param: Input& in - Input data stream object.
//
//!return: TOC* - Always return zero.
//
//!exc: std::bad_alloc - Memory allocation failed.
//!exc: read_failure - Failed to skip over the TOC.
//
TOC* TOC::read( Input& in )
{
  in.skipCurrentObject();
  return 0;
}


//-----------------------------------------------------------------------------
//
//: Reads an TOC object from an Input object.
//
// This method is used to read the table of contents.
//
//!param: Input& in - Input stream object.
//
//!return: 0 - absent or corrupt TOC; this - everything was fine.
//
//!exc: std::bad_alloc - Memory allocation failed.
//!exc: read_failure - Read failed.
//
TOC* TOC::reallyRead( Input& in )
{
  // Read frame header variables
  //
  INT_4U frameSize;
  in >> ULeapS;
  if ( in.getDataFormatVersion() == 4 ) {
    INT_4S localTime;
    in >> localTime;
  }
  in >> frameSize;

  if ( in.getDataFormatVersion() > 4 )
    {
      for ( unsigned long i = 0; i < frameSize; i++ )
	{
	  INT_4U dq;
	  in >> dq;
	  dataQuality.push_back( dq );
	}
    }

  // GTimeS
  for ( unsigned long i = 0; i < frameSize; i++ )
    {
      INT_4U times;
      in >> times;
      GTime.push_back( Time( times ) ); // puts new objects into the std::vector
    }

  // GTimeN
  for ( unsigned long i = 0; i < frameSize; i++ )
    {
      INT_4U timen;
      in >> timen;
      GTime[ i ].setNSec( timen ); // sets the actual nanosec time for objects in the vect
    }

  for ( unsigned long i = 0; i < frameSize; i++ )
    {
      REAL_8 delta_time;
      in >> delta_time;
      dt.push_back( delta_time );
    }

  for ( unsigned long i = 0; i < frameSize; i++ )
    {
      INT_4S r;
      in >> r;
      runs.push_back( r );
    }

  for ( unsigned long i = 0; i < frameSize; i++ )
    {
      INT_4U fn;
      in >> fn;
      frame.push_back( fn );
    }

  for ( unsigned long i = 0; i < frameSize; i++ )
    {
      INT_8U p;
      in >> p;
      position.push_back( p );
    }

  for ( unsigned long i = 0; i < frameSize; i++ )
    {
      INT_8U p;
      in >> p;
      positionADC.push_back( p );
    }

  for ( unsigned long i = 0; i < frameSize; i++ )
    {
      INT_8U p;
      in >> p;
      positionSer.push_back( p );
    }

  for ( unsigned long i = 0; i < frameSize; i++ )
    {
      INT_8U p;
      in >> p;
      positionTable.push_back( p );
    }

  for ( unsigned long i = 0; i < frameSize; i++ )
    {
      INT_8U p;
      in >> p;
      positionMsg.push_back( p );
    }

  // Read structure header variables
  //
  INT_4U shidSize;
  in >> shidSize;
  for ( unsigned long i = 0; i < shidSize; i++ )
    {
      INT_2U shid;
      in >> shid;
      SHid.push_back( shid );
    }
  for ( unsigned long i = 0; i < shidSize; i++ )
    {
      std::string name;
      in >> name;
      SHname.push_back( name );
    }

  // Read detector data
  //
  if ( in.getDataFormatVersion() > 4 ) {
    INT_4U nDetector;
    std::vector< std::string > name;
    in >> nDetector;

    for ( unsigned int i = 0; i < nDetector; i++ ) {
      std::string n;
      in >> n;
      name.push_back(n);
    }

    for ( unsigned int i = 0; i < nDetector; i++ ) {
      INT_8U pos;
      in >> pos;
      detectorNamePositionMap.insert(std::pair<std::string, INT_8U>(name[i], pos));
    }
  }

  // Read static data
  //
  INT_4U nStat;
  in >> nStat;
  
  for ( unsigned int i = 0; i < nStat; ++i )
    {
      std::string name, detector;
      INT_4U nVersions;
      in >> name >> detector >> nVersions;


      std::vector< INT_4U > tStart( nVersions );

      for( std::vector< INT_4U >::iterator iter = tStart.begin(),
              end_iter = tStart.end(); iter != end_iter; ++iter )
      {
 	 in >> ( *iter );
      }


      std::vector< INT_4U > tEnd( nVersions );
	  	  
      for( std::vector< INT_4U >::iterator iter = tEnd.begin(),
              end_iter = tEnd.end(); iter != end_iter; ++iter )
      {
	 in >> ( *iter );
      }


      std::vector< INT_4U > version( nVersions );

      for( std::vector< INT_4U >::iterator iter = version.begin(), 
	     end_iter = version.end(); iter != end_iter; ++iter )
      {
	 in >> ( *iter );
      }


      std::vector< INT_8U > pos( nVersions );

      for( std::vector< INT_8U >::iterator iter = pos.begin(),
             end_iter = pos.end(); iter != end_iter; ++iter )
      {
	 in >> ( *iter );
      }


      std::vector< std::pair< std::vector< INT_4U >, INT_8U > > v;

      for( unsigned int j = 0; j < nVersions; ++j )
      {
	  std::vector< INT_4U > vv;

	  vv.push_back( tStart[ j ] );
	  vv.push_back( tEnd[ j ] );
	  vv.push_back( version[ j ] );
	  v.push_back( std::pair< std::vector< INT_4U >, INT_8U>( vv, pos[ j ] ));
      }

      statNamePositionMap.insert( SNP_value_type( name, SNP_value( detector, v ) ) );
    }


  // Read ADC data
  //
  INT_4U nADC;
  in >> nADC;

  if ( nADC > 0 )
  {
    std::vector< std::string > adcName( nADC );
    for( std::vector< std::string >::iterator iter = adcName.begin(),
            end_iter = adcName.end(); iter != end_iter; ++iter )
    {
       in >> ( *iter );
    }


    std::vector< INT_4U > adcChannelNumber( nADC );
    for( std::vector< INT_4U >::iterator iter = adcChannelNumber.begin(),
            end_iter = adcChannelNumber.end(); iter != end_iter; ++iter )
    {
       in >> ( *iter );
    }


    std::vector< INT_4U > adcChannelGroup( nADC );
    for( std::vector< INT_4U >::iterator iter = adcChannelGroup.begin(),
            end_iter = adcChannelGroup.end(); iter != end_iter; ++iter )
    {
       in >> ( *iter );
    }


    // Next thing to read in is the 2D array of FrAdcData positions vs. Frame number
    //
    std::vector< std::vector< INT_8U > > adcPositions( nADC );

    for( std::vector< std::vector< INT_8U > >::iterator iter = adcPositions.begin(),
	   end_iter = adcPositions.end(); iter != end_iter; ++iter )
    {
       for( INT_4U j = 0; j < frameSize; ++j )
       {
          INT_8U p;
	  in >> p;
	  iter->push_back( p );
       }
    }  


#define DONT_RESIZE_HASH_MAP

#ifdef DONT_RESIZE_HASH_MAP
    // Create ADC map
    //
    for (unsigned int i = 0; i < nADC; ++i )
    {
       vector< INT_4U > ids;

       ids.push_back( adcChannelNumber[ i ] );
       ids.push_back( adcChannelGroup[ i ] );

       adcNamePositionMap.insert( make_pair( adcName[ i ], 
                                             make_pair( adcPositions[ i ], ids ) ) );
    }
#else
    // Create ADC map
    //
    adcNamePositionMap.resize( nADC );
    for (unsigned int i = 0; i < nADC; ++i )
      {
	std::pair< std::vector<INT_8U >, std::vector< INT_4U > >&
	  p = adcNamePositionMap[ adcName[ i ] ];
	p.first = adcPositions[ i ];
	p.second.push_back( adcChannelNumber[ i ] );
	p.second.push_back( adcChannelGroup[ i ] );
      }
#endif
  }

  
  // Read Proc data
  //
  INT_4U nProc;
  in >> nProc;

  if ( nProc > 0 )
  {
    std::vector< std::string > procName( nProc );

    for( std::vector< std::string >::iterator iter = procName.begin(),
	   end_iter = procName.end(); iter != end_iter; ++iter )
    {
       in >> ( *iter );
    }

    // Read in the 2D array of FrProcData positions vs. Frame number
    //
    std::vector< std::vector< INT_8U > > procPositions( nProc );
    for( std::vector< std::vector< INT_8U > >::iterator iter = procPositions.begin(),
	   end_iter = procPositions.end(); iter != end_iter; ++iter )
    {
       for( unsigned int j = 0; j < frameSize; ++j )
       {
	  INT_8U p;
	  in >> p;
	  iter->push_back( p );
       }
    }  
  
    // Create Proc map
    //
    for (unsigned int i = 0; i < nProc; i++ )
    {
       procNamePositionMap.insert( std::pair< std::string, std::vector<INT_8U> >( procName[ i ], procPositions[ i ] ));
    }    
  }

  // Read Sim data
  //
  INT_4U nSim;
  in >> nSim;
  if ( nSim > 0 )
  {
    vector< string > simName( nSim );

    for( vector< string >::iterator iter = simName.begin(),
	   end_iter = simName.end(); iter != end_iter; ++iter )
    {
       in >> ( *iter );
    }

    // Read in the 2D array of FrSimData positions vs. Frame number
    //
    vector< vector< INT_8U > > simPositions( nSim );
    for( vector< vector< INT_8U > >::iterator iter = simPositions.begin(),
	   end_iter = simPositions.end(); iter != end_iter; ++iter )
    {
       for( unsigned int j = 0; j < frameSize; ++j )
       {
	  INT_8U p;
	  in >> p;
	  iter->push_back( p );
       }
    }  
  
    // Create Sim map
    //
    for (unsigned int i = 0; i < nSim; i++ )
    {
       simNamePositionMap.insert( std::pair< std::string, std::vector<INT_8U> >( simName[ i ], simPositions[ i ] ));
    }    
  }

  // Read Serial data
  //
  INT_4U nSer;
  in >> nSer;
  if ( nSer > 0 )
  {
     vector< string > serName( nSer );

     for( vector< string >::iterator iter = serName.begin(), end_iter = serName.end();
          iter != end_iter; ++iter )
     {
        in >> ( *iter );
     }

    // Read in the 2D array of FrSerData positions vs. Frame number
    //
    vector< vector< INT_8U > > serPositions( nSer );
    for( vector< vector< INT_8U > >::iterator iter = serPositions.begin(),
	   end_iter = serPositions.end(); iter != end_iter; ++iter )
    {
       for( unsigned int j = 0; j < frameSize; ++j )
       {
	  INT_8U p;
	  in >> p;
	  iter->push_back( p );
       }
    }  
  
    // Create Ser map
    //
    for( unsigned int i = 0; i < nSer; ++i )
    {
       serNamePositionMap.insert( std::pair< std::string, std::vector<INT_8U> >( serName[ i ], serPositions[ i ] ));
    }    
  }


  // Read Summary data
  //
  INT_4U nSum;
  in >> nSum;
  if ( nSum > 0 )
  {
     vector< string > sumName( nSum );

     for( vector< string >::iterator iter = sumName.begin(),
	    end_iter = sumName.end(); iter != end_iter; ++iter )
     {
        in >> ( *iter );
     }

     // Read in the 2D array of FrSumData positions vs. Frame number
     //
     vector< vector< INT_8U > > sumPositions( nSum );
     for( vector< vector< INT_8U > >::iterator iter = sumPositions.begin(),
	    end_iter = sumPositions.end(); iter != end_iter; ++iter )
     {
	for( unsigned int j = 0; j < frameSize; ++j )
	{
	   INT_8U p;
	   in >> p;
	   iter->push_back( p );
	}
     }  
  
    // Create Sum map
    //
    for(unsigned int i = 0; i < nSum; i++ )
    {
       sumNamePositionMap.insert( std::pair< std::string, std::vector<INT_8U> >( sumName[ i ], sumPositions[ i ] ));
    }    
  }


  // Read Event data
  //
  INT_4U nEventType;
  in >> nEventType;

  if( nEventType > 0 )
  {
     vector< string > eventName( nEventType );

     if( in.getDataFormatVersion() == 4 )
     {
       for( vector< string >::iterator iter = eventName.begin(),
	      end_iter = eventName.end(); iter != end_iter; ++iter )
       {
	  in >> ( *iter );

	  EVENTNPI map_iter = eventNamePositionMap.find( *iter );
	  if ( map_iter == eventNamePositionMap.end() )
	  {
	     eventNamePositionMap.insert( std::pair< std::string, std::vector< TOC::eventAttr > >( *iter, std::vector< TOC::eventAttr >() ));
	  }
       }

       vector< TOC::eventAttr > eva( nEventType );
       for( vector< TOC::eventAttr >::iterator iter = eva.begin(),
	      end_iter = eva.end(); iter != end_iter; ++iter )
       {
	  INT_4U time;
	  in >> time;
	  iter->gtime.setSec( time );
       }

       for( vector< TOC::eventAttr >::iterator iter = eva.begin(),
	      end_iter = eva.end(); iter != end_iter; ++iter )
       {
	  INT_4U time;
	  in >> time;
	  iter->gtime.setNSec( time );
       }

       for( vector< TOC::eventAttr >::iterator iter = eva.begin(),
	      end_iter = eva.end(); iter != end_iter; ++iter )
       {
	  in >> ( iter->position );
 
	  // :TODO: it should be possible to read the amplitude from the structure in a file
	  iter->amplitude = .0;
       }


       vector< string >::const_iterator name_iter( eventName.begin() );

       for( vector< TOC::eventAttr >::iterator iter = eva.begin(),
	      end_iter = eva.end(); iter != end_iter; ++iter ) 
       {
 	  eventNamePositionMap[ *name_iter ].push_back( *iter );
          ++name_iter;
       }
     }
     else // Not version 4
     {
       for( vector< string >::iterator iter = eventName.begin(),
	      end_iter = eventName.end(); iter != end_iter; ++iter )
       {
	  in >> ( *iter );
	  eventNamePositionMap.insert(
             std::pair< std::string, std::vector< TOC::eventAttr > >( *iter, 
                                                                      vector< TOC::eventAttr >() ));
       }

       vector< INT_4U > nEvent( nEventType );
       for( vector< INT_4U >::iterator iter = nEvent.begin(),
               end_iter = nEvent.end(); iter != end_iter; ++iter )
       {
	  in >> ( *iter );
       }
      

       vector< INT_4U >::const_iterator event_iter( nEvent.begin() );

       for( vector< string >::iterator iter = eventName.begin(),
	      end_iter = eventName.end(); iter != end_iter;
            ++iter, ++event_iter )
       {
	  for( unsigned int j = 0; j < ( *event_iter ); ++j )
	  {
	     INT_4U time;
	     in >> time;
	     eventNamePositionMap[ *iter ].push_back( eventAttr( Time( time, 0 ), .0, 0 ));
	  }
       }


       event_iter = nEvent.begin();

       for( vector< string >::iterator iter = eventName.begin(),
	      end_iter = eventName.end(); iter != end_iter;
            ++iter, ++event_iter )      
       {
	  for( unsigned int j = 0; j < ( *event_iter ); ++j )
	  {
	     INT_4U time;
	     in >> time;
	     eventNamePositionMap[ *iter ][ j ].gtime.setNSec( time );
	  }
       }


       event_iter = nEvent.begin();

       for( vector< string >::iterator iter = eventName.begin(),
	      end_iter = eventName.end(); iter != end_iter;
            ++iter, ++event_iter )      
       {
	  for( unsigned int j = 0; j < ( *event_iter ); ++j )
	  {
	     REAL_4 a;
	     in >> a;
	     eventNamePositionMap[ *iter ][ j ].amplitude = a;
	  }
       }

      
       event_iter = nEvent.begin();

       for( vector< string >::iterator iter = eventName.begin(),
	      end_iter = eventName.end(); iter != end_iter;
            ++iter, ++event_iter )      
       {
	  for( unsigned int j = 0; j < ( *event_iter ); ++j )
	  {
	     INT_8U p;
	     in >> p;
	     eventNamePositionMap[ *iter ][ j ].position = p;
	  }
       }
    }
  }


  // Read SimEvt data
  //
  INT_4U nSimEventType;
  in >> nSimEventType;

  if ( nSimEventType > 0 )
  {
     vector< string > simEventName( nSimEventType );

     if ( in.getDataFormatVersion() == 4 )
     {
        for( vector< string >::iterator iter = simEventName.begin(),
	       end_iter = simEventName.end(); iter != end_iter; ++iter )
        {
	  in >> ( *iter );

	  SIMEVENTNPI map_iter = simEventNamePositionMap.find( *iter );
	  if( map_iter == simEventNamePositionMap.end() )
	  {
	     simEventNamePositionMap.insert( 
                pair< string, vector< TOC::eventAttr > >( *iter, vector< TOC::eventAttr >() ));
	  }
	}

        vector< TOC::eventAttr > eva( nSimEventType );
        for( vector< TOC::eventAttr >::iterator iter = eva.begin(),
		end_iter = eva.end(); iter != end_iter; ++iter )
	{
  	   INT_4U time;
	   in >> time;
	   iter->gtime.setSec( time );
        }

        for( vector< TOC::eventAttr >::iterator iter = eva.begin(),
		end_iter = eva.end(); iter != end_iter; ++iter )
	{
	   INT_4U time;
	   in >> time;
	   iter->gtime.setNSec( time );
	}

        for( vector< TOC::eventAttr >::iterator iter = eva.begin(),
		end_iter = eva.end(); iter != end_iter; ++iter )
	{
	   in >> ( iter->position );

	   // :TODO: it should be possible to read the amplitude from the structure in a file
	   iter->amplitude = .0;
        }

        vector< string >::const_iterator name_iter( simEventName.begin() );
        for( vector< TOC::eventAttr >::iterator iter = eva.begin(),
		end_iter = eva.end(); iter != end_iter; ++iter, ++name_iter )
	{
	   simEventNamePositionMap[ *name_iter ].push_back( *iter );
	}
     }
     else // not version 4
     {
        for( vector< string >::iterator iter = simEventName.begin(),
	       end_iter = simEventName.end(); iter != end_iter; ++iter )
	{
	   in >> ( *iter );
	   simEventNamePositionMap.insert( 
              pair< string, vector< TOC::eventAttr > >( *iter, 
                                                        vector< TOC::eventAttr >() ));
	}

        vector< INT_4U > nSimEvent( nSimEventType );
        for( vector< INT_4U >::iterator iter = nSimEvent.begin(), 
	       end_iter = nSimEvent.end(); iter != end_iter; ++iter )
	{
	   in >> ( *iter );
	}
      
        vector< string >::const_iterator name_iter( simEventName.begin() );
        for( vector< INT_4U >::iterator iter = nSimEvent.begin(), 
	       end_iter = nSimEvent.end(); iter != end_iter; ++iter, ++name_iter )
	{
     	   for( unsigned int j = 0; j < ( *iter ); ++j )
	   {
	      INT_4U time;
	      in >> time;
	      simEventNamePositionMap[ *name_iter ].push_back( eventAttr( Time( time, 0 ), .0, 0 ));
 	   }
	}
      

        name_iter = simEventName.begin();
        for( vector< INT_4U >::iterator iter = nSimEvent.begin(), 
	       end_iter = nSimEvent.end(); iter != end_iter; ++iter, ++name_iter )
	{
     	   for( unsigned int j = 0; j < ( *iter ); ++j )
	   {
	      INT_4U time;
	      in >> time;
	      simEventNamePositionMap[ *name_iter ][ j ].gtime.setNSec( time );
 	   }
	}

        name_iter = simEventName.begin();
        for( vector< INT_4U >::iterator iter = nSimEvent.begin(), 
	       end_iter = nSimEvent.end(); iter != end_iter; ++iter, ++name_iter )
	{
     	   for( unsigned int j = 0; j < ( *iter ); ++j )
	   {
	      REAL_4 a;
	      in >> a;
	      simEventNamePositionMap[ *name_iter ][ j ].amplitude = a;
 	   }
	}


        name_iter = simEventName.begin();
        for( vector< INT_4U >::iterator iter = nSimEvent.begin(), 
	       end_iter = nSimEvent.end(); iter != end_iter; ++iter, ++name_iter )
	{
     	   for( unsigned int j = 0; j < ( *iter ); ++j )
	   {
	      INT_8U p;
	      in >> p;
	      simEventNamePositionMap[ *name_iter ][ j ].position = p;
 	   }
	}
     }
  }

  return this;
}


//-----------------------------------------------------------------------------
//
//: Writes an TOC object to an Output object.
//
//!param: Output& out out - Output stream object.
//
//!exc: write_failure - Write failed.
//
void TOC::write( Output& out ) const
{
  // Calculate the length
  unsigned long toc_length = 8 // length, class, instance
    + sizeof(ULeapS) + sizeof(INT_4U)     // leap seconds, num of frames
    + frame.size() * (
		      4  // `dataQuality'
		      + 2 * 4   // std::vectors `GTimeS' and `GTimeN'
		      + 8     // `dt' std::vector
		      + 2 * 4 // `runs' and `frame' std::vectors
		      + 8 * 5     // position std::vectors
		      )
    + sizeof(INT_4U) + SHid.size() * sizeof(INT_2U) // SH data

    /* Detector */
    + sizeof(INT_4U) // nDetector

    /* Static data */
    + sizeof(INT_4U) // nStat

    /* Adc data */
    + sizeof(INT_4U) + adcNamePositionMap.size() * sizeof(INT_4U) * 2 // nADC, channelID and groupID
    + sizeof(INT_8U) * adcNamePositionMap.size() * frame.size() // positionADC

    /* Proc data */
    + sizeof(INT_4U) // nProc
    + sizeof(INT_8U) * procNamePositionMap.size() * frame.size() // positionProc

    /* Sim data */
    + sizeof(INT_4U) // nSim
    + sizeof(INT_8U) * simNamePositionMap.size() * frame.size() // positionSim

    /* Serial data */
    + sizeof(INT_4U) // nSer
    + sizeof(INT_8U) * serNamePositionMap.size() * frame.size() // positionSer

    /* Summary data */
    + sizeof(INT_4U) // num
    + sizeof(INT_8U) * sumNamePositionMap.size() * frame.size() // positionSum
    ;

  // Add the length of SH names std::vector
  //
  for ( unsigned long i = 0; i < SHname.size(); i++ )
    {
      toc_length += SHname[ i ].length() + 3;
    }

  // Add the lengths of all detector variables
  //
  for ( DNPI i = detectorNamePositionMap.begin(); i != detectorNamePositionMap.end(); i++ )
    {
      toc_length +=
	i->first.length() + 3 // nameDetector
	+ sizeof(INT_8U); // positionDetector
    }

  // Add the length of all static data variables
  //
  for ( SNPI i = statNamePositionMap.begin(); i != statNamePositionMap.end(); i++ )
    {
      toc_length +=
	i->first.length() + 3 // nameStat
	+ i->second.first.length() + 3 // detector
	+ sizeof(INT_4U) // nVersions
	+ i->second.second.size() * (3 * sizeof(INT_4U) + sizeof(INT_8U)); // tStart, tEnd, version and positionStat
    }

  // Add the length of ADC `name' variable std::vector;
  //
  for ( ANPI i = adcNamePositionMap.begin(); i != adcNamePositionMap.end(); i++ )
    {
      toc_length += i->first.length() + 3;
    }

  // Add the length of Proc `name' variable std::vector;
  //
  for ( PNPI i = procNamePositionMap.begin(); i != procNamePositionMap.end(); i++ )
    {
      toc_length += i->first.length() + 3;
    }

  // Add the length of Sim `name' variable std::vector;
  //
  for ( SIMNPI i = simNamePositionMap.begin(); i != simNamePositionMap.end(); i++ )
    {
      toc_length += i->first.length() + 3;
    }

  // Add the length of Serial `name' variable std::vector;
  //
  for ( SERNPI i = serNamePositionMap.begin(); i != serNamePositionMap.end(); i++ )
    {
      toc_length += i->first.length() + 3;
    }

  //
  // Add the length of Summary `name' variable std::vector;
  //
  for ( SUMNPI i = sumNamePositionMap.begin(); i != sumNamePositionMap.end(); i++ )
    {
      toc_length += i->first.length() + 3;
    }

  /* Event data */
  toc_length += sizeof(INT_4U); // nEventType
  for ( EVENTNPI i = eventNamePositionMap.begin(); i != eventNamePositionMap.end(); i++ )
    toc_length += i->first.length() + 3 + 4 + i->second.size() * 20;

  /* SimEvt data */
  toc_length += sizeof(INT_4U); // nSimEventType
  for ( EVENTNPI i = simEventNamePositionMap.begin(); i != simEventNamePositionMap.end(); i++ )
    toc_length += i->first.length() + 3 + 4 + i->second.size() * 20;

  if (SHid.rend() == find(SHid.rbegin(), SHid.rend(), getSH().getClass())) {
    // Add our name (FrToc) length, it will be there in SHname and SHid after out.startRecord() call
    // (only if it is not there already)
    toc_length += getSH().getName().length() + 3 + sizeof(INT_2U);
  }

  out.startRecord( ID_TOC, toc_length, this );
  out << ULeapS << (INT_4U) frame.size();

  for ( unsigned long i = 0; i < frame.size(); i++ )
    {
      out << dataQuality[ i ];
    }

  for ( unsigned long i = 0; i < frame.size(); i++ )
    {
      out << GTime[ i ].getSec();
    }

  for ( unsigned long i = 0; i < frame.size(); i++ )
    {
      out << GTime[ i ].getNSec();
    }

  for ( unsigned long i = 0; i < frame.size(); i++ )
    {
      out << dt[ i ];
    }

  for ( unsigned long i = 0; i < frame.size(); i++ )
    {
      out << runs[ i ];
    }

  for ( unsigned long i = 0; i < frame.size(); i++ )
    {
      out << frame[ i ];
    }

  for ( unsigned long i = 0; i < frame.size(); i++ )
    {
      out << position[ i ];
    }

  for ( unsigned long i = 0; i < frame.size(); i++ )
    {
      out << positionADC[ i ];
    }

  for ( unsigned long i = 0; i < frame.size(); i++ )
    {
      out << positionSer[ i ];
    }

  for ( unsigned long i = 0; i < frame.size(); i++ )
    {
      out << positionTable[ i ];
    }

  for ( unsigned long i = 0; i < frame.size(); i++ )
    {
      out << positionMsg[ i ];
    }

  out << (INT_4U) SHid.size();
  for ( unsigned long i = 0; i < SHid.size(); i++ )
    {
      out << SHid[ i ];
    }
  for ( unsigned long i = 0; i < SHname.size(); i++ )
    {
      out << SHname[ i ];
    }

  // Write detector
  out << (INT_4U) detectorNamePositionMap.size();
  for ( DNPI i = detectorNamePositionMap.begin(); i != detectorNamePositionMap.end(); i++ )
    {
      out << i->first;
    }

  for ( DNPI i = detectorNamePositionMap.begin(); i != detectorNamePositionMap.end(); i++ )
    {
      out << i->second;
    }

  // Write Stat data
  out << (INT_4U) statNamePositionMap.size();
  for ( SNPI i = statNamePositionMap.begin(); i != statNamePositionMap.end(); i++ )
    {
      out << i->first; // nameStat
      out << i->second.first; // detector
      out << (INT_4U) i->second.second.size(); // nVersions
      // Send tStart, tEnd, version
      for ( unsigned k = 0; k < 3; k++ )
	{
	  for ( unsigned j = 0; j < i->second.second.size(); j++ )
	    {
	      out << i->second.second[j].first[k];
	    }
	}
      // positionStat
      for ( unsigned j = 0; j < i->second.second.size(); j++ )
	out << i->second.second[j].second;
    }

  // Write ADC data
  out << (INT_4U) adcNamePositionMap.size();
  for ( ANPI i = adcNamePositionMap.begin(); i != adcNamePositionMap.end(); i++ )
    {
      out << i->first;
    }

  for ( ANPI i = adcNamePositionMap.begin(); i != adcNamePositionMap.end(); i++ )
    {
      out << i->second.second[ 0 ];
    }

  for ( ANPI i = adcNamePositionMap.begin(); i != adcNamePositionMap.end(); i++ )
    {
      out << i->second.second[ 1 ];
    }

  for ( ANPI i = adcNamePositionMap.begin(); i != adcNamePositionMap.end(); i++ )
    {
      for ( unsigned int j = 0 ; j < i->second.first.size(); j++ )
	{
	  out << i->second.first[ j ];
	}
    }

  // Write proc data
  out << (INT_4U) procNamePositionMap.size();
  for ( PNPI i = procNamePositionMap.begin(); i != procNamePositionMap.end(); i++ )
    {
      out << i->first;
    }

  for ( PNPI i = procNamePositionMap.begin(); i != procNamePositionMap.end(); i++ )
    {
      for ( unsigned int j = 0 ; j < i->second.size(); j++ )
	{
	  out << i->second[ j ];
	}
    }


  // Write Sim data
  out << (INT_4U) simNamePositionMap.size();
  for ( SIMNPI i = simNamePositionMap.begin(); i != simNamePositionMap.end(); i++ )
    {
      out << i->first;
    }

  for ( PNPI i = simNamePositionMap.begin(); i != simNamePositionMap.end(); i++ )
    {
      for ( unsigned int j = 0 ; j < i->second.size(); j++ )
	{
	  out << i->second[ j ];
	}
    }

  // Write serial data
  out << (INT_4U) serNamePositionMap.size();
  for ( SERNPI i = serNamePositionMap.begin(); i != serNamePositionMap.end(); i++ )
    {
      out << i->first;
    }

  for ( SERNPI i = serNamePositionMap.begin(); i != serNamePositionMap.end(); i++ )
    {
      for ( unsigned int j = 0 ; j < i->second.size(); j++ )
	{
	  out << i->second[ j ];
	}
    }

  // Write summary data
  out << (INT_4U) sumNamePositionMap.size();
  for ( SUMNPI i = sumNamePositionMap.begin(); i != sumNamePositionMap.end(); i++ )
    {
      out << i->first;
    }

  for ( SUMNPI i = sumNamePositionMap.begin(); i != sumNamePositionMap.end(); i++ )
    {
      for ( unsigned int j = 0 ; j < i->second.size(); j++ )
	{
	  out << i->second[ j ];
	}
    }


  // Write event data
  out << (INT_4U) eventNamePositionMap.size();
  for ( EVENTNPI i = eventNamePositionMap.begin(); i != eventNamePositionMap.end(); i++ )
    out << i->first;
  for ( EVENTNPI i = eventNamePositionMap.begin(); i != eventNamePositionMap.end(); i++ )
    out << (INT_4U)(i->second.size());
  for ( EVENTNPI i = eventNamePositionMap.begin(); i != eventNamePositionMap.end(); i++ )
    for ( unsigned j = 0; j < i->second.size(); j++ )
      out << (INT_4U)(i->second[ j ].gtime.getSec());
  for ( EVENTNPI i = eventNamePositionMap.begin(); i != eventNamePositionMap.end(); i++ )
    for ( unsigned j = 0; j < i->second.size(); j++ )
      out << i->second[ j ].gtime.getNSec();
  for ( EVENTNPI i = eventNamePositionMap.begin(); i != eventNamePositionMap.end(); i++ )
    for ( unsigned j = 0; j < i->second.size(); j++ )
      out << i->second[ j ].amplitude;
  for ( EVENTNPI i = eventNamePositionMap.begin(); i != eventNamePositionMap.end(); i++ )
    for ( unsigned j = 0; j < i->second.size(); j++ )
      out << i->second[ j ].position;

  // Write simEvent data
  out << (INT_4U) simEventNamePositionMap.size();
  for ( SIMEVENTNPI i = simEventNamePositionMap.begin(); i != simEventNamePositionMap.end(); i++ )
    out << i->first;
  for ( SIMEVENTNPI i = simEventNamePositionMap.begin(); i != simEventNamePositionMap.end(); i++ )
    out << (INT_4U)(i->second.size());
  for ( SIMEVENTNPI i = simEventNamePositionMap.begin(); i != simEventNamePositionMap.end(); i++ )
    for ( unsigned j = 0; j < i->second.size(); j++ )
      out << (INT_4U)(i->second[ j ].gtime.getSec());
  for ( SIMEVENTNPI i = simEventNamePositionMap.begin(); i != simEventNamePositionMap.end(); i++ )
    for ( unsigned j = 0; j < i->second.size(); j++ )
      out << i->second[ j ].gtime.getNSec();
  for ( SIMEVENTNPI i = simEventNamePositionMap.begin(); i != simEventNamePositionMap.end(); i++ )
    for ( unsigned j = 0; j < i->second.size(); j++ )
      out << i->second[ j ].amplitude;
  for ( SIMEVENTNPI i = simEventNamePositionMap.begin(); i != simEventNamePositionMap.end(); i++ )
    for ( unsigned j = 0; j < i->second.size(); j++ )
      out << i->second[ j ].position;


  out.endRecord( this );
}

   
//-----------------------------------------------------------------------------
//
//: Gets the class ID.
//
//!return: INT_2U -- TOC class ID.
//
//!exc: None.   
INT_2U TOC::getClassId() const
{
    return ID_TOC;
}
   
