#ifndef FrameCPP_VERSION_6_FrBase_HH
#define FrameCPP_VERSION_6_FrBase_HH

#include <iostream>
#include <stdexcept>

#include "general/types.hh"

#include "framecpp/Version6/PTR_STRUCT.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    class IFrameStream;
    class OFrameStream;
#if WORKING
    class Dictionary;
#endif /* WORKING */

    class FrBase
    {
    public:
      enum class_types_t
      {
	CLASS_NULL = 0,
	CLASS_FR_SH = 1,
	CLASS_FR_SE = 2,
	LAST_PRE_DEFINED = CLASS_FR_SE
      };

      class TooSmallException
	: public std::runtime_error
      {
      public:
	TooSmallException( const FrBase& Base, INT_4U Version );
      private:
	std::string format( const FrBase& Base, INT_4U Version );
      };

      FrBase( INT_2U Class );

      FrBase( const FrBase& Source );

      virtual ~FrBase( );

      virtual INT_8U Bytes( ) const;

      static INT_8U Bytes( INT_2U Version );

      virtual std::string ErrorInfo( ) const;

      INT_2U GetClassId( ) const;

      INT_4U GetInstance( ) const;

      INT_8U GetLength( ) const;

      PTR_STRUCT GetNext( ) const;

      bool IsInstanceSet( ) const;

      void SetInstance( INT_4U Instance ) const;

      void SetLength( ) const;

      void SetNext( const PTR_STRUCT& Next );

      INT_8U Skip( const IFrameStream& Stream ) const;

      void UnsetInstance( ) const;

      void read( IFrameStream& Stream );

      void Verify( ) const;

      virtual void Write( OFrameStream& Stream ) const;

      virtual void WriteDescription( OFrameStream& Stream ) const;

      const FrBase& operator=( const FrBase& Base );

    protected:
      void write( OFrameStream& Stream ) const;
	
    private:
      struct image_t
      {
	mutable INT_8U  length;
	INT_2U	class_id;
	mutable INT_4U	instance;
      } m_data;

      PTR_STRUCT	m_next;

      mutable bool	m_instance_set;

      //: Don't want anyone calling the default constructor.
      FrBase( );

    };

    inline INT_2U FrBase::
    GetClassId( ) const
    {
      return m_data.class_id;
    }
    
    inline INT_4U FrBase::
    GetInstance( ) const
    {
      return m_data.instance;
    }
    
    inline INT_8U FrBase::
    GetLength( ) const
    {
      return m_data.length;
    }

    inline PTR_STRUCT FrBase::
    GetNext( ) const
    {
      return m_next;
    }

    inline bool FrBase::
    IsInstanceSet( ) const
    {
      return m_instance_set;
    }

    inline void FrBase::
    SetInstance( INT_4U Instance ) const
    {
      m_data.instance = Instance;
      m_instance_set = true;
    }

    inline void FrBase::
    SetLength( ) const
    {
      m_data.length = Bytes( );
    }

    inline void FrBase::
    SetNext( const PTR_STRUCT& Next )
    {
      m_next = Next;
    }

    inline void FrBase::
    UnsetInstance( ) const
    {
      m_instance_set = false;
    }

  } // namespace - Version_6

} // namespace FrameCPP

std::ostream&
operator<<( std::ostream& Stream, const FrameCPP::Version_6::FrBase& Base );

#endif /* FrameCPP_VERSION_6_FrBase_HH */
