#include "framecpp/config.h"

#include "framecpp/Version3_4_5/detector.hh"

#include "framecpp/Version6/FrCommon.hh"
#include "framecpp/Version6/Frame5ReaderBase.hh"
#include "framecpp/Version6/FrDetector.hh"
#include "framecpp/Version6/FrSE.hh"
#include "framecpp/Version6/FrSH.hh"
#include "framecpp/Version6/IFrameStream.hh"
#include "framecpp/Version6/OFrameStream.hh"
#include "framecpp/Version6/PTR_STRUCT.hh"

#define	LM_DEBUG 0

#if LM_DEBUG
#define	AT() std::cerr << __FILE__ << " " << __LINE__ << std::endl;
#else
#define	AT()
#endif

using namespace FrameCPP::Version_6;

namespace {
  class MapDetectorRef
    : public IFrameStream::MapRef< FrameCPP::Version_3_4_5::Detector, FrDetector >
  {
  public:
    MapDetectorRef( IFrameStream& Stream,
		 const FrameCPP::Version_3_4_5::Detector* Detector5,
		 FrDetector& Detector6 )
      : IFrameStream::MapRef< FrameCPP::Version_3_4_5::Detector, FrDetector >( Stream, Detector5, Detector6 )
    {
    }

  protected:
   
    void operator()( const void* Ref5Ptr,
		     const PTR_STRUCT& PtrStruct );
  }; // class - MapDetectorRef
   
   
  void MapDetectorRef::operator()( const void* Ref5Ptr,
                                   const PTR_STRUCT& PtrStruct )
  {
      if ( Ref5Ptr == &(m_struct_5->refMore( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefAux( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refMoreTable( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefTable( ), PtrStruct );
      }
   
      return;
   }   
} // namespace - anonymous


const char* FrDetector::getStaticName()
{
   static const CHAR* class_name( "FrDetector" );
   return class_name;
}


FrDetector::
FrDetector( )
  : FrBase( CLASS_FR_DETECTOR )
{
  m_data.prefix[0] = ' ';
  m_data.prefix[1] = ' ';
}

FrDetector::
FrDetector( const std::string& Name,
	    const char* Prefix,
	    const REAL_8 Longitude,
	    const REAL_8 Latitude,
	    const REAL_4 Elevation,
	    const REAL_4 ArmXazimuth,
	    const REAL_4 ArmYazimuth,
	    const REAL_4 ArmXaltitude,
	    const REAL_4 ArmYaltitude,
	    const REAL_4 ArmXmidpoint,
	    const REAL_4 ArmYmidpoint,
	    const INT_4S LocalTime )
  : FrBase( CLASS_FR_DETECTOR )
{
  m_data.name = Name;
  m_data.prefix[0] = Prefix[0];
  m_data.prefix[1] = Prefix[1];
  m_data.longitude = Longitude;
  m_data.latitude = Latitude;
  m_data.elevation = Elevation;
  m_data.armXazimuth = ArmXazimuth;
  m_data.armYazimuth = ArmYazimuth;
  m_data.armXaltitude = ArmXaltitude;
  m_data.armYaltitude = ArmYaltitude;
  m_data.armXmidpoint = ArmXmidpoint;
  m_data.armYmidpoint = ArmYmidpoint;
  m_data.localTime = LocalTime;
}

FrDetector::
FrDetector( const FrDetector& detector )
  : FrBase( CLASS_FR_DETECTOR )
{
  AT( );
  m_data.name = detector.m_data.name;
  AT( );
  m_data.prefix[0] = detector.m_data.prefix[0];
  AT( );
  m_data.prefix[1] = detector.m_data.prefix[1];
  AT( );
  m_data.longitude = detector.m_data.longitude;
  AT( );
  m_data.latitude = detector.m_data.latitude;
  AT( );
  m_data.elevation = detector.m_data.elevation;
  AT( );
  m_data.armXazimuth = detector.m_data.armXazimuth;
  AT( );
  m_data.armYazimuth = detector.m_data.armYazimuth;
  AT( );
  m_data.armXaltitude = detector.m_data.armXaltitude;
  AT( );
  m_data.armYaltitude = detector.m_data.armYaltitude;
  AT( );
  m_data.armXmidpoint = detector.m_data.armXmidpoint;
  AT( );
  m_data.armYmidpoint = detector.m_data.armYmidpoint;
  AT( );
  m_data.localTime = detector.m_data.localTime;
  AT( );
  m_data.aux = detector.m_data.aux;
  AT( );
  m_data.table = detector.m_data.table;
  AT( );
}

#if ENABLE_DEPRICATED_FRAMECPP
FrDetector::
FrDetector( const std::string& name,
	    REAL_8 longitude, REAL_8 latitude, REAL_4 elevation,
	    REAL_4 armXazimuth, REAL_4 armYazimuth,
	    REAL_4 armXaltitude, REAL_4 armYaltitude,
	    REAL_4 armXmidpoint, REAL_4 armYmidpoint,
	    INT_4S localTime, INT_4U /* dataQuality */,
	    const std::string& /* qaBitList */ )
  : FrBase( CLASS_FR_DETECTOR )
{
  m_data.name = name;
  m_data.prefix[0] = ' ';
  m_data.prefix[1] = ' ';
  m_data.longitude = longitude;
  m_data.latitude = latitude;
  m_data.elevation = elevation;
  m_data.armXazimuth = armXazimuth;
  m_data.armYazimuth = armYazimuth;
  m_data.armXaltitude = armXaltitude;
  m_data.armYaltitude = armYaltitude;
  m_data.armXmidpoint = armXmidpoint;
  m_data.armYmidpoint = armYmidpoint;
  m_data.localTime = localTime;
}
#endif /* ENABLE_DEPRICATED_FRAMECPP */


const std::string& FrDetector::GetName( ) const 
{
   return m_data.name;
}


#if ENABLE_DEPRICATED_FRAMECPP
const std::string& FrDetector::getName() const
{
   return GetName( );
}
#endif


INT_8U FrDetector::
Bytes( ) const
{
  INT_8U ret( FrBase::Bytes( ) );

  ret +=
    m_data.name.Bytes( ) +
    sizeof( m_data.prefix ) +
    sizeof( m_data.longitude ) +
    sizeof( m_data.latitude ) +
    sizeof( m_data.elevation ) +
    sizeof( m_data.armXazimuth ) +
    sizeof( m_data.armYazimuth ) +
    sizeof( m_data.armXaltitude ) +
    sizeof( m_data.armYaltitude ) +
    sizeof( m_data.armXmidpoint ) +
    sizeof( m_data.armYmidpoint ) +
    sizeof( m_data.localTime ) +
    PTR_STRUCT::Bytes( ) + 	// aux
    PTR_STRUCT::Bytes( ) + 	// table
    PTR_STRUCT::Bytes( )	// next
    ;
  return ret;
}

FrDetector& FrDetector::
Merge( const FrDetector& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += getStaticName( );

  throw std::domain_error( msg );
  return *this;
}

FrDetector* FrDetector::
ReadDynamic( IFrameStream& Stream, const FrBase& Base )
{
  std::auto_ptr< FrDetector > ret( new FrDetector );
  ret->read( Stream );
  ret->SetInstance( Base.GetInstance( ) );
  return ret.release( );
}

void FrDetector::
Write( OFrameStream& Stream ) const
{
  Stream << m_data.name;
  Stream.write( m_data.prefix, sizeof( m_data.prefix ) );
  Stream << m_data.longitude
	 << m_data.latitude
	 << m_data.elevation
	 << m_data.armXazimuth
	 << m_data.armYazimuth
	 << m_data.armXaltitude
	 << m_data.armYaltitude
	 << m_data.armXmidpoint
	 << m_data.armYmidpoint
	 << m_data.localTime
	 << m_data.aux.containerCast( )
	 << m_data.table.containerCast( )
	 << GetNext( )
    ;
}

void FrDetector::
WriteDescription( OFrameStream& Stream ) const
{
  Stream.WriteDictionary( FrSH( getStaticName(), CLASS_FR_DETECTOR,
				"Detector Data Structure" ) );
  Stream.WriteDictionary( FrSE( "name", "STRING" ) );
  Stream.WriteDictionary( FrSE( "prefix", "CHAR[2]""" ) );
  Stream.WriteDictionary( FrSE( "longitude", "REAL_8" ) );
  Stream.WriteDictionary( FrSE( "latitude", "REAL_8" ) );
  Stream.WriteDictionary( FrSE( "elevation", "REAL_4" ) );
  Stream.WriteDictionary( FrSE( "armXazimuth", "REAL_4" ) );
  Stream.WriteDictionary( FrSE( "armYazimuth", "REAL_4" ) );
  Stream.WriteDictionary( FrSE( "armXaltitude", "REAL_4" ) );
  Stream.WriteDictionary( FrSE( "armYaltitude", "REAL_4" ) );
  Stream.WriteDictionary( FrSE( "armXmidpoint", "REAL_4" ) );
  Stream.WriteDictionary( FrSE( "armYmidpoint", "REAL_4" ) );
  Stream.WriteDictionary( FrSE( "localTime", "INT_4S" ) );

  Stream.WriteDictionary( FrSE( "aux",
				PTR_STRUCT::Desc( FrVect::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "table",
				PTR_STRUCT::Desc( FrTable::getStaticName() ) ) );

  Stream.WriteDictionary( FrSE( "next", PTR_STRUCT::Desc( getStaticName() ) ) );
}

void FrDetector::
read( IFrameStream& Stream )
{
  using namespace FrameCPP::Version_3_4_5;

  std::auto_ptr< Detector >	ver5;
  PTR_STRUCT			next;
      
  switch( Stream.GetVersion( ) )
  {
  case DATA_FORMAT_VERSION:
    {
      Stream >> m_data.name;
      Stream.read( m_data.prefix, sizeof( m_data.prefix ) );
      Stream >> m_data.longitude
	     >> m_data.latitude
	     >> m_data.elevation
	     >> m_data.armXazimuth
	     >> m_data.armYazimuth
	     >> m_data.armXaltitude
	     >> m_data.armYaltitude
	     >> m_data.armXmidpoint
	     >> m_data.armYmidpoint
	     >> m_data.localTime
	     >> m_data.aux.containerCast( )
	     >> m_data.table.containerCast( )
	     >> next
	;
    }
    break;
  case 5:
    ver5.reset( Detector::read( Stream.GetFrame5Reader( ) ) );
    next = NULL_PTR_STRUCT;
    break;
  case 4:
    ver5.reset( Detector::read4( Stream.GetFrame5Reader( ) ) );
    next = NULL_PTR_STRUCT;
    break;
  }
  if ( ver5.get( ) )
  {
    m_data.name = ver5->getName( );
    m_data.longitude = ver5->getLongitude( );
    m_data.latitude = ver5->getLatitude( );
    m_data.elevation = ver5->getElevation( );
    m_data.armXazimuth = ver5->getArmXAzimuth( );
    m_data.armYazimuth = ver5->getArmYAzimuth( );
    m_data.armXaltitude = ver5->getArmXAltitude( );
    m_data.armYaltitude = ver5->getArmYAltitude( );
    m_data.armXmidpoint = ver5->getArmXMidpoint( );
    m_data.armYmidpoint = ver5->getArmYMidpoint( );
    m_data.localTime = ver5->getLocalTime( );

    m_data.prefix[ 0 ] = ' ';
    m_data.prefix[ 1 ] = ' ';

    MapDetectorRef	map_table( Stream, ver5.get( ), *this );
    map_table.Map();
  }
  //---------------------------------------------------------------------
  // Add the reference to the next
  //---------------------------------------------------------------------

  SetNext( next );
}
