#ifndef FrameCPP_VERSION_6_FrEndOfFile_HH
#define FrameCPP_VERSION_6_FrEndOfFile_HH

#include "framecpp/Version6/FrBase.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    class FrEndOfFile: public FrBase
    {
    public:

      static const CHAR* getStaticName();

      //: Default constructor
      FrEndOfFile( );

      //: Number of bytes needed to write this structure
      INT_8U Bytes( ) const;

      static INT_8U Bytes( INT_2U Version );

      //: Retrieve the number of frames in this file
      INT_4U	GetNFrames( ) const;

      //: Total number of bytes in this file (0 if NOT computed)
      INT_8U	GetNBytes( ) const;

      //: Checksum schemes
      INT_4U	GetChkType( ) const;
      
      //: File checksum
      INT_4U	GetChkSum( ) const;

      //: Retrieve the start of the TOC, 0 if there is none.
      INT_8U GetSeekTOC( ) const;

      //: Read the structure from stream
      static FrEndOfFile* ReadDynamic( IFrameStream& Stream,
				       const FrBase& Base );

      ///: Establish the number of bytes written
      void SetNBytes( INT_8U NumberOfBytes );

      ///: Establish the number of frames written
      void SetNFrames( INT_4U NumberOfFrames );

      ///: Establish where the toc is
      void SetSeekTOC( INT_8U Position );

      //: Write the structure to the stream
      virtual void Write( OFrameStream& Stream ) const;

      //: Write the structure description to the stream
      virtual void WriteDescription( OFrameStream& Stream ) const;

    private:
      mutable struct image
      {
	INT_4U	nFrames;
	INT_8U	nBytes;
	INT_4U	chkType;
	INT_4U	chkSum;
	INT_8U	seekTOC;
      } m_data;

      //: Actually read the structure specific data off the stream
      void read( IFrameStream& Stream );
    }; // class - FrEndOfFile

    //: Retrieve the number of frames in this file
    inline INT_4U FrEndOfFile::
    GetNFrames( ) const
    {
      return m_data.nFrames;
    }

    //: Total number of bytes in this file (0 if NOT computed)
    inline INT_8U FrEndOfFile::
    GetNBytes( ) const
    {
      return m_data.nBytes;
    }
    
    //: Checksum schemes
    inline INT_4U FrEndOfFile::
    GetChkType( ) const
    {
      return m_data.chkType;
    }
      
    //: File checksum
    inline INT_4U FrEndOfFile::
    GetChkSum( ) const
    {
      return m_data.chkSum;
    }

    inline INT_8U FrEndOfFile::
    GetSeekTOC( ) const
    {
      return m_data.seekTOC;
    }

    inline void FrEndOfFile::
    SetNBytes( INT_8U NumberOfBytes )
    {
      m_data.nBytes = NumberOfBytes;
    }

    inline void FrEndOfFile::
    SetNFrames( INT_4U NumberOfFrames )
    {
      m_data.nFrames = NumberOfFrames;
    }

    inline void FrEndOfFile::
    SetSeekTOC( INT_8U Position )
    {
      m_data.seekTOC = Position;
    }

  } // namespace - Version_6
} // namespace - FrameCPP

#endif /* FrameCPP_VERSION_6_FrEndOfFile_HH */
