#ifndef FrameCPP_VERSION_6_FrEndOfFrame_HH
#define FrameCPP_VERSION_6_FrEndOfFrame_HH

#include "general/types.hh"

#include "framecpp/Version6/FrBase.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    class FrEndOfFrame: public FrBase
    {
    public:

      static const CHAR* getStaticName();

      //: Default constructor
      FrEndOfFrame( );

      //: Default constructor
      FrEndOfFrame( INT_4S Run, INT_4U Frame );

      //: Return the number of bytes needed to write this object
      INT_8U Bytes( ) const;

      //: Return run number of frame
      INT_4S	GetRun( ) const;

      //: Return frame number
      INT_4U	GetFrame( ) const;

      //: Return check type
      INT_4U	GetChkType( ) const;

      //: Return check sum
      INT_4U	GetChkSum( ) const;

      //: Read the structure from stream
      static FrEndOfFrame* ReadDynamic( IFrameStream& Stream,
					const FrBase& Base );

      //: Write the structure to the stream
      virtual void Write( OFrameStream& Stream ) const;

      //: Write the structure description to the stream
      virtual void WriteDescription( OFrameStream& Stream ) const;

    private:
      mutable struct image_t
      {
	INT_4S	run;
	INT_4U	frame;
	INT_4U	chkType;
	INT_4U	chkSum;
      } m_data;

      void read( IFrameStream& Stream );
    };

    //: Return run number of frame
    inline INT_4S FrEndOfFrame::
    GetRun( ) const
    {
      return m_data.run;
    }

    //: Return frame number
    inline INT_4U FrEndOfFrame::
    GetFrame( ) const
    {
      return m_data.frame;
    }
    
    //: Return check type
    inline INT_4U FrEndOfFrame::
    GetChkType( ) const
    {
      return m_data.chkType;
    }

    //: Return check sum
    inline INT_4U FrEndOfFrame::
    GetChkSum( ) const
    {
      return m_data.chkSum;
    }

  } // namespace - Version_6
} // namespce - FrameCPP

#endif /* FrameCPP_VERSION_6_FrEndOfFrame_HH */
