#ifndef FrameCPP_VERSION_6_FrEvent_HH
#define FrameCPP_VERSION_6_FrEvent_HH

#include <string>
#include <vector>

#include "framecpp/FrameCPP.hh"

#include "framecpp/Version6/SearchContainer.hh"
#include "framecpp/Version6/FrBase.hh"
#include "framecpp/Version6/FrCommon.hh"
#include "framecpp/Version6/FrTable.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    class FrEvent: public FrBase
    {
    public:
      typedef std::pair< STRING, REAL_4 >	Param_type;
      typedef std::vector< Param_type >	ParamList_type;

      typedef SearchContainer< FrVect, &FrVect::GetName >	data_type;
      typedef SearchContainer< FrTable, &FrTable::GetName >	table_type;

      typedef data_type::iterator iterator;
      typedef data_type::const_iterator const_iterator;
      typedef table_type::iterator table_iterator;
      typedef table_type::const_iterator const_table_iterator;

      static const CHAR* getStaticName();

      //: Default constructor
      FrEvent( );
      FrEvent( const FrEvent& eventData );
      FrEvent( const std::string& name, const std::string& comment,
	       const std::string& inputs,
	       const GPSTime& time,
	       REAL_4 timeBefore, REAL_4 timeAfter,
	       INT_4U eventStatus, REAL_4 amplitude,
	       REAL_4 prob,
	       const std::string& statistics,
	       const ParamList_type& parameters );

      //: Number of bytes needed to write this structure
      INT_8U Bytes( ) const;

      const std::string& GetName() const;
      const std::string& GetComment( ) const { return m_data.comment; }
      const std::string& GetInputs( ) const { return m_data.inputs; }
      const GPSTime& GetGTime( ) const { return m_data.GTime; }
      REAL_4 GetTimeBefore( ) const { return m_data.timeBefore; }
      REAL_4 GetTimeAfter( ) const { return m_data.timeAfter; }
      INT_4U GetEventStatus( ) const { return m_data.eventStatus; }
      REAL_4 GetAmplitude( ) const { return m_data.amplitude; }
      REAL_4 GetProbability( ) const { return m_data.probability; }
      const std::string& GetStatistics( ) const { return m_data.statistics; }
      const ParamList_type& GetParam( ) const { return m_data.Params; }
      ParamList_type& GetParam( ) { return m_data.Params; }
      const data_type& RefData( ) const { return m_data.data; }
      data_type& RefData( ) { return m_data.data; }
      const table_type& RefTable( ) const { return m_data.table; }
      table_type& RefTable( ) { return m_data.table; }
   
      //: Merge with another FrEvent
      FrEvent& Merge( const FrEvent& RHS );

      //: Read the structure from stream
      static FrEvent* ReadDynamic( IFrameStream& Stream,
				  const FrBase& Base );

      //: Write the structure to the stream
      virtual void Write( OFrameStream& Stream ) const;

      //: Write the structure description to the stream
      virtual void WriteDescription( OFrameStream& Stream ) const;

    private:
      struct {
	STRING	name;
	STRING	comment;
	STRING	inputs;
	GPSTime	GTime;
	REAL_4	timeBefore;
	REAL_4	timeAfter;
	INT_4U	eventStatus;
	REAL_4	amplitude;
	REAL_4	probability;
	STRING	statistics;
	ParamList_type	Params;
	data_type		data;
	table_type		table;
      } m_data;

      void read( IFrameStream& Stream );
#if ENABLE_DEPRICATED_FRAMECPP
    public:
      typedef data_type DataContainer;
      typedef table_type TableContainer;

      typedef std::pair< std::string, REAL_4 >	Param5_type;
      typedef std::vector< Param5_type >	ParamList5_type;

      FrEvent( const std::string& name, const std::string& comment,
	       const std::string& inputs,
	       const GPSTime& time,
	       REAL_4 timeBefore, REAL_4 timeAfter,
	       INT_4U eventStatus, REAL_4 amplitude,
	       REAL_4 prob,
	       const std::string& statistics,
	       const ParamList5_type& parameters );

      const std::string& getName() const;
      const std::string& getComment() const { return GetComment( ); }
      const std::string& getInputs() const { return GetInputs( ); }
      const GPSTime& getGTime() const { return GetGTime( ); }
      REAL_4 getTimeBefore() const { return GetTimeBefore( ); }
      REAL_4 getTimeAfter() const { return GetTimeAfter( ); }
      INT_4U getEventStatus() const { return GetEventStatus( ); }
      REAL_4 getAmplitude() const { return GetAmplitude( ); }
      REAL_4 getProbability() const { return GetProbability( ); }
      const std::string& getStatistics( ) const { return GetStatistics( ); }
      const ParamList_type& getParameters( ) { return GetParam( ); }

      DataContainer& refData() { return RefData( ); }
      const DataContainer& refData() const { return RefData( ); }
      TableContainer& refTable()  { return RefTable( ); }
      const TableContainer& refTable() const { return RefTable( ); }
#endif
    };

  } // namespace - Version_6
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_6_FrEvent_HH */
