#include "framecpp/config.h"

#include "framecpp/Version3_4_5/rawdata.hh"

#include "framecpp/Version6/FrCommon.hh"
#include "framecpp/Version6/Frame5ReaderBase.hh"
#include "framecpp/Version6/FrRawData.hh"
#include "framecpp/Version6/FrSE.hh"
#include "framecpp/Version6/FrSH.hh"
#include "framecpp/Version6/IFrameStream.hh"
#include "framecpp/Version6/OFrameStream.hh"
#include "framecpp/Version6/PTR_STRUCT.hh"

using namespace FrameCPP::Version_6;

namespace {
  class MapRawDataRef
    : public IFrameStream::MapRef< FrameCPP::Version_3_4_5::RawData, FrRawData >
  {
  public:
    MapRawDataRef( IFrameStream& Stream,
		 const FrameCPP::Version_3_4_5::RawData* RawData5,
		 FrRawData& RawData6 )
      : IFrameStream::MapRef< FrameCPP::Version_3_4_5::RawData, FrRawData >( Stream, RawData5, RawData6 )
    {
    }

  protected:
   
    // virtual
    void operator()( const void* Ref5Ptr,
                     const PTR_STRUCT& PtrStruct );

  }; // class - MapRawDataRef
   
   
  void MapRawDataRef::operator()( const void* Ref5Ptr,
 			          const PTR_STRUCT& PtrStruct )
  {
      if ( Ref5Ptr == &(m_struct_5->refSerial( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefFirstSer( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refAdc( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefFirstAdc( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refTable( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefFirstTable( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refMsg( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefLogMsg( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refMore( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefMore( ), PtrStruct );
      }
   
      return;
  }   
   
} // namespace - anonymous


const CHAR* FrRawData::getStaticName()
{
   static const CHAR* class_name( "FrRawData" );
   return class_name;
}


#define TRACE_MEMORY 0
#if TRACE_MEMORY
#define MEM_ALLOCATE() std::cerr << "MEMORY: Allocate: " << FrRawData::getStaticName() << " " << (void*)this << std::endl;
#define MEM_DELETE() std::cerr << "MEMORY: Delete: " << FrRawData::getStaticName() << " " << (void*)this << std::endl;
#else
#define MEM_ALLOCATE()
#define MEM_DELETE()
#endif

FrRawData::
FrRawData( )
  : FrBase( CLASS_FR_RAW_DATA )
{
  MEM_ALLOCATE( );
}

FrRawData::
FrRawData( const FrRawData& rawData )
  : FrBase( CLASS_FR_RAW_DATA )
{
  MEM_ALLOCATE( );
  m_data.name = rawData.m_data.name;
  m_data.firstSer = rawData.m_data.firstSer;
  m_data.firstAdc = rawData.m_data.firstAdc;
  m_data.firstTable = rawData.m_data.firstTable;
  m_data.logMsg = rawData.m_data.logMsg;
  m_data.more = rawData.m_data.more;
}

FrRawData::
FrRawData( const std::string& name )
  : FrBase( CLASS_FR_RAW_DATA )
{
  MEM_ALLOCATE( );
  m_data.name = name;
}

FrRawData::
~FrRawData( )
{
  MEM_DELETE( );
}

const std::string& FrRawData::GetName() const
{ 
   return m_data.name;
}


#if ENABLE_DEPRICATED_FRAMECPP
const std::string& FrRawData::getName() const 
{
   return GetName();
}
#endif


INT_8U FrRawData::
Bytes( ) const
{
  return
    FrBase::Bytes( ) +
    m_data.name.Bytes( ) +
    PTR_STRUCT::Bytes( ) +	// firstSer
    PTR_STRUCT::Bytes( ) +	// firstAdc
    PTR_STRUCT::Bytes( ) +	// firstTable
    PTR_STRUCT::Bytes( ) +	// logMsg
    PTR_STRUCT::Bytes( )	// more
    ;
}

FrRawData* FrRawData::
ReadDynamic( IFrameStream& Stream, const FrBase& Base )
{
  std::auto_ptr< FrRawData > ret( new FrRawData );
  ret->read( Stream );
  ret->SetInstance( Base.GetInstance( ) );
  return ret.release( );
}

void FrRawData::
Write( OFrameStream& Stream ) const
{
  Stream
    << m_data.name
    << m_data.firstSer.containerCast( )
    << m_data.firstAdc.containerCast( )
    << m_data.firstTable.containerCast( )
    << m_data.logMsg.containerCast( )
    << m_data.more.containerCast( )
    ;
}

void FrRawData::
WriteDescription( OFrameStream& Stream ) const
{
  Stream.WriteDictionary( FrSH( getStaticName(), CLASS_FR_RAW_DATA,
				"Raw Data Structure" ) );

  Stream.WriteDictionary( FrSE( "name", "STRING" ) );
  Stream.WriteDictionary( FrSE( "firstSer",
				PTR_STRUCT::Desc( FrSerData::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "firstAdc",
				PTR_STRUCT::Desc( FrAdcData::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "firstTable",
				PTR_STRUCT::Desc( FrTable::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "logMsg",
				PTR_STRUCT::Desc( FrMsg::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "more",
				PTR_STRUCT::Desc( FrVect::getStaticName() ) ) );

  return;
}

void FrRawData::
read( IFrameStream& Stream )
{
  using namespace FrameCPP::Version_3_4_5;

  std::auto_ptr< RawData >	ver5;
      
  switch( Stream.GetVersion( ) )
  {
  case DATA_FORMAT_VERSION:
    {
      Stream
	>> m_data.name
	>> m_data.firstSer.containerCast( )
	>> m_data.firstAdc.containerCast( )
	>> m_data.firstTable.containerCast( )
	>> m_data.logMsg.containerCast( )
	>> m_data.more.containerCast( )
	;
    }
    break;
  case 5:
  case 4:
    ver5.reset( RawData::read( Stream.GetFrame5Reader( ) ) );
    break;
  case 3:
    ver5.reset( RawData::read3( Stream.GetFrame5Reader( ) ) );
    break;
  }
  if ( ver5.get( ) )
  {
    MapRawDataRef	map_refs( Stream, ver5.get( ), *this );
    map_refs.Map();
  }
  //---------------------------------------------------------------------
  // Add the reference to the next
  //---------------------------------------------------------------------
  SetNext( NULL_PTR_STRUCT );
}

FrRawData::m_data_type::
m_data_type( )
  : firstSer( false ),
    firstAdc( false )
{
}
