#include "framecpp/config.h"

#include "framecpp/Version3_4_5/serdata.hh"

#include "framecpp/Version6/FrCommon.hh"
#include "framecpp/Version6/Frame5ReaderBase.hh"
#include "framecpp/Version6/FrSerData.hh"
#include "framecpp/Version6/FrSE.hh"
#include "framecpp/Version6/FrSH.hh"
#include "framecpp/Version6/IFrameStream.hh"
#include "framecpp/Version6/OFrameStream.hh"
#include "framecpp/Version6/PTR_STRUCT.hh"

using namespace FrameCPP::Version_6;

namespace {
  class MapSerDataRef
    : public IFrameStream::MapRef< FrameCPP::Version_3_4_5::SerData, FrSerData >
  {
  public:
    MapSerDataRef( IFrameStream& Stream,
		 const FrameCPP::Version_3_4_5::SerData* SerData5,
		 FrSerData& SerData6 )
      : IFrameStream::MapRef< FrameCPP::Version_3_4_5::SerData, FrSerData >( Stream, SerData5, SerData6 )
    {
    }

  protected:
   
    // virtual
    void operator()( const void* Ref5Ptr,
                     const PTR_STRUCT& PtrStruct );
   
  }; // class - MapSerDataRef
   
   
  void MapSerDataRef::operator()( const void* Ref5Ptr,
                   	          const PTR_STRUCT& PtrStruct )
  {
      if ( Ref5Ptr == &(m_struct_5->refData( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefSerial( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refTable( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefTable( ), PtrStruct );
      }
   
      return;
  }   
   
} // namespace - anonymous


const CHAR* FrSerData::getStaticName()
{
   static const CHAR* class_name( "FrSerData" );
   return class_name;
}


FrSerData::
FrSerData( )
  : FrBase( CLASS_FR_SER_DATA )
{
}

FrSerData::
FrSerData( const FrSerData& SerData )
  : FrBase( CLASS_FR_SER_DATA )
{
  m_data.name = SerData.m_data.name;
  m_data.time = SerData.m_data.time;
  m_data.sampleRate = SerData.m_data.sampleRate;
  m_data.data = SerData.m_data.data;
  m_data.serial = SerData.m_data.serial;
  m_data.table = SerData.m_data.table;
}

FrSerData::
FrSerData( const std::string& name, const GPSTime& time, REAL_4 sampleRate )
  : FrBase( CLASS_FR_SER_DATA )
{
  m_data.name = name;
  m_data.time = time;
  m_data.sampleRate = sampleRate;
}


const std::string& FrSerData::GetName() const
{ 
   return m_data.name;
}


#if ENABLE_DEPRICATED_FRAMECPP
const std::string& FrSerData::getName( ) const
{
   return GetName( );
}
#endif

INT_8U FrSerData::
Bytes( ) const
{
  return
    FrBase::Bytes( ) +
    m_data.name.Bytes( ) +
    FrameCPP::Version_6::Bytes( m_data.time ) +
    sizeof( m_data.sampleRate ) +
    m_data.data.Bytes( ) +
    PTR_STRUCT::Bytes( ) +	// serial
    PTR_STRUCT::Bytes( ) +	// table
    PTR_STRUCT::Bytes( )	// next
    ;
}

FrSerData& FrSerData::
Merge( const FrSerData& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += getStaticName( );

  throw std::domain_error( msg );
  return *this;
}

FrSerData* FrSerData::
ReadDynamic( IFrameStream& Stream, const FrBase& Base )
{
  std::auto_ptr< FrSerData > ret( new FrSerData );
  ret->read( Stream );
  ret->SetInstance( Base.GetInstance( ) );
  return ret.release( );
}

void FrSerData::
Write( OFrameStream& Stream ) const
{
  Stream << m_data.name
	 << m_data.time
	 << m_data.sampleRate
	 << m_data.data
	 << m_data.serial.containerCast( )
	 << m_data.table.containerCast( )
	 << GetNext( )
    ;
}

void FrSerData::
WriteDescription( OFrameStream& Stream ) const
{
  Stream.WriteDictionary( FrSH( getStaticName(), CLASS_FR_SER_DATA,
				"Serial Data Structure" ) );

  Stream.WriteDictionary( FrSE( "name", "STRING" ) );
  Stream.WriteDictionary( FrSE( "timeSec", "INT_4U" ) );
  Stream.WriteDictionary( FrSE( "timeNsec", "INT_4U" ) );
  Stream.WriteDictionary( FrSE( "sampleRate", "REAL_4" ) );
  Stream.WriteDictionary( FrSE( "data", "STRING" ) );

  Stream.WriteDictionary( FrSE( "serial",
				PTR_STRUCT::Desc( FrVect::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "table",
				PTR_STRUCT::Desc( FrTable::getStaticName() ) ) );

  Stream.WriteDictionary( FrSE( "next", PTR_STRUCT::Desc( getStaticName() ) ) );

  return;
}

void FrSerData::
read( IFrameStream& Stream )
{
  using namespace FrameCPP::Version_3_4_5;

  std::auto_ptr< SerData >	ver5;
  PTR_STRUCT			next;
      
  switch( Stream.GetVersion( ) )
  {
  case DATA_FORMAT_VERSION:
    {
      Stream >> m_data.name
	     >> m_data.time
	     >> m_data.sampleRate
	     >> m_data.data
	     >> m_data.serial.containerCast( )
	     >> m_data.table.containerCast( )
	     >> next
	;
    }
    break;
  case 5:
  case 4:
    ver5.reset( SerData::read( Stream.GetFrame5Reader( ) ) );
    next = Stream.GetNextNode5( );
    break;
  case 3:
    ver5.reset( SerData::read3( Stream.GetFrame5Reader( ) ) );
    next = Stream.GetNextNode5( );
    break;
  }
  if ( ver5.get( ) )
  {
    m_data.name = ver5->getName( );
    m_data.time = GPSTime( ver5->getTime( ).getSec( ),
			   ver5->getTime( ).getNSec( ) );
    m_data.sampleRate = ver5->getSampleRate( );
    m_data.data = ver5->getData( );

    MapSerDataRef	map_refs( Stream, ver5.get( ), *this );
    map_refs.Map();
  }
  //---------------------------------------------------------------------
  // Add the reference to the next
  //---------------------------------------------------------------------
  SetNext( next );
}
