#ifndef FrameCPP_VERSION_6_FrSimData_HH
#define FrameCPP_VERSION_6_FrSimData_HH

#include <vector>

#include "framecpp/Version6/SearchContainer.hh"
#include "framecpp/Version6/FrBase.hh"
#include "framecpp/Version6/FrHistory.hh"
#include "framecpp/Version6/FrTable.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    class FrSimData: public FrBase
    {
    public:

      static const CHAR* getStaticName();

      typedef SearchContainer< FrVect, &FrVect::GetName >	data_type;
      typedef SearchContainer< FrVect, &FrVect::GetName >	input_type;
      typedef SearchContainer< FrTable, &FrTable::GetName >	table_type;

      typedef data_type::iterator iterator;
      typedef data_type::const_iterator const_iterator;
      typedef input_type::iterator input_iterator;
      typedef input_type::const_iterator const_input_iterator;
      typedef table_type::iterator table_iterator;
      typedef table_type::const_iterator const_table_iterator;

      //: Default constructor
      FrSimData( );
      FrSimData( const FrSimData& SimData );
      FrSimData( const std::string& name,
		 const std::string& comment,
		 REAL_4 sampleRate, 
		 REAL_8 fshift,
		 REAL_4 phase );

      //: Number of bytes needed to write this structure
      INT_8U Bytes( ) const;

      const std::string& GetName() const;
      const std::string& GetComment() const { return m_data.comment; }
      REAL_4 GetSampleRate() const { return m_data.sampleRate; }
      REAL_8 GetTimeOffset() const { return m_data.timeOffset; }
      REAL_8 GetFShift() const { return m_data.fShift; }
      REAL_4 GetPhase() const { return m_data.phase; }
      const data_type& RefData( ) const { return m_data.data; }
      data_type& RefData( ) { return m_data.data; }
      const input_type& RefInput( ) const { return m_data.input; }
      input_type& RefInput( ) { return m_data.input; }
      const table_type& RefTable( ) const { return m_data.table; }
      table_type& RefTable( ) { return m_data.table; }
   
      //: Merge with another FrAdcData
      FrSimData& Merge( const FrSimData& RHS );

      //: Read the structure from stream
      static FrSimData* ReadDynamic( IFrameStream& Stream,
				  const FrBase& Base );

      //: Write the structure to the stream
      virtual void Write( OFrameStream& Stream ) const;

      //: Write the structure description to the stream
      virtual void WriteDescription( OFrameStream& Stream ) const;

    private:
      struct {
	STRING	name;
	STRING	comment;
	REAL_4	sampleRate;
	REAL_8	timeOffset;
	REAL_8	fShift;
	REAL_4	phase;
	data_type		data;
	input_type		input;
	table_type		table;
      } m_data;

      void read( IFrameStream& Stream );
#if ENABLE_DEPRICATED_FRAMECPP
    public:
      typedef data_type DataContainer;
      typedef input_type InputContainer;
      typedef table_type TableContainer;

      const std::string& getName() const;

      const std::string& getComment() const
      {
	return GetComment( );
      }
      REAL_4 getSampleRate() const
      {
	return GetSampleRate( );
      }
      REAL_8 getFShift() const
      {
	return GetFShift( );
      }
      REAL_4 getPhase() const
      {
	return GetPhase( );
      }

      const data_type&	refData( ) const
      {
	return RefData( );
      }
      data_type&		refData( )
      {
	return RefData( );
      }
      const input_type&	refInput( ) const
      {
	return RefInput( );
      }
      input_type&		refInput( )
      {
	return RefInput( );
      }
      const table_type&	refTable( ) const
      {
	return RefTable( );
      }
      table_type&		refTable( )
      {
	return RefTable( );
      }

#endif // ENABLE_DEPRICATED_FRAMECPP
    };
  } // namespace - Version_6
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_6_FrSimData_HH */
