#include "framecpp/config.h"

#include "framecpp/Version3_4_5/simevent.hh"

#include "framecpp/Version6/FrCommon.hh"
#include "framecpp/Version6/Frame5ReaderBase.hh"
#include "framecpp/Version6/FrSimEvent.hh"
#include "framecpp/Version6/FrSE.hh"
#include "framecpp/Version6/FrSH.hh"
#include "framecpp/Version6/IFrameStream.hh"
#include "framecpp/Version6/OFrameStream.hh"
#include "framecpp/Version6/PTR_STRUCT.hh"

using namespace FrameCPP::Version_6;

namespace {
  class MapSimEventRef
    : public IFrameStream::MapRef< FrameCPP::Version_3_4_5::SimEvent, FrSimEvent >
  {
  public:
    MapSimEventRef( IFrameStream& Stream,
		 const FrameCPP::Version_3_4_5::SimEvent* SimEvent5,
		 FrSimEvent& SimEvent6 )
      : IFrameStream::MapRef< FrameCPP::Version_3_4_5::SimEvent, FrSimEvent >( Stream, SimEvent5, SimEvent6 )
    {
    }

  protected:
   
    // virtual
    void operator()( const void* Ref5Ptr,
                     const PTR_STRUCT& PtrStruct );

  }; // class - MapSimEventRef
   

  void MapSimEventRef::operator()( const void* Ref5Ptr,
 			           const PTR_STRUCT& PtrStruct )
  {
      if ( Ref5Ptr == &(m_struct_5->refData( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefData( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refTable( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefTable( ), PtrStruct );
      }
   
      return;
  }
   
} // namespace - anonymous

const CHAR* FrSimEvent::getStaticName()
{
   static const CHAR* class_name( "FrSimEvent" );
   return class_name;
}


FrSimEvent::
FrSimEvent( )
  : FrBase( CLASS_FR_SIM_EVENT )
{
}

FrSimEvent::
FrSimEvent( const FrSimEvent& eventData )
  : FrBase( CLASS_FR_SIM_EVENT )
{
  m_data.name = eventData.m_data.name;
  m_data.comment = eventData.m_data.comment;
  m_data.inputs = eventData.m_data.inputs;
  m_data.GTime = eventData.m_data.GTime;
  m_data.timeBefore = eventData.m_data.timeBefore;
  m_data.timeAfter = eventData.m_data.timeAfter;
  m_data.amplitude = eventData.m_data.amplitude;
  m_data.Params = eventData.m_data.Params;
  m_data.data = eventData.m_data.data;
  m_data.table = eventData.m_data.table;
}
FrSimEvent::
FrSimEvent( const std::string& name, const std::string& inputs, 
	    const GPSTime& GTime,
	    REAL_4 timeBefore, REAL_4 timeAfter,
	    REAL_4 amplitude )
  : FrBase( CLASS_FR_SIM_EVENT )
{
  m_data.name = name;
  m_data.inputs = inputs;
  m_data.GTime = GTime;
  m_data.timeBefore = timeBefore;
  m_data.timeAfter = timeAfter;
  m_data.amplitude = amplitude;
}


const std::string& FrSimEvent::GetName( ) const
{
   return m_data.name;
}


void FrSimEvent::AppendComment( const std::string& Comment )
{
   return FrameCPP::Version_6::AppendComment( m_data.comment, Comment );
}   


#if ENABLE_DEPRICATED_FRAMECPP
const std::string& FrSimEvent::getName() const
{
   return GetName( );
}
#endif


INT_8U FrSimEvent::
Bytes( ) const
{
  INT_8U	ret( FrBase::Bytes( ) );

  ret +=
    m_data.name.Bytes( ) +
    m_data.comment.Bytes( ) +
    m_data.inputs.Bytes( ) +
    FrameCPP::Version_6::Bytes( m_data.GTime ) +
    sizeof( m_data.timeBefore ) +
    sizeof( m_data.timeAfter ) +
    sizeof( m_data.amplitude ) +
    sizeof( INT_2U );	// nParam
  for ( ParamList_type::const_iterator i ( m_data.Params.begin( ) );
	i != ( m_data.Params.end( ) );
	i++ )
  {
    ret += sizeof( Param_type::second_type ) +
      (*i).first.Bytes( );
  }

  ret +=
    PTR_STRUCT::Bytes( ) +	// data
    PTR_STRUCT::Bytes( ) +	// table
    PTR_STRUCT::Bytes( )	// next
    ;
  return ret;
}

FrSimEvent& FrSimEvent::
Merge( const FrSimEvent& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += getStaticName( );

  throw std::domain_error( msg );
  return *this;
}

FrSimEvent* FrSimEvent::
ReadDynamic( IFrameStream& Stream, const FrBase& Base )
{
  std::auto_ptr< FrSimEvent > ret( new FrSimEvent );
  ret->read( Stream );
  ret->SetInstance( Base.GetInstance( ) );
  return ret.release( );
}

void FrSimEvent::
Write( OFrameStream& Stream ) const
{
  Stream << m_data.name
	 << m_data.comment
	 << m_data.inputs
	 << m_data.GTime
	 << m_data.timeBefore
	 << m_data.timeAfter
	 << m_data.amplitude
	 << m_data.Params
	 << m_data.data.containerCast( )
	 << m_data.table.containerCast( )
	 << GetNext( )
    ;
}

void FrSimEvent::
WriteDescription( OFrameStream& Stream ) const
{
  Stream.WriteDictionary( FrSH( getStaticName(), CLASS_FR_SIM_EVENT,
				"Simulated Event Data Structure" ) );

  Stream.WriteDictionary( FrSE( "name", "STRING" ) );
  Stream.WriteDictionary( FrSE( "comment", "STRING" ) );
  Stream.WriteDictionary( FrSE( "inputs", "STRING" ) );
  Stream.WriteDictionary( FrSE( "GTimeS", "INT_4U" ) );
  Stream.WriteDictionary( FrSE( "GTimeN", "INT_4U" ) );
  Stream.WriteDictionary( FrSE( "timeBefore", "REAL_4" ) );
  Stream.WriteDictionary( FrSE( "timeAfter", "REAL_4" ) );
  Stream.WriteDictionary( FrSE( "amplitude", "REAL_4" ) );
  Stream.WriteDictionary( FrSE( "nParam", "INT_2U" ) );
  Stream.WriteDictionary( FrSE( "parameters", "REAL_4[nParam]" ) );
  Stream.WriteDictionary( FrSE( "parameterNames", "STRING[nParam]" ) );

  Stream.WriteDictionary( FrSE( "data",
				PTR_STRUCT::Desc( FrVect::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "table",
				PTR_STRUCT::Desc( FrTable::getStaticName() ) ) );

  Stream.WriteDictionary( FrSE( "next", PTR_STRUCT::Desc( getStaticName() ) ) );

  return;
}

void FrSimEvent::
read( IFrameStream& Stream )
{
  using namespace FrameCPP::Version_3_4_5;

  std::auto_ptr< SimEvent >	ver5;
  PTR_STRUCT			next;
      
  switch( Stream.GetVersion( ) )
  {
  case DATA_FORMAT_VERSION:
    {
      INT_2U	nParam;

      Stream >> m_data.name
	     >> m_data.comment
	     >> m_data.inputs
	     >> m_data.GTime
	     >> m_data.timeBefore
	     >> m_data.timeAfter
	     >> m_data.amplitude
	     >> nParam
	;
      Stream.ReadVector( m_data.Params, nParam );

      Stream >> m_data.data.containerCast( )
	     >> m_data.table.containerCast( )
	     >> next
	;
    }
    break;
  case 5:
  case 4:
    ver5.reset( SimEvent::read( Stream.GetFrame5Reader( ) ) );
    next = Stream.GetNextNode5( );
    break;
  }
  if ( ver5.get( ) )
  {
    m_data.name = ver5->getName( );
    m_data.comment = ver5->getComment( );
    m_data.comment = ver5->getInputs( );
    m_data.GTime =
      General::GPSTime( ver5->getGTime( ).getSec( ),
			ver5->getGTime( ).getNSec( ) );
    m_data.timeBefore = ver5->getTimeBefore( );
    m_data.timeBefore = ver5->getTimeAfter( );
    m_data.amplitude = ver5->getAmplitude( );

    m_data.Params.resize( 0 );

    MapSimEventRef	map_refs( Stream, ver5.get( ), *this );
    map_refs.Map();
  }
  //---------------------------------------------------------------------
  // Add the reference to the next
  //---------------------------------------------------------------------
  SetNext( next );
}
