#ifndef FrameCPP_VERSION_6_FrSimEvent_HH
#define FrameCPP_VERSION_6_FrSimEvent_HH

#include <string>
#include <vector>

#include "framecpp/Version6/SearchContainer.hh"
#include "framecpp/Version6/FrBase.hh"
#include "framecpp/Version6/FrTable.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    class FrSimEvent: public FrBase
    {
    public:
      typedef std::pair< STRING, REAL_4 >	Param_type;
      typedef std::vector< Param_type >		ParamList_type;

      typedef SearchContainer< FrVect, &FrVect::GetName >	data_type;
      typedef SearchContainer< FrTable, &FrTable::GetName >	table_type;

      typedef data_type::iterator iterator;
      typedef data_type::const_iterator const_iterator;
      typedef table_type::iterator table_iterator;
      typedef table_type::const_iterator const_table_iterator;

      static const CHAR* getStaticName();

      //: Default constructor
      FrSimEvent( );
      FrSimEvent( const FrSimEvent& SimEvent );
      FrSimEvent( const std::string& name, const std::string& inputs = "", 
		  const GPSTime& GTime = GPSTime( 0, 0 ),
		  REAL_4 timeBefore = 0.0, REAL_4 timeAfter = 0.0,
		  REAL_4 amplitude = 0.0 );

      //: Number of bytes needed to write this structure
      INT_8U Bytes( ) const;

      const std::string& GetName( ) const;

      const std::string& GetComment( ) const
      {
        return m_data.comment;
      }
      const std::string& GetInputs( ) const
      {
        return m_data.inputs;
      }
      const GPSTime& GetGTime( ) const
      {
        return m_data.GTime;
      }
      REAL_4 GetTimeBefore( ) const
      {
        return m_data.timeBefore;
      }
      REAL_4 GetTimeAfter( ) const
      {
        return m_data.timeAfter;
      }
      REAL_4 GetAmplitude( ) const
      {
        return m_data.amplitude;
      }
  
      const data_type& RefData( ) const
      {
        return m_data.data;
      }
  
      //: Get a read/write reference to the list of parameters
      ParamList_type& GetParam( ) {
	return m_data.Params;
      }

      //: Get a read-only reference to the list of parameters
      const ParamList_type& GetParam( ) const
      {
	return m_data.Params;
      }

      data_type& RefData( )
      {
        return m_data.data;
      }
  
      const table_type& RefTable( ) const
      {
        return m_data.table;
      }
  
      table_type& RefTable( )
      {
        return m_data.table;
      }
  
      void AppendComment( const std::string& Comment );
   
      //: Merge with another FrAdcData
      FrSimEvent& Merge( const FrSimEvent& RHS );

      //: Read the structure from stream
      static FrSimEvent* ReadDynamic( IFrameStream& Stream,
				  const FrBase& Base );

      //: Write the structure to the stream
      virtual void Write( OFrameStream& Stream ) const;

      //: Write the structure description to the stream
      virtual void WriteDescription( OFrameStream& Stream ) const;

    private:
      struct {
	STRING	name;
	STRING	comment;
	STRING	inputs;
	GPSTime	GTime;
	REAL_4	timeBefore;
	REAL_4	timeAfter;
	REAL_4	amplitude;
	ParamList_type	Params;
	data_type		data;
	table_type		table;
      } m_data;

      void read( IFrameStream& Stream );
#if ENABLE_DEPRICATED_FRAMECPP
    public:
      typedef data_type DataContainer;
      typedef table_type TableContainer;

      const std::string& getName() const;
      const std::string& getComment() const { return GetComment( ); }
      const std::string& getInputs() const { return GetInputs( ); }
      const GPSTime& getGTime() const { return GetGTime( ); }
      REAL_4 getTimeBefore() const { return GetTimeBefore( ); }
      REAL_4 getTimeAfter() const { return GetTimeAfter( ); }
      REAL_4 getAmplitude() const { return GetAmplitude( ); }

      DataContainer& refData() { return RefData( ); }
      const DataContainer& refData() const{ return RefData( ); }
      TableContainer& refTable(){ return RefTable( ); }
      const TableContainer& refTable() const{ return RefTable( ); }

      void appendComment( const std::string& comment ) { AppendComment( comment ); }
#endif
    };
  } // namespace - Version_6
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_6_FrSimEvent_HH */
