#include "framecpp/config.h"

#include <algorithm>

#include "framecpp/Version3_4_5/statdata.hh"

#include "framecpp/Version6/Frame5ReaderBase.hh"
#include "framecpp/Version6/FrDetector.hh"
#include "framecpp/Version6/FrStatData.hh"
#include "framecpp/Version6/FrSE.hh"
#include "framecpp/Version6/FrSH.hh"
#include "framecpp/Version6/IFrameStream.hh"
#include "framecpp/Version6/OFrameStream.hh"
#include "framecpp/Version6/PTR_STRUCT.hh"

using namespace FrameCPP::Version_6;

namespace {
  class MapStatDataRef
    : public IFrameStream::MapRef< FrameCPP::Version_3_4_5::StatData, FrStatData >
  {
  public:
    MapStatDataRef( IFrameStream& Stream,
		 const FrameCPP::Version_3_4_5::StatData* StatData5,
		 FrStatData& StatData6 )
      : IFrameStream::MapRef< FrameCPP::Version_3_4_5::StatData, FrStatData >( Stream, StatData5, StatData6 )
    {
    }

  protected:
   
    // virtual
    void operator()( const void* Ref5Ptr,
                     const PTR_STRUCT& PtrStruct );

  }; // class - MapStatDataRef
   
   
  void MapStatDataRef::operator()( const void* Ref5Ptr,
			           const PTR_STRUCT& PtrStruct )
  {
      if ( Ref5Ptr == &(m_struct_5->refData( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefData( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refTable( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefTable( ), PtrStruct );
      }
   
      return;
  }
   
} // namespace - anonymous


const CHAR* FrStatData::getStaticName()
{
   static const CHAR* class_name( "FrStatData" );
   return class_name;
}


FrStatData::Query::
~Query( )
{
  for( data_type::iterator
	 cur = m_fr_stat_data.begin( ),
	 last = m_fr_stat_data.end( );
       cur != last;
       ++cur )
  {
    if ( (*cur).s_own )
    {
      delete (*cur).s_fr_stat_data;
      (*cur).s_fr_stat_data = (FrStatData*)NULL;
    }
  }
}

void FrStatData::Query::
Add( const std::string& NamePattern,
     const std::string& DetectorPattern,
     const General::GPSTime& StartTime,
     const General::GPSTime& EndTime,
     const INT_4U Version )
{
  //---------------------------------------------------------------------
  //:TODO: Verify mode to be table of contents
  //---------------------------------------------------------------------
  const FrTOC*	toc( m_stream->GetTOC( ) );
  //---------------------------------------------------------------------
  // Establish regex patterns
  //---------------------------------------------------------------------
  Regex	name_regex( NamePattern );
  Regex	detector_regex( DetectorPattern );
  const FrTOC::MapStatType_type&
    frStatData( toc->GetStatType( ) );

  RegexMatch	name_match;
  RegexMatch	detector_match;

  //---------------------------------------------------------------------
  // Loop over all FrStatData structures looking for matches
  //---------------------------------------------------------------------
  for ( FrTOC::MapStatType_type::const_iterator
	  cur = frStatData.begin( ),
	  last = frStatData.end( );
	cur != last;
	++cur )
  {
    if ( // Verify name match
	 name_match.match( name_regex, (*cur).first.s_name.c_str( ) ) &&
	 // Verify detector match
	 detector_match.match( detector_regex,
			       (*cur).first.s_detector.c_str( ) )
	 )
    {
      //-----------------------------------------------------------------
      // Add the FrStatData to the results along with any relevant
      //   detector information.
      //-----------------------------------------------------------------
      add( (*cur).first, (*cur).second,
	   // Round down
	   StartTime.GetSeconds( ),
	   // Round up
	   ( ( EndTime.GetNanoseconds( ) > 0 )
	     ? ( EndTime.GetSeconds( ) + 1 )
	     : ( EndTime.GetSeconds( ) ) ),
	   Version );
    }
  }
  //---------------------------------------------------------------------
  // Read into memory anything that has been requested
  //---------------------------------------------------------------------
  load( );
}

void FrStatData::Query::
Reset( IFrameStream& Stream )
{
  m_fr_stat_data.resize( 0 );
  m_stream = &Stream;
  m_dirty = false;
}

FrStatData* FrStatData::Query::
operator[]( INT_4U Index )
{
  if ( Index >= size( ) )
  {
    throw std::range_error( "Index out of range for FrStatData::Query" );
  }
  m_fr_stat_data[ Index ].s_own = false;
  return m_fr_stat_data[ Index ].s_fr_stat_data;
}

void FrStatData::Query::
add( const FrTOC::StatTypeKey& Key, 
     const FrTOC::StatType_type& Data,
     INT_4U Start,
     INT_4U Stop,
     INT_4U Version )
{
  for ( INT_4U x = 0, last = Data.tStart.size( );
	x < last;
	++x )
  {
    if ( // Verify Time range match
	( ( ( Start == 0) && ( Stop == 0 ) ) ||
	  ( ( Start >= Data.tStart[ x ] ) && ( Start < Data.tEnd[ x ] ) ) ||
	  ( ( Stop >= Data.tStart[ x ] ) && ( Stop < Data.tEnd[ x ] ) ) ||
	  ( ( Start < Data.tStart[ x ] ) &&
	    ( Stop >= Data.tStart[ x ] ) ) ) &&
	// Verify Version
	( ( Version == FrStatData::ALL_VERSIONS ) ||
	  ( Version == FrStatData::LATEST_VERSION ) ||
	  ( Version == Data.version[ x ] ) )
	)
    {
      if ( Version == FrStatData::LATEST_VERSION )
      {
	bool found = false;
	for ( data_type::iterator
		cur_data = m_fr_stat_data.begin( ),
		last_data = m_fr_stat_data.end( );
	      cur_data != last_data;
	      ++cur_data )
	{
	  if ( ( (*cur_data).s_nameStat == Key.s_name ) &&
	       ( (*cur_data).s_detector == Key.s_detector ) &&	       
	       ( (*cur_data).s_tStart == Data.tStart[ x ] ) &&
	       ( (*cur_data).s_tEnd == Data.tEnd[ x ] ) )
	  {
	    if ( (*cur_data).s_version < Data.version[ x ] )
	    {
	      (*cur_data).s_fr_stat_pos = Data.positionStat[ x ];
	      if ( (*cur_data).s_own && (*cur_data).s_fr_stat_data )
	      {
		delete (*cur_data).s_fr_stat_data;
		m_fr_stat_data.back( ).s_fr_stat_data = (FrStatData*)NULL;
	      }
	      (*cur_data).s_version = Data.version[ x ];
	      m_dirty = true;
	    }
	    found = true;
	    break;
	  }
	}
	if ( found == false )
	{
	  m_dirty = true;
	  m_fr_stat_data.push_back( query_info_type( ) );
	  m_fr_stat_data.back( ).s_fr_stat_pos = Data.positionStat[ x ];
	  m_fr_stat_data.back( ).s_fr_stat_data = (FrStatData*)NULL;
	  m_fr_stat_data.back( ).s_own = true;
	  m_fr_stat_data.back( ).s_nameStat = Key.s_name;
	  m_fr_stat_data.back( ).s_detector = Key.s_detector;
	  m_fr_stat_data.back( ).s_tStart = Data.tStart[ x ];
	  m_fr_stat_data.back( ).s_tEnd = Data.tEnd[ x ];
	  m_fr_stat_data.back( ).s_version = Data.version[ x ];
	}
      }
      else
      {
	m_dirty = true;
	m_fr_stat_data.push_back( query_info_type( ) );
	m_fr_stat_data.back( ).s_fr_stat_pos = Data.positionStat[ x ];
	m_fr_stat_data.back( ).s_fr_stat_data = (FrStatData*)NULL;
	m_fr_stat_data.back( ).s_own = true;
	m_fr_stat_data.back( ).s_nameStat = Key.s_name;
	m_fr_stat_data.back( ).s_detector = Key.s_detector;
	m_fr_stat_data.back( ).s_tStart = Data.tStart[ x ];
	m_fr_stat_data.back( ).s_tEnd = Data.tEnd[ x ];
	m_fr_stat_data.back( ).s_version = Data.version[ x ];
      }
    }
  }
}

void FrStatData::Query::
load( )
{
  if ( m_dirty )
  {
    // Sort the list by file position in ascending order
    std::sort( m_fr_stat_data.begin( ),
	       m_fr_stat_data.end( ),
	       CompareFilePosition( ) );

    // Loop over list loading in those that have not yet been read.
    for ( data_type::iterator
	    cur = m_fr_stat_data.begin( ),
	    last = m_fr_stat_data.end( );
	  cur != last;
	  ++cur )
    {
      if ( (*cur).s_fr_stat_data == (FrStatData*)NULL )
      {
	std::auto_ptr< FrStatData >
	  stat_data = m_stream->ReadFrStatData( (*cur).s_fr_stat_pos );
	(*cur).s_fr_stat_data = stat_data.release( );
	(*cur).s_own = true;
      }
    }
    m_dirty = false;
  }
}

const INT_4U FrStatData::ALL_VERSIONS   = 0xFFFFFFFF;
const INT_4U FrStatData::LATEST_VERSION = 0xFFFFFFFE;

FrStatData::
FrStatData( )
  : FrBase( CLASS_FR_STAT_DATA )
{
}

FrStatData::
FrStatData( const FrStatData& statData )
  : FrBase( CLASS_FR_STAT_DATA )
{
  m_data.name = statData.m_data.name;
  m_data.comment = statData.m_data.comment;
  m_data.representation = statData.m_data.representation;
  m_data.timeStart = statData.m_data.timeStart;
  m_data.timeEnd = statData.m_data.timeEnd;
  m_data.version = statData.m_data.version;
  m_data.detector = statData.m_data.detector;
  m_data.data = statData.m_data.data;
  m_data.table = statData.m_data.table;
}

FrStatData::
FrStatData( const std::string& name, const std::string& comment,
	    const std::string& representation,
	    INT_4U timeStart,INT_4U timeEnd,
	    INT_4U version )
  : FrBase( CLASS_FR_STAT_DATA )
{
  m_data.name = name;
  m_data.comment = comment;
  m_data.representation = representation;
  m_data.timeStart = timeStart;
  m_data.timeEnd = timeEnd;
  m_data.version = version;
}


const std::string& FrStatData::GetName() const
{ 
   return m_data.name;
}


#if ENABLE_DEPRICATED_FRAMECPP
const std::string& FrStatData::getName() const
{ 
   return GetName( );
}
#endif

INT_8U FrStatData::
Bytes( ) const
{
  INT_8U	ret( FrBase::Bytes( ) );

  ret +=
    m_data.name.Bytes( ) +
    m_data.comment.Bytes( ) +
    m_data.representation.Bytes( ) +
    sizeof( m_data.timeStart ) +
    sizeof( m_data.timeEnd ) +
    sizeof( m_data.version ) +
    PTR_STRUCT::Bytes( ) +	// detector
    PTR_STRUCT::Bytes( ) +	// data
    PTR_STRUCT::Bytes( )	// table
    ;
  return ret;
}

FrStatData& FrStatData::
Merge( const FrStatData& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += getStaticName( );

  throw std::domain_error( msg );
  return *this;
}

FrStatData* FrStatData::
ReadDynamic( IFrameStream& Stream, const FrBase& Base )
{
  std::auto_ptr< FrStatData > ret( new FrStatData );
  ret->read( Stream );
  ret->SetInstance( Base.GetInstance( ) );
  return ret.release( );
}

void FrStatData::
Write( OFrameStream& Stream ) const
{
  Stream << m_data.name
	 << m_data.comment
	 << m_data.representation
	 << m_data.timeStart
	 << m_data.timeEnd
	 << m_data.version
	 << PTR_STRUCT( m_data.detector->GetClassId( ),
			m_data.detector->GetInstance( ) )
	 << m_data.data.containerCast( )
	 << m_data.table.containerCast( )
    ;
}

void FrStatData::
WriteDescription( OFrameStream& Stream ) const
{
  Stream.WriteDictionary( FrSH( getStaticName(), CLASS_FR_STAT_DATA,
				"Static Data Structure" ) );

  Stream.WriteDictionary( FrSE( "name", "STRING" ) );
  Stream.WriteDictionary( FrSE( "comment", "STRING" ) );
  Stream.WriteDictionary( FrSE( "representation", "STRING" ) );
  Stream.WriteDictionary( FrSE( "timeStart", "INT_4U" ) );
  Stream.WriteDictionary( FrSE( "timeEnd", "INT_4U" ) );
  Stream.WriteDictionary( FrSE( "version", "INT_4U" ) );

  Stream.WriteDictionary( FrSE( "detector",
				PTR_STRUCT::Desc( FrDetector::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "data", PTR_STRUCT::Desc( FrVect::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "table",
				PTR_STRUCT::Desc( FrTable::getStaticName() ) ) );

  return;
}

void FrStatData::
read( IFrameStream& Stream )
{
  using namespace FrameCPP::Version_3_4_5;

  std::auto_ptr< StatData >	ver5;
      
  switch( Stream.GetVersion( ) )
  {
  case DATA_FORMAT_VERSION:
    {
      void* detector = &(m_data.detector);
      Stream >> m_data.name
	     >> m_data.comment
	     >> m_data.representation
	     >> m_data.timeStart
	     >> m_data.timeEnd
	     >> m_data.version
	     >> (FrBase**)detector
	     >> m_data.data.containerCast( )
	     >> m_data.table.containerCast( )
	;
    }
    break;
  case 5:
  case 4:
    ver5.reset( StatData::read( Stream.GetFrame5Reader( ) ) );
    break;
  case 3:
    ver5.reset( StatData::read3( Stream.GetFrame5Reader( ) ) );
    break;
  }
  if ( ver5.get( ) )
  {
    m_data.name = ver5->getName( );
    m_data.comment = ver5->getComment( );
    m_data.comment = ver5->getRepresentation( );
    m_data.timeStart = ver5->getTimeStart( );
    m_data.timeEnd = ver5->getTimeEnd( );
    m_data.version = ver5->getVersion( );



    //-------------------------------------------------------------------
    // Map the containers
    //-------------------------------------------------------------------

    MapStatDataRef	map_refs( Stream, ver5.get( ), *this );
    map_refs.Map();

    //-------------------------------------------------------------------
    // Map the pointers
    //-------------------------------------------------------------------

    Frame5ReaderBase::Pointers&
      pointers( Stream.GetExposedFrame5Reader( ).mPointers );

    for ( Frame5ReaderBase::Pointers::iterator i( pointers.begin( ) );
	  i != pointers.end( );
	  i++ )
    {
      void*	detector5 = &(ver5->mDetector);
      if ( (*i).first == (Base**)( detector5 ) )
      {
	void* detector6 = &m_data.detector;
	Stream.AddPointer( (FrBase**)detector6,
			   PTR_STRUCT( (*i).second.getClass( ),
				       (*i).second.getInstance( ) ) );
      }
    }
    pointers.resize( 0 );
  }
}
