#ifndef FrameCPP_VERSION_6_FrStatData_HH
#define FrameCPP_VERSION_6_FrStatData_HH

#include <vector>

#include "framecpp/Version6/SearchContainer.hh"
#include "framecpp/Version6/FrBase.hh"
#include "framecpp/Version6/FrCommon.hh"
#include "framecpp/Version6/FrTable.hh"
#include "framecpp/Version6/FrTOC.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    class FrDetector;
    class IFrameStream;

    class FrStatData: public FrBase
    {
    public:
      typedef SearchContainer< FrVect, &FrVect::GetName >	data_type;
      typedef SearchContainer< FrTable, &FrTable::GetName >	table_type;

      typedef data_type::iterator iterator;
      typedef data_type::const_iterator const_iterator;
      typedef table_type::iterator table_iterator;
      typedef table_type::const_iterator const_table_iterator;

      static const CHAR* getStaticName();

      static const INT_4U ALL_VERSIONS;
      static const INT_4U LATEST_VERSION;

      class Query
      {
      public:
	~Query( );
	void Add( const std::string& NamePattern,
		  const std::string& DetectorPattern,
		  const General::GPSTime& StartTime,
		  const General::GPSTime& EndTime,
		  const INT_4U Version );

	void Reset( IFrameStream& Stream );
	INT_4U size( ) const;

	FrStatData* operator[]( INT_4U Index );
      private:

	struct query_info_type {
	  FrStatData*	s_fr_stat_data;
	  INT_8U	s_fr_stat_pos;
	  bool		s_own;
	  // Information needed when looking for the latest version
	  std::string	s_nameStat;
	  std::string	s_detector;
	  INT_4U	s_tStart;
	  INT_4U	s_tEnd;
	  INT_4U	s_version;
	};

	typedef std::vector< query_info_type >	data_type;

	class CompareFilePosition {
	public:
	  int operator( )( const query_info_type& LHS,
			   const query_info_type& RHS ) const;
	};

	std::list< const FrDetector* >	m_detectors;
	bool				m_dirty;
	data_type			m_fr_stat_data;
	IFrameStream* 			m_stream;

	void add( const FrTOC::StatTypeKey& Key, 
		  const FrTOC::StatType_type& Data,
		  INT_4U Start,
		  INT_4U Stop,
		  INT_4U Version );

	void load( );
	std::auto_ptr< FrStatData >
	read_fr_stat_data( IFrameStream& Stream,
			   const std::string& Detector,
			   INT_8U PositionOfStatData );

      };

      //: Default constructor
      FrStatData( );
      FrStatData( const FrStatData& statData );
      FrStatData( const std::string& name, const std::string& comment,
		  const std::string& representation,
		  INT_4U timeStart,INT_4U timeEnd,
		  INT_4U version );

      //: Number of bytes needed to write this structure
      INT_8U Bytes( ) const;

      const std::string& GetName() const;
      const std::string& GetComment( ) const { return m_data.comment; }
      const std::string& GetRepresentation( ) const { return m_data.representation; }
      INT_4U GetTimeStart( ) const { return m_data.timeStart; }
      INT_4U GetTimeEnd( ) const { return m_data.timeEnd; }
      INT_4U GetVersion( ) const { return m_data.version; }
      FrDetector* GetDetector( ) { return m_data.detector; }
      const FrDetector* GetDetector( ) const { return m_data.detector; }
      const data_type&	RefData( ) const { return m_data.data; }
      data_type& RefData() { return m_data.data; }
      const table_type&	RefTable() const { return m_data.table; }
      table_type& RefTable() { return m_data.table; }
      void SetDetector( FrDetector* Detector )
      {
	m_data.detector = Detector;
	return;
      }   

      //: Merge with another FrAdcData
      FrStatData& Merge( const FrStatData& RHS );

      //: Read the structure from stream
      static FrStatData* ReadDynamic( IFrameStream& Stream,
				  const FrBase& Base );

      //: Write the structure to the stream
      virtual void Write( OFrameStream& Stream ) const;

      //: Write the structure description to the stream
      virtual void WriteDescription( OFrameStream& Stream ) const;

    private:
      struct {
	STRING		name;
	STRING		comment;
	STRING		representation;
	INT_4U		timeStart;
	INT_4U		timeEnd;
	INT_4U		version;
	FrDetector*	detector;
	data_type		data;
	table_type		table;
      } m_data;

      void read( IFrameStream& Stream );
#if ENABLE_DEPRICATED_FRAMECPP
    public:
      typedef data_type DataContainer;
      typedef table_type TableContainer;

      const std::string& getName() const;
      const std::string& getComment() const { return GetComment( ); }
      const std::string& getRepresentation() const { return GetRepresentation( ); }
      INT_4U getTimeStart() const { return GetTimeStart( ); }
      INT_4U getTimeEnd() const { return GetTimeEnd( ); }
      INT_4U getVersion() const { return GetVersion( ); }
      FrDetector* getDetector() { return GetDetector( ); }
      const FrDetector* getDetector() const { return GetDetector( ); }

      DataContainer& refData() { return RefData( ); }
      const DataContainer& refData() const { return RefData( ); }
      TableContainer& refTable() { return RefTable( ); }
      const TableContainer& refTable() const { return RefTable( ); }

      void setDetector( FrDetector* detector ) { SetDetector( detector ); }
#endif
    };

    inline int FrStatData::Query::CompareFilePosition::
    operator( )( const query_info_type& LHS,
		 const query_info_type& RHS ) const
    {
      return LHS.s_fr_stat_pos < RHS.s_fr_stat_pos;
    }

    inline INT_4U FrStatData::Query::
    size( ) const
    {
      return m_fr_stat_data.size( );
    }

  } // namespace - Version_6
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_6_FrStatData_HH */
