#include "framecpp/config.h"

#include "framecpp/Version3_4_5/summary.hh"

#include "framecpp/Version6/Frame5ReaderBase.hh"
#include "framecpp/Version6/FrSummary.hh"
#include "framecpp/Version6/FrSE.hh"
#include "framecpp/Version6/FrSH.hh"
#include "framecpp/Version6/IFrameStream.hh"
#include "framecpp/Version6/OFrameStream.hh"
#include "framecpp/Version6/PTR_STRUCT.hh"

using namespace FrameCPP::Version_6;

namespace {
  class MapSummaryRef
    : public IFrameStream::MapRef< FrameCPP::Version_3_4_5::Summary, FrSummary >
  {
  public:
    MapSummaryRef( IFrameStream& Stream,
		 const FrameCPP::Version_3_4_5::Summary* Summary5,
		 FrSummary& Summary6 )
      : IFrameStream::MapRef< FrameCPP::Version_3_4_5::Summary, FrSummary >( Stream, Summary5, Summary6 )
    {
    }

  protected:
   
    // virtual
    void operator()( const void* Ref5Ptr,
                     const PTR_STRUCT& PtrStruct );

  }; // class - MapSummaryRef
   
   
  void MapSummaryRef::operator()( const void* Ref5Ptr,
			        const PTR_STRUCT& PtrStruct )
  {
      if ( Ref5Ptr == &(m_struct_5->refMoments( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefMoments( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refTable( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefTable( ), PtrStruct );
      }
   
      return;
  }   
} // namespace - anonymous


const CHAR* FrSummary::getStaticName()
{
   static const CHAR* class_name( "FrSummary" );
   return class_name;
}


FrSummary::
FrSummary( )
  : FrBase( CLASS_FR_SUMMARY )
{
}

FrSummary::
FrSummary( const FrSummary& Summary )
  : FrBase( CLASS_FR_SUMMARY )
{
  m_data.name = Summary.m_data.name;
  m_data.comment = Summary.m_data.comment;
  m_data.test = Summary.m_data.test;
  m_data.GTime = Summary.m_data.GTime;
  m_data.moments = Summary.m_data.moments;
  m_data.table = Summary.m_data.table;
}

FrSummary::
FrSummary( const std::string& name,
	   const std::string& comment,
	   const std::string& test,
	   const GPSTime& gtime )
  : FrBase( CLASS_FR_SUMMARY )
{
  m_data.name = name;
  m_data.comment = comment;
  m_data.test = test;
  m_data.GTime = gtime;
}


const std::string& FrSummary::GetName( ) const
{
   return m_data.name;
}


#if ENABLE_DEPRICATED_FRAMECPP
const std::string& FrSummary::getName( ) const
{
   return GetName( );
}
#endif


INT_8U FrSummary::
Bytes( ) const
{
  INT_8U	ret( FrBase::Bytes( ) );

  ret +=
    m_data.name.Bytes( ) +
    m_data.comment.Bytes( ) +
    m_data.test.Bytes( ) +
    FrameCPP::Version_6::Bytes( m_data.GTime ) +
    PTR_STRUCT::Bytes( ) +	// moments
    PTR_STRUCT::Bytes( ) +	// table
    PTR_STRUCT::Bytes( )	// next
    ;
  return ret;
}

FrSummary& FrSummary::
Merge( const FrSummary& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += getStaticName( );

  throw std::domain_error( msg );
  return *this;
}

FrSummary* FrSummary::
ReadDynamic( IFrameStream& Stream, const FrBase& Base )
{
  std::auto_ptr< FrSummary > ret( new FrSummary );
  ret->read( Stream );
  ret->SetInstance( Base.GetInstance( ) );
  return ret.release( );
}

void FrSummary::
Write( OFrameStream& Stream ) const
{
  Stream << m_data.name
	 << m_data.comment
	 << m_data.test
	 << m_data.GTime
	 << m_data.moments.containerCast( )
	 << m_data.table.containerCast( )
	 << GetNext( )
    ;
}

void FrSummary::
WriteDescription( OFrameStream& Stream ) const
{
  Stream.WriteDictionary( FrSH( getStaticName(), CLASS_FR_SUMMARY,
				"Summary Data Structure" ) );

  Stream.WriteDictionary( FrSE( "name", "STRING" ) );
  Stream.WriteDictionary( FrSE( "comment", "STRING" ) );
  Stream.WriteDictionary( FrSE( "test", "STRING" ) );
  Stream.WriteDictionary( FrSE( "GTimeS", "INT_4U" ) );
  Stream.WriteDictionary( FrSE( "GTimeN", "INT_4U" ) );

  Stream.WriteDictionary( FrSE( "moments",
				PTR_STRUCT::Desc( FrVect::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "table",
				PTR_STRUCT::Desc( FrTable::getStaticName() ) ) );

  Stream.WriteDictionary( FrSE( "next", PTR_STRUCT::Desc( getStaticName() ) ) );

  return;
}

void FrSummary::
read( IFrameStream& Stream )
{
  using namespace FrameCPP::Version_3_4_5;

  std::auto_ptr< Summary >	ver5;
  PTR_STRUCT			next;
      
  switch( Stream.GetVersion( ) )
  {
  case DATA_FORMAT_VERSION:
    {
      Stream >> m_data.name
	     >> m_data.comment
	     >> m_data.test
	     >> m_data.GTime
	     >> m_data.moments.containerCast( )
	     >> m_data.table.containerCast( )
	     >> next
	;
    }
    break;
  case 5:
  case 4:
    ver5.reset( Summary::read( Stream.GetFrame5Reader( ) ) );
    next = Stream.GetNextNode5( );
    break;
  }
  if ( ver5.get( ) )
  {
    m_data.name = ver5->getName( );
    m_data.comment = ver5->getComment( );
    m_data.comment = ver5->getTest( );
    m_data.GTime =
      General::GPSTime( ver5->getGTime( ).getSec( ),
			ver5->getGTime( ).getNSec( ) );

    MapSummaryRef	map_refs( Stream, ver5.get( ), *this );
    map_refs.Map();
  }
  //---------------------------------------------------------------------
  // Add the reference to the next
  //---------------------------------------------------------------------
  SetNext( next );
}
