#ifndef FrameCPP_VERSION_6_FrSummary_HH
#define FrameCPP_VERSION_6_FrSummary_HH

#include <vector>

#include "framecpp/Version6/SearchContainer.hh"
#include "framecpp/Version6/FrBase.hh"
#include "framecpp/Version6/FrCommon.hh"
#include "framecpp/Version6/FrTable.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    class FrSummary: public FrBase
    {
    public:
      struct Param_type {
	REAL_4	Param;
	STRING	ParamName;
      };

      typedef std::vector< Param_type >	ParamList_type;

      typedef SearchContainer< FrVect, &FrVect::GetName >	moments_type;
      typedef SearchContainer< FrTable, &FrTable::GetName >	table_type;

      typedef moments_type::iterator iterator;
      typedef moments_type::const_iterator const_iterator;
      typedef table_type::iterator table_iterator;
      typedef table_type::const_iterator const_table_iterator;

      static const CHAR* getStaticName();

      //: Default constructor
      FrSummary( );
      FrSummary( const FrSummary& Summary );
      FrSummary( const std::string& name,
		 const std::string& comment,
		 const std::string& test,
		 const GPSTime& gtime );

      //: Number of bytes needed to write this structure
      INT_8U Bytes( ) const;

      const std::string& GetName( ) const;

      const std::string& GetComment() const
      {
        return m_data.comment;
      }
  
      const std::string& GetTest() const
      {
        return m_data.test;
      }
  
      const GPSTime& GetGTime() const
      {
        return m_data.GTime;
      }
  
      const moments_type& RefMoments( ) const
      {
        return m_data.moments;
      }
  
      moments_type& RefMoments( )
      {
        return m_data.moments;
      }
  
      const table_type& RefTable( ) const
      {
        return m_data.table;
      }
  
      table_type& RefTable( )
      {
        return m_data.table;
      }   
   
      //: Merge with another FrAdcData
      FrSummary& Merge( const FrSummary& RHS );

      //: Read the structure from stream
      static FrSummary* ReadDynamic( IFrameStream& Stream,
				  const FrBase& Base );

      //: Write the structure to the stream
      virtual void Write( OFrameStream& Stream ) const;

      //: Write the structure description to the stream
      virtual void WriteDescription( OFrameStream& Stream ) const;

    private:
      struct {
	STRING		name;
	STRING		comment;
	STRING		test;
	GPSTime		GTime;
	moments_type	moments;
	table_type		table;
      } m_data;

      void read( IFrameStream& Stream );

#if ENABLE_DEPRICATED_FRAMECPP
    public:
      typedef moments_type MomentsContainer;
      typedef table_type TableContainer;

      const std::string& getName( ) const;

      const std::string& getComment() const
      {
	return GetComment( );
      }
      const std::string& getTest( ) const
      {
	return GetTest( );
      }
      const GPSTime& getGTime( ) const
      { 
	return GetGTime( );
      }

      const moments_type&	refMoments( ) const
      {
	return RefMoments( );
      }
      moments_type&	refMoments( )
      {
	return RefMoments( );
      }
      const table_type&	refTable( ) const
      {
	return RefTable( );
      }
      table_type&		refTable( )
      {
	return RefTable( );
      }
#endif
    };
  } // namespace - Version_6
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_6_FrSummary_HH */
