#include "framecpp/config.h"

#include "general/types.hh"

//#include "framecpp/Version3_4_5/SearchContainer.icc"

#include "framecpp/Version3_4_5/frame.hh"
#include "framecpp/Version3_4_5/searchcontainer.hh"
//#include "framecpp/Version3_4_5/SearchContainer.icc"

#include "framecpp/Version6/FrCommon.hh"
#include "framecpp/Version6/Frame5ReaderBase.hh"
#include "framecpp/Version6/FrameH.hh"
#include "framecpp/Version6/FrSE.hh"
#include "framecpp/Version6/FrSH.hh"
#include "framecpp/Version6/IFrameStream.hh"
#include "framecpp/Version6/OFrameStream.hh"
#include "framecpp/Version6/Util.hh"

using namespace FrameCPP::Version_6;

namespace {
  class MapFrameRef
    : public IFrameStream::MapRef< FrameCPP::Version_3_4_5::Frame, FrameH >
  {
  public:
    MapFrameRef( IFrameStream& Stream,
		 const FrameCPP::Version_3_4_5::Frame* Frame5,
		 FrameH& Frame6 )
      : IFrameStream::MapRef< FrameCPP::Version_3_4_5::Frame, FrameH >
    ( Stream, Frame5, Frame6 )
    {
    }

  protected:
   
    // virtual
    void operator()( const void* Ref5Ptr,
                     const PTR_STRUCT& PtrStruct );
   
  }; // class - MapFrameRef
   
   
  void MapFrameRef::operator()( const void* Ref5Ptr,
			        const PTR_STRUCT& PtrStruct )
  {
      if ( Ref5Ptr == &(m_struct_5->refType( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefType( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refUser( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefUser( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refDetectorSim( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefDetectSim( ), PtrStruct );
      }
      else if ( Ref5Ptr == &(m_struct_5->refDetectorProc( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefDetectProc( ), PtrStruct );
      }
      else if ( Ref5Ptr == &( m_struct_5->refHistory( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefHistory( ), PtrStruct );
      }
      else if ( Ref5Ptr == &( m_struct_5->refProcData( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefProcData( ), PtrStruct );
      }
      else if ( Ref5Ptr == &( m_struct_5->refSimData( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefSimData( ), PtrStruct );
      }
      else  if ( Ref5Ptr == &(m_struct_5->refEvent( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefEvent( ), PtrStruct );
      }
      else  if ( Ref5Ptr == &(m_struct_5->refSimEvent( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefSimEvent( ), PtrStruct );
      }
      else  if ( Ref5Ptr == &(m_struct_5->refSummary( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefSummaryData( ), PtrStruct );
      }
      else  if ( Ref5Ptr == &(m_struct_5->refAux( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefAuxData( ), PtrStruct );
      }
      else  if ( Ref5Ptr == &(m_struct_5->refTable( ) ) )
      {
	m_stream.AddContainer( m_struct_6.RefAuxTable( ), PtrStruct );
      }
   
      return;
  }   
} // namespace - anonymous


const CHAR* FrameH::getStaticName()
{
   static const CHAR* class_name( "FrameH" );
   return class_name;
}


#define TRACE_MEMORY 0
#if TRACE_MEMORY
#define MEM_ALLOCATE() std::cerr << "MEMORY: Allocate: " << FrameH::getStaticName() << " " << (void*)this << std::endl;
#define MEM_DELETE() std::cerr << "MEMORY: Delete: " << FrameH::getStaticName() << " " << (void*)this << std::endl;
#else
#define MEM_ALLOCATE()
#define MEM_DELETE()
#endif

#define	LM_DEBUG 0

#if LM_DEBUG
#define	AT() std::cerr << __FILE__ << " " << __LINE__ << std::endl;
#else
#define	AT()
#endif

FrameH::
FrameH( )
  : FrBase( CLASS_FRAME_H ),
    m_owns_raw_data( true )
{
  MEM_ALLOCATE();
  m_data.rawData = (FrRawData*)NULL;
  m_data.dt = 0;
}

FrameH::
FrameH( const FrameH& frame,
	detector_copy_type	Copy )
  : FrBase( CLASS_FRAME_H ),
    m_owns_raw_data( frame.m_owns_raw_data )
{
  AT( );
  MEM_ALLOCATE();
  m_data.rawData = (FrRawData*)NULL;
  m_data.name = frame.m_data.name;
  m_data.run = frame.m_data.run;
  m_data.frame = frame.m_data.frame;
  m_data.dataQuality = frame.m_data.dataQuality;
  m_data.GTime = frame.m_data.GTime;
  m_data.ULeapS = frame.m_data.ULeapS;
  m_data.dt = frame.m_data.dt;

  AT( );
  m_data.type = frame.m_data.type;
  AT( );
  m_data.user = frame.m_data.user;
  if ( Copy == SHALLOW_COPY_DETECTORS )
  {
    AT( );
    m_data.detectSim = frame.m_data.detectSim;
    AT( );
    m_data.detectProc = frame.m_data.detectProc;
  }
  else
  {
    for ( const_detectSim_iterator d( frame.m_data.detectSim.begin( ) );
	  d != frame.m_data.detectSim.end( );
	  d++ )
    {
      m_data.detectSim.append( new FrDetector( *(*d) ),
			       false, /* allocate */
			       true /* own */  );
    }
    for ( const_detectProc_iterator d( frame.m_data.detectProc.begin( ) );
	  d != frame.m_data.detectProc.end( );
	  d++ )
    {
      m_data.detectProc.append( new FrDetector( *(*d) ),
				false, /* allocate */
				true /* own */ );
    }
  }
  AT( );
  m_data.history = frame.m_data.history;
  try
  {
    if ( frame.m_data.rawData != NULL)
    {
      if ( m_owns_raw_data )
      {
	m_data.rawData = new FrRawData( *frame.m_data.rawData );
      }
      else
      {
	m_data.rawData = frame.m_data.rawData;
      }
    }
  }
  catch( ... )
  {
    AT( );
    if ( m_owns_raw_data )
    {
      delete m_data.rawData;
      m_data.rawData = (FrRawData*)NULL;
    }
    throw;
  }
  AT( );
  m_data.procData = frame.m_data.procData;
  AT( );
  m_data.simData = frame.m_data.simData;
  AT( );
  m_data.event = frame.m_data.event;
  AT( );
  m_data.simEvent = frame.m_data.simEvent;
  AT( );
  m_data.summaryData = frame.m_data.summaryData;
  AT( );
  m_data.auxData = frame.m_data.auxData;
  AT( );
  m_data.auxTable = frame.m_data.auxTable;
}

FrameH::
FrameH( const std::string& name,
	INT_4S run,
	INT_4U frame,
	const GPSTime& time,
	INT_2U uLeapS,
	const REAL_8& dt,
	INT_4U dqual )
  : FrBase( CLASS_FRAME_H ),
    m_owns_raw_data( true )
{
  MEM_ALLOCATE();
  m_data.name = name;
  m_data.run = run;
  m_data.frame = frame;
  m_data.dataQuality = dqual;
  m_data.GTime = time;
  m_data.ULeapS = uLeapS;
  m_data.dt = dt;
  m_data.rawData = (FrRawData*)NULL;
}

FrameH::
~FrameH( )
{
  MEM_DELETE();
  if ( m_owns_raw_data )
  {
    delete m_data.rawData;
  }
  m_data.rawData = (FrRawData*)NULL;
}


const std::string& FrameH::GetName( ) const
{
   return m_data.name;
}


#if ENABLE_DEPRICATED_FRAMECPP
const std::string& FrameH::getName() const
{
   return GetName( );
}
#endif


INT_8U FrameH::
Bytes( ) const
{
  return FrBase::Bytes( ) +
    m_data.name.Bytes( ) +
    sizeof( m_data.run ) + //	run;
    sizeof( m_data.frame ) + //	frame;
    sizeof( m_data.dataQuality ) + //	dataQuality;
    sizeof( INT_4U ) + //	GTime Seconds;
    sizeof( INT_4U ) + //	GTime Nanoeconds;
    sizeof( m_data.ULeapS ) + //	ULeapS;
    sizeof( m_data.dt ) + //	dt;
    //-------------------------------------------------------------------
    ( PTR_STRUCT::Bytes( ) * 13 );	// All the pointer elements
    ;
}

FrameH* FrameH::
ReadDynamic( IFrameStream& Stream, const FrBase& Base )
{
  std::auto_ptr< FrameH > ret( new FrameH );
  ret->read( Stream );
  ret->SetInstance( Base.GetInstance( ) );
  return ret.release( );
}

void FrameH::
ReadSubset( IFrameStream& Stream, INT_4U ContainerSet )
{
  if ( ! ( ContainerSet & DETECT_SIM ) )
  {
    Stream.RemoveContainer( RefDetectSim( ) );
  }
  if ( ! ( ContainerSet & DETECT_PROC ) )
  {
    Stream.RemoveContainer( RefDetectProc( ) );
  }
  if ( ! ( ContainerSet & HISTORY ) )
  {
    Stream.RemoveContainer( RefHistory( ) );
  }
  Stream.RemoveContainer( RefType( ) );
  Stream.RemoveContainer( RefUser( ) );
  if ( ! ( ContainerSet & RAW_DATA ) )
  {
    void* rawData = &m_data.rawData;

    Stream.RemovePointer( (FrBase**)(rawData) );
  }
  Stream.RemoveContainer( RefProcData( ) );
  Stream.RemoveContainer( RefSimData( ) );
  Stream.RemoveContainer( RefEvent( ) );
  Stream.RemoveContainer( RefSimEvent( ) );
  Stream.RemoveContainer( RefSummaryData( ) );
  if ( ! ( ContainerSet & AUX_DATA ) )
  {
    Stream.RemoveContainer( RefAuxData( ) );
  }
  if ( ! ( ContainerSet & AUX_TABLE ) )
  {
    Stream.RemoveContainer( RefAuxTable( ) );
  }
}

void FrameH::
SetRawData( const FrRawData& r ) 
{
    if ( m_owns_raw_data )
    {
      delete m_data.rawData;
      m_data.rawData = (FrRawData*)NULL;
    }
    
    m_data.rawData = new FrRawData( r );
    m_owns_raw_data = true;
}

void FrameH::
SetRawData( const FrRawData* r ) 
{
    if ( r == m_data.rawData )
    {
      return;
    }
    if ( m_owns_raw_data )
    {
      delete m_data.rawData;
      m_data.rawData = (FrRawData*)NULL;
    }
    
    m_data.rawData = new FrRawData( *r );
    m_owns_raw_data = true;
}

void FrameH::
SetRawData( FrRawData* r, bool allocate, bool owns )
{
    if ( r == m_data.rawData )
    {
       return;
    }
    if ( m_owns_raw_data )
    {
      delete m_data.rawData;
      m_data.rawData = (FrRawData*)NULL;
    }
    
    if ( allocate )
    {
      owns = true;
      m_data.rawData = new FrRawData( *r );
    }
    else
    {
      m_data.rawData = r;
    }

    m_owns_raw_data = owns;
}

void FrameH::
Write( OFrameStream& Stream ) const
{
  //---------------------------------------------------------------------
  // Check to see if per frame checksums are to be calculated
  //---------------------------------------------------------------------
  if ( Stream.IsCalculatingChecksumFrame( ) )
  {
    //-------------------------------------------------------------------
    // Yes ...
    // 1) Reset the buffer
    // 2) Add in the FrBase info
    //-------------------------------------------------------------------
    Stream.SetChecksumMethodFrame( );

    INT_8U	length = GetLength( );
    INT_2U	class_id = GetClassId( );
    INT_4U	instance = GetInstance( );

    Stream.calcChecksumFrame( &length, sizeof( length ) );
    Stream.calcChecksumFrame( &class_id, sizeof( class_id ) );
    Stream.calcChecksumFrame( &instance, sizeof( instance ) );
  }

  Stream << m_data.name
	 << m_data.run
	 << m_data.frame
	 << m_data.dataQuality
	 << m_data.GTime
	 << m_data.ULeapS
	 << m_data.dt
	 << m_data.type.containerCast( )
	 << m_data.user.containerCast( )
	 << m_data.detectSim.containerCast( )
	 << m_data.detectProc.containerCast( )
	 << m_data.history.containerCast( );
  Stream.WriteFrBasePtr( m_data.rawData );
  Stream << m_data.procData.containerCast( )
	 << m_data.simData.containerCast( )
	 << m_data.event.containerCast( )
	 << m_data.simEvent.containerCast( )
	 << m_data.summaryData.containerCast( )
	 << m_data.auxData.containerCast( )
	 << m_data.auxTable.containerCast( )
    ;
}

void FrameH::
WriteDescription( OFrameStream& Stream ) const
{
  Stream.WriteDictionary( FrSH( getStaticName(), CLASS_FRAME_H,
				"Frame Header Structure" ) );
  Stream.WriteDictionary
    ( FrSE( "name", "STRING",
	    "") );
  Stream.WriteDictionary
    ( FrSE( "run", "INT_4S",
	    "") );
  Stream.WriteDictionary
    ( FrSE( "frame", "INT_4U",
	    "") );
  Stream.WriteDictionary
    ( FrSE( "dataQuality", "INT_4U",
	    "") );
  Stream.WriteDictionary
    ( FrSE( "GTimeS", "INT_4U",
	    "") );
  Stream.WriteDictionary
    ( FrSE( "GTimeN", "INT_4U",
	    "") );
  Stream.WriteDictionary
    ( FrSE( "ULeapS", "INT_2U",
	    "") );
  Stream.WriteDictionary
    ( FrSE( "dt", "REAL_8",
	    "") );

  Stream.WriteDictionary( FrSE( "type",
				PTR_STRUCT::Desc( FrVect::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "user",
				PTR_STRUCT::Desc( FrVect::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "detectSim",
				PTR_STRUCT::Desc( FrDetector::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "detectProc",
				PTR_STRUCT::Desc( FrDetector::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "history",
				PTR_STRUCT::Desc(FrHistory::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "rawData", 
				PTR_STRUCT::Desc( FrRawData::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "procData",
				PTR_STRUCT::Desc( FrProcData::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "simData",
				PTR_STRUCT::Desc( FrSimData::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "event",
				PTR_STRUCT::Desc( FrEvent::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "simEvent",
				PTR_STRUCT::Desc( FrSimEvent::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "summaryData",
				PTR_STRUCT::Desc( FrSummary::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "auxData",
				PTR_STRUCT::Desc( FrVect::getStaticName() ) ) );
  Stream.WriteDictionary( FrSE( "auxTable",
				PTR_STRUCT::Desc( FrTable::getStaticName() ) ) );

}

void FrameH::
read( IFrameStream& Stream )
{
  using namespace FrameCPP::Version_3_4_5;

  std::auto_ptr< Frame > ver5;

  switch( Stream.GetVersion( ) )
  {
  case DATA_FORMAT_VERSION:
    {
      void* rawData = &(m_data.rawData);

      Stream >> m_data.name
	     >> m_data.run
	     >> m_data.frame
	     >> m_data.dataQuality
	     >> m_data.GTime
	     >> m_data.ULeapS
	     >> m_data.dt
	     >> m_data.type.containerCast( )
	     >> m_data.user.containerCast( )
	     >> m_data.detectSim.containerCast( )
	     >> m_data.detectProc.containerCast( )
	     >> m_data.history.containerCast( )
	     >> (FrBase**)rawData
	     >> m_data.procData.containerCast( )
	     >> m_data.simData.containerCast( )
	     >> m_data.event.containerCast( )
	     >> m_data.simEvent.containerCast( )
	     >> m_data.summaryData.containerCast( )
	     >> m_data.auxData.containerCast( )
	     >> m_data.auxTable.containerCast( )
	;
    }
    break;
  case 5:
    ver5.reset( Frame::read( Stream.GetFrame5Reader( ) ) );
    break;
  case 4:
    ver5.reset( Frame::read4( Stream.GetFrame5Reader( ) ) );
    break;
  case 3:
    ver5.reset( Frame::read3( Stream.GetFrame5Reader( ) ) );
    break;
  }
  if ( ver5.get( ) )
  {
    m_data.name = ver5->mName;
    m_data.run = ver5->mRun;
    m_data.frame = ver5->mFrame;
    m_data.dataQuality = ver5->mDataQuality;
    m_data.GTime = General::GPSTime( ver5->mGTime.getSec( ),
				ver5->mGTime.getNSec( ) );
    m_data.ULeapS = ver5->mULeapS;
    m_data.dt = ver5->mDt;

    MapFrameRef	map_refs( Stream, ver5.get( ), *this );
    map_refs.Map();

    //-------------------------------------------------------------------
    // Map the pointers
    //-------------------------------------------------------------------
    void* rawData5;
    void* rawData6;

    Frame5ReaderBase::Pointers&
      pointers( Stream.GetExposedFrame5Reader( ).mPointers );

    for ( Frame5ReaderBase::Pointers::iterator i( pointers.begin( ) );
	  i != pointers.end( );
	  i++ )
    {
      rawData5 = &(ver5->mRawData);

      if ( (*i).first == rawData5 )
      {
	rawData6 = &m_data.rawData;
	Stream.AddPointer( (FrBase**)(rawData6),
			   PTR_STRUCT( (*i).second.getClass( ),
				       (*i).second.getInstance( ) ) );
      }
    }
    pointers.resize( 0 );
  }
}

FrameH::m_data_type::
m_data_type( )
  : name( ),
    GTime( ),
    type( ),
    user( ),
    detectSim( ),
    detectProc( ),
    history( ),
    rawData( (FrRawData*)NULL ),
    procData( false ),
    simData( false ),
    event( false ),
    simEvent( false),
    summaryData( false ),
    auxData( ),
    auxTable( )
{
}

//-----------------------------------------------------------------------
// Instantiate SearchContainers used in this class
//-----------------------------------------------------------------------
