#ifndef FrameCPP_VERSION_6_FrameH_HH
#define FrameCPP_VERSION_6_FrameH_HH

#include "general/types.hh"

#include "framecpp/Version6/SearchContainer.hh"

#include "framecpp/Version6/FrBase.hh"
#include "framecpp/Version6/FrDetector.hh"
#include "framecpp/Version6/FrEvent.hh"
#include "framecpp/Version6/FrHistory.hh"
#include "framecpp/Version6/FrProcData.hh"
#include "framecpp/Version6/FrRawData.hh"
#include "framecpp/Version6/FrSimData.hh"
#include "framecpp/Version6/FrSimEvent.hh"
#include "framecpp/Version6/FrSummary.hh"
#include "framecpp/Version6/FrTable.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/GPSTime.hh"
#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    class FrameH: public FrBase
    {
    public:
      enum
      {
	DETECT_SIM	= 0x0001,
	DETECT_PROC	= 0x0002,
	HISTORY		= 0x0004,
	RAW_DATA	= 0x0008,
	AUX_DATA	= 0x0010,
	AUX_TABLE	= 0x0020
      };

      typedef enum
      {
	DEEP_COPY_DETECTORS,
	SHALLOW_COPY_DETECTORS
      } detector_copy_type;

      typedef int dummy_type;

      typedef SearchContainer< FrVect, &FrVect::GetName >	type_type;
      typedef SearchContainer< FrVect, &FrVect::GetName  >	user_type;
      typedef SearchContainer< FrDetector, &FrDetector::GetName >
              detectSim_type;
      typedef SearchContainer< FrDetector, &FrDetector::GetName >
              detectProc_type;
      typedef SearchContainer< FrHistory, &FrHistory::GetName >	history_type;
      typedef SearchContainer< FrProcData, &FrProcData::GetName > procData_type;
      typedef SearchContainer< FrSimData, &FrSimData::GetName >	simData_type;
      typedef SearchContainer< FrEvent, &FrEvent::GetName >	event_type;
      typedef SearchContainer< FrSimEvent, &FrSimEvent::GetName >
              simEvent_type;
      typedef SearchContainer< FrSummary, &FrSummary::GetName >	summaryData_type;
      typedef SearchContainer< FrVect, &FrVect::GetName  >	auxData_type;
      typedef SearchContainer< FrTable, &FrTable::GetName  >	auxTable_type;

      typedef  type_type::iterator type_iterator;
      typedef  type_type::const_iterator const_type_iterator;
      typedef  user_type::iterator user_iterator;
      typedef  user_type::const_iterator const_user_iterator;
      typedef  detectSim_type::iterator detectSim_iterator;
      typedef  detectSim_type::const_iterator const_detectSim_iterator;
      typedef  detectProc_type::iterator detectProc_iterator;
      typedef  detectProc_type::const_iterator const_detectProc_iterator;
      typedef  history_type::iterator history_iterator;
      typedef  history_type::const_iterator const_history_iterator;
      typedef  procData_type::iterator procData_iterator;
      typedef  procData_type::const_iterator const_procData_iterator;
      typedef  simData_type::iterator simData_iterator;
      typedef  simData_type::const_iterator const_simData_iterator;
      typedef  event_type::iterator event_iterator;
      typedef  event_type::const_iterator const_event_iterator;
      typedef  simEvent_type::iterator simEvent_iterator;
      typedef  simEvent_type::const_iterator const_simEvent_iterator;
      typedef  summaryData_type::iterator summaryData_iterator;
      typedef  summaryData_type::const_iterator const_summaryData_iterator;
      typedef  auxData_type::iterator auxData_iterator;
      typedef  auxData_type::const_iterator const_auxData_iterator;
      typedef  auxTable_type::iterator auxTable_iterator;
      typedef  auxTable_type::const_iterator const_auxTable_iterator;

      typedef  type_type::hash_iterator type_hash_iterator;
      typedef  type_type::const_hash_iterator const_type_hash_iterator;
      typedef  user_type::hash_iterator user_hash_iterator;
      typedef  user_type::const_hash_iterator const_user_hash_iterator;
      typedef  detectSim_type::hash_iterator detectSim_hash_iterator;
      typedef  detectSim_type::const_hash_iterator const_detectSim_hash_iterator;
      typedef  detectProc_type::hash_iterator detectProc_hash_iterator;
      typedef  detectProc_type::const_hash_iterator const_detectProc_hash_iterator;
      typedef  history_type::hash_iterator history_hash_iterator;
      typedef  history_type::const_hash_iterator const_history_hash_iterator;
      typedef  procData_type::hash_iterator procData_hash_iterator;
      typedef  procData_type::const_hash_iterator const_procData_hash_iterator;
      typedef  simData_type::hash_iterator simData_hash_iterator;
      typedef  simData_type::const_hash_iterator const_simData_hash_iterator;
      typedef  event_type::hash_iterator event_hash_iterator;
      typedef  event_type::const_hash_iterator const_event_hash_iterator;
      typedef  simEvent_type::hash_iterator simEvent_hash_iterator;
      typedef  simEvent_type::const_hash_iterator const_simEvent_hash_iterator;
      typedef  summaryData_type::hash_iterator summaryData_hash_iterator;
      typedef  summaryData_type::const_hash_iterator const_summaryData_hash_iterator;
      typedef  auxData_type::hash_iterator auxData_hash_iterator;
      typedef  auxData_type::const_hash_iterator const_auxData_hash_iterator;
      typedef  auxTable_type::hash_iterator auxTable_hash_iterator;
      typedef  auxTable_type::const_hash_iterator const_auxTable_hash_iterator;

      static const CHAR* getStaticName();

      //: Default constructor
      FrameH( );
      FrameH( const FrameH& frame,
	      detector_copy_type Copy = SHALLOW_COPY_DETECTORS );
      FrameH( const std::string& name,
	      INT_4S run,
	      INT_4U frame,
	      const GPSTime& time,
	      INT_2U uLeapS,
	      const REAL_8& dt,
	      INT_4U dqual = 0 );
      ~FrameH( );

      //: Number of bytes needed to write this structure
      INT_8U Bytes( ) const;

      const std::string& GetName( ) const;
  
      INT_4S GetRun( ) const
      {
        return m_data.run;
      }
  
      INT_4U GetFrame( ) const
      {
        return m_data.frame;
      }
  
      INT_4U GetDataQuality( ) const
      {
        return m_data.dataQuality;
      }
  
      const GPSTime& GetGTime( ) const
      {
        return m_data.GTime;
      }
  
      INT_2U GetULeapS( ) const
      {
        return m_data.ULeapS;
      }
  
      REAL_8 GetDt( ) const
      {
        return m_data.dt;
      }
  
      const FrRawData* GetRawData( ) const
      {
        return m_data.rawData;
      }
  
      FrRawData* GetRawData( )
      {
        return m_data.rawData;
      }
  
      const type_type& RefType( ) const
      {
        return m_data.type;
      }
  
      type_type& RefType( )
      {
        return m_data.type;
      }
  
      const user_type& RefUser( ) const
      {
        return m_data.user;
      }
  
      user_type& RefUser( )
      {
        return m_data.user;
      }
  
      const detectSim_type& RefDetectSim( ) const
      {
        return m_data.detectSim;
      }
  
      detectSim_type& RefDetectSim( )
      {
        return m_data.detectSim;
      }
  
      const detectProc_type& RefDetectProc( ) const
      {
        return m_data.detectProc;
      }
  
      detectProc_type& RefDetectProc( )
      {
        return m_data.detectProc;
      }
  
      const history_type& RefHistory( ) const
      {
        return m_data.history;
      }
  
      history_type& RefHistory( )
      {
        return m_data.history;
      }
  
      const procData_type& RefProcData( ) const
      {
        return m_data.procData;
      }
  
      procData_type& RefProcData( )
      {
        return m_data.procData;
      }
  
      const simData_type& RefSimData( ) const
      {
        return m_data.simData;
      }
  
      simData_type& RefSimData( )
      {
        return m_data.simData;
      }
  
      const event_type& RefEvent( ) const
      {
        return m_data.event;
      }
  
      event_type& RefEvent( )
      {
        return m_data.event;
      }
  
      const simEvent_type& RefSimEvent( ) const
      {
        return m_data.simEvent;
      }
  
      simEvent_type& RefSimEvent( )
      {
        return m_data.simEvent;
      }
  
      const summaryData_type& RefSummaryData( ) const
      {
        return m_data.summaryData;
      }
  
      summaryData_type& RefSummaryData( )
      {
        return m_data.summaryData;
      }
  
      const auxData_type& RefAuxData( ) const
      {
        return m_data.auxData;
      }
  
      auxData_type& RefAuxData( )
      {
        return m_data.auxData;
      }
  
      const auxTable_type& RefAuxTable( ) const
      {
        return m_data.auxTable;
      }
  
      auxTable_type& RefAuxTable( )
      {
        return m_data.auxTable;
      }
  
      void
      SetRun( INT_4S Run )
      {
	m_data.run = Run;
      }

      void SetDataQuality( INT_4U DataQuality )
      {
        m_data.dataQuality = DataQuality;
      }
  
      void SetDt( REAL_8 Dt )
      {
        m_data.dt = Dt;
      }
  
      void SetGTime( const GPSTime& GTime )
      {
        m_data.GTime = GTime;
      }   
   
      void SetRawData( const FrRawData& r );
      void SetRawData( const FrRawData* r );
      void SetRawData( FrRawData* r, bool allocate, bool owns );

      //: Read the structure from stream
      static FrameH* ReadDynamic( IFrameStream& Stream,
				  const FrBase& Base );

      void ReadSubset( IFrameStream& Stream, INT_4U ContainerSet );

      //: Write the structure to the stream
      virtual void Write( OFrameStream& Stream ) const;

      //: Write the structure description to the stream
      virtual void WriteDescription( OFrameStream& Stream ) const;

    private:
      friend class IFrameStream;

      struct m_data_type {
	STRING	name;
	INT_4S	run;
	INT_4U	frame;
	INT_4U	dataQuality;
	GPSTime	GTime;
	INT_2U	ULeapS;
	REAL_8	dt;
	//---------------------------------------------------------------
	type_type		type;
	user_type		user;
	detectSim_type	detectSim;
	detectProc_type	detectProc;
	mutable history_type	history;
	FrRawData*	rawData;
	procData_type	procData;
	simData_type	simData;
	event_type		event;
	simEvent_type	simEvent;
	summaryData_type	summaryData;
	auxData_type	auxData;
	auxTable_type	auxTable;

	m_data_type( );
      } m_data;

      bool m_owns_raw_data;

      void read( IFrameStream& Stream );

#if ENABLE_DEPRICATED_FRAMECPP
    public:
      typedef type_type TypeContainer;
      typedef user_type UserContainer;
      typedef detectSim_type DetectorSimContainer;
      typedef detectProc_type DetectorProcContainer;
      typedef history_type HistoryContainer;
      typedef procData_type ProcDataContainer;
      typedef simData_type SimDataContainer;
      typedef event_type EventContainer;
      typedef simEvent_type SimEventContainer;
      typedef summaryData_type SummaryContainer;
      typedef auxData_type AuxContainer;
      typedef auxTable_type TableContainer;

      typedef  detectSim_iterator detectorSim_iterator;
      typedef  const_detectSim_iterator const_detectorSim_iterator;
      typedef  detectProc_iterator detectorProc_iterator;
      typedef  const_detectProc_iterator const_detectorProc_iterator;
      typedef  summaryData_iterator summary_iterator;
      typedef  const_summaryData_iterator const_summary_iterator;
      typedef  auxData_iterator aux_iterator;
      typedef  const_auxData_iterator const_aux_iterator;
      typedef  auxTable_iterator table_iterator;
      typedef  const_auxTable_iterator const_table_iterator;

      typedef  detectProc_hash_iterator detectorProc_hash_iterator;
      typedef  const_detectProc_hash_iterator const_detectorProc_hash_iterator;
      typedef  detectSim_hash_iterator detectorSim_hash_iterator;
      typedef  const_detectSim_hash_iterator const_detectorSim_hash_iterator;
      typedef  summaryData_hash_iterator summary_hash_iterator;
      typedef  const_summaryData_hash_iterator const_summary_hash_iterator;

      const std::string& getName() const;
      INT_4S getRun() const { return GetRun( ); }
      INT_4U getFrame() const { return GetFrame( ); }
      INT_4U getDataQuality() const { return GetDataQuality( ); }
      const GPSTime& getGTime() const { return GetGTime( ); }
      INT_2U getULeapS() const { return GetULeapS( ); }
      REAL_8 getDt() const { return GetDt( ); }
      // bool isRawDataOwned() const;

      FrRawData* getRawData() { return GetRawData( ); }
      const FrRawData* getRawData() const { return GetRawData( ); }

      TypeContainer& refType() { return RefType( ); }
      const TypeContainer& refType() const { return RefType( ); }
      UserContainer& refUser() { return RefUser( ); }
      const UserContainer& refUser() const { return RefUser( ); }
      DetectorSimContainer& refDetectorSim() { return RefDetectSim( ); }
      const DetectorSimContainer& refDetectorSim() const { return RefDetectSim( ); }
       DetectorProcContainer& refDetectorProc() { return RefDetectProc( ); }
      const DetectorProcContainer& refDetectorProc() const { return RefDetectProc( ); }
      HistoryContainer& refHistory() { return RefHistory( ); }
      const HistoryContainer& refHistory() const { return RefHistory( ); }
      ProcDataContainer& refProcData() { return RefProcData( ); }
      const ProcDataContainer& refProcData() const { return RefProcData( ); }
      SimDataContainer& refSimData() { return RefSimData( ); }
      const SimDataContainer& refSimData() const { return RefSimData( ); }
      EventContainer& refEvent() { return RefEvent( ); }
      const EventContainer& refEvent() const { return RefEvent( ); }
      SimEventContainer& refSimEvent() { return RefSimEvent( ); }
      const SimEventContainer& refSimEvent() const { return RefSimEvent( ); }
      SummaryContainer& refSummary() { return RefSummaryData( ); }
      const SummaryContainer& refSummary() const { return RefSummaryData( ); }
      AuxContainer& refAux() { return RefAuxData( ); }
      const AuxContainer& refAux() const { return RefAuxData( ); }
      TableContainer& refTable() { return RefAuxTable( ); }
      const TableContainer& refTable() const { return RefAuxTable( ); }

      void setDt( REAL_8 Dt ) { SetDt( Dt ); };

      void setRawData( const FrRawData& r ) { SetRawData( r ); }
      void setRawData( const FrRawData* r ) { SetRawData( r ); }
      void setRawData( FrRawData* r, bool allocate = true , bool owns = true ) { SetRawData( r, allocate, owns ); }
#endif
    };

  } // namespace - Version_6
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_6_FrameH_HH */
