#ifndef FrameCPP_VERSION_6_OFrameStream_HH
#define FrameCPP_VERSION_6_OFrameStream_HH

#include <list>
#include <memory>
#include <string>
#include <vector>

#include "general/types.hh"
#include "general/hash_map.hh"

#include "framecpp/CheckSum.hh"

#include "framecpp/Common/OFrameStream.hh"

#include "framecpp/Version6/FrameStreamBase.hh"
#include "framecpp/Version6/Container.hh"
#include "framecpp/Version6/Dictionary.hh"
#include "framecpp/Version6/FrameH.hh"
#include "framecpp/Version6/FrEndOfFile.hh"
#include "framecpp/Version6/FrEndOfFrame.hh"
#include "framecpp/Version6/FrHeader.hh"
#include "framecpp/Version6/FrTOC.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/SearchContainer.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    class FrBase;
    class FrStatData;
    class Frame5ReaderBase;
    class FrameH;

    //: Class to handle frame I/O.
    class OFrameStream
      : FrameStreamBase,
	public Common::OFrameStream
    {
    public:
      static const std::string DEFAULT_ORIGINATOR;

      //: Constructor
      //
      //!param: std::iostream& Stream - Input or Output stream
      OFrameStream( std::ostream& Stream,
		    const std::vector< Dictionary::io_calls_type >& LibraryDefs,
		    INT_4U NumberOfClassTypes,
		    CheckSum::kind_type CheckSumMethod = CheckSum::CRC ,
		    const std::string& Originator = DEFAULT_ORIGINATOR );

      OFrameStream( std::ostream& Stream,
		    CheckSum::kind_type CheckSumMethod = CheckSum::CRC,
		    const std::string& Originator = DEFAULT_ORIGINATOR );
      //: Destructor
      //
      ~OFrameStream( );

      //: Write data to the stream
      template< class T >
      OFrameStream& operator<<( const T* Data );

      //: Write a vector of data to the stream
      template< class T >
      OFrameStream& operator<<( const std::vector< T >& Data );

      //: Write data to the stream
      template< class T >
      OFrameStream& operator<<( const T& Data );
      //: Write container to the stream
      template< class T >
      OFrameStream& operator<<( Container< T >& Data );

      //: Write container to the stream
      template< class T >
      OFrameStream& operator<<( const Container< T >& Data );

      //: Append an object to the current stream
      void AppendObject( FrBase& Object );

      // Write an object to the stream
      //
      //!param: FrBase& Object - Object to be written. The instance value
      //+ will be adjusted appropriately.
      void write( FrBase& Object );

      void write( const void*Buffer, INT_8U Length );

      void WriteDictionary( FrBase& Base );

      void WriteDictionary( const FrBase& Base );

      void WriteFrame( FrameH& Frame,
		       CheckSum::kind_type CheckSumMethod = CheckSum::NONE );

      void WriteFrame( FrameH& Frame,
		       FrVect::compression_scheme_type Method,
		       int Level,
		       CheckSum::kind_type CheckSumMethod = CheckSum::NONE );

      void WriteFrBasePtr( const FrBase* Object );
      void WriteFrStatData( FrStatData& StatData );

      void WriteObject( FrBase& Base,
			bool Compress = false,
			FrVect::compression_scheme_type Method = FrVect::RAW,
			int Level = 0);

      void close( );

    private:
      friend void FrameH::Write( OFrameStream& Stream ) const;
      friend void FrEndOfFile::Write( OFrameStream& Stream ) const;
      friend void FrEndOfFrame::Write( OFrameStream& Stream ) const;
      friend void FrHeader::write( OFrameStream& fs );

      typedef std::list< FrStatData* >			fr_stat_list_type;
      typedef std::map< FrBase*, fr_stat_list_type >	fr_stat_map_type;
      typedef std::list< FrBase* > 			print_stack_type;

      // Table of contents to use.
      // When reading the stream, the TOC is consulted for object offsets.
      // When writing, the TOC is generated as objects are written and
      // written to file when the FrameStream is delete or closed.
      FrTOC		m_toc;

      std::vector< Dictionary::callback_key_type >	m_active_callbacks;
      std::vector< Dictionary::Callback* > 	m_callback_methods;
      Dictionary::Callback*			m_instantiate_cb;

      print_stack_type				m_print_stack;
      print_stack_type::iterator			m_print_stack_insertion;

      fr_stat_map_type		m_fr_stat_map;

      OFrameStream( const OFrameStream& );
      const OFrameStream& operator=( const OFrameStream& );

      void init( INT_4U NumberOfClassTypes,
		 CheckSum::kind_type CheckSumMethod,
		 const std::string& Originator );
    }; // class - FrameStream

    inline void OFrameStream::
    write( const void* Buffer, INT_8U Length )
    {
      Common::OFrameStream::write( Buffer, Length );
    }
  } // namespace - Version_6
} // namespace - FrameCPP

#endif /* FrameCPP_VERSION_6_OFrameStream_HH */
