#ifndef FrameCPP_VERSION_6_Reference_HH
#define FrameCPP_VERSION_6_Reference_HH

#include "general/hash_map.hh"
#include "general/types.hh"

#include "framecpp/ContainerBasic.hh"

#include "framecpp/Version6/PTR_STRUCT.hh"

#include <vector>

namespace FrameCPP
{
  namespace Version_6
  {
    class IFrameStream;
    class FrBase;
    class Dictionary;

    class Reference
    {
    public:
      Reference( INT_4U ClassCount, const Dictionary& Dictionary );
      ~Reference( );

      template< class T >
      void AddContainer( FrameCPP::ContainerBasic< T >& C,
			 const PTR_STRUCT& Ref );

      template< class T >
      void RemoveContainer( FrameCPP::ContainerBasic< T >& C );

      void AddNextRef( const PTR_STRUCT& PtrStruct );

      void AddObject( FrBase* Object );

      void AddPointer( FrBase** Pointer, const PTR_STRUCT& Ref );

      PTR_STRUCT GetReferenceToPointer( const FrBase** Pointer ) const;

      bool GetRecursiveReference( PTR_STRUCT& Reference );

      bool IsRecursiveObject( const FrBase& Base ) const;

      void RemovePointer( FrBase** Pointer );

      void RemoveReference( const PTR_STRUCT& Object );

      void Reset( bool PurgeDetectors = false );

      void ResolveLinks( const IFrameStream& Stream );

      void SetFrameVersion( INT_4U FrameVersion );

      void SetFrameLibraryRevision( INT_4U Revision );

      void SetRecursiveMode( bool Mode );

      void SetTracking( bool Tracking );

      void StartNewObject( );

    private:
    
      //: No copy constructor
      Reference( const Reference& );
   
      //: No assignment operator
      Reference& operator=( const Reference& );
   
      class private_type;
      const Dictionary& m_dictionary;
      INT_4U		m_class_count;

      private_type*	p;
    };
  } // namespace - Version_6
} // namespace - FrameCPP

#endif /* FrameCPP_VERSION_6_Reference_HH */
