#ifndef FRAMECPP_VERSION_6__STRING_CC
#define FRAMECPP_VERSION_6__STRING_CC

#include "framecpp/config.h"

#include <vector>

#include "framecpp/Version6/IFrameStream.hh"
#include "framecpp/Version6/OFrameStream.hh"
#include "framecpp/Version6/STRING.hh"

#define LM_DEBUG 0

using namespace FrameCPP::Version_6;

const INT_4U STRING::MAX_STRING_LENGTH =
( 1 << ( 8 * sizeof( STRING::length_of_string_type ) ) );

STRING::
STRING( )
   : std::string()
{
}

STRING::
STRING( const std::string& Value )
{
  length_error( Value );
  std::string::operator=( Value );
}

STRING::
STRING( const char* Value )
{
  length_error( Value );
  std::string::operator=( Value );
}

STRING& STRING::
operator=( const std::string& Source )
{
  length_error( Source );
  std::string::operator=( Source );
  return *this;
}

STRING& STRING::
operator=( const char* Source )
{
  length_error( Source );
  std::string::operator=( Source );
  return *this;
}

   
INT_8U STRING::Bytes( ) const
{
   return length( ) + 1 + sizeof( length_of_string_type );
}

INT_8U STRING::Bytes( const std::string& String )
{
   return String.length( ) + 1 + sizeof( length_of_string_type );
}   

void STRING::
length_error( const std::string& Buffer )
{
  if ( Buffer.length( ) > MAX_STRING_LENGTH )
  {
    std::ostringstream msg;

    msg << "maximum size of STRING exceeded: "
	<< Buffer.length( ) << " > " << MAX_STRING_LENGTH
      ;
    throw std::length_error( msg.str( ) );
  }
}

void STRING::
length_error( const char* Buffer )
{
  if ( strlen( Buffer ) > MAX_STRING_LENGTH )
  {
    std::ostringstream msg;

    msg << "maximum size of STRING exceeded: "
	<< strlen( Buffer ) << " > " << MAX_STRING_LENGTH
      ;
    throw std::length_error( msg.str( ) );
  }
}
#endif /* FRAMECPP_VERSION_6__STRING_CC */
