#include <sstream>

#include "ldas/ldasconfig.hh"

#include "util.hh"
#include "framecpp/FrameCPP.hh"
#include "framecpp/Version3_4_5/types.hh"
#include "framecpp/Version6/FrCommon.hh"


//-----------------------------------------------------------------------------
//!return: std::string - The string representation of the version of frameCPP
std::string FrameCPP::
GetVersion( )
{
  std::ostringstream	oss;

  oss << "frameCPP-" << LDAS_VERSION;

  return oss.str( );
}

//-----------------------------------------------------------------------------
// The time information that is returned by this function is based on when the
// top level configure.in ( ldas/configure.in) file was committed to CVS.
// The time value is in UTC time as that is what CVS uses internally.
//
//!return: std::string - The string representation of the cvs date
std::string FrameCPP::
GetCVSDate( )
{
  return LDAS_CVS_DATE;
}

//-----------------------------------------------------------------------------
//!return: std::string - The string representation of when the library was
//+ built
std::string FrameCPP::
GetBuildDate( )
{
  return LDAS_BUILD_DATE;
}

//-----------------------------------------------------------------------------
//!return: INT_2U - current version of frame spec.
INT_2U FrameCPP::
GetDataFormatVersion( )
{
  return FRAME_SPEC_CURRENT;
}

//!return: INT_2U - current minor version for the frame library
INT_2U FrameCPP::
GetFrameLibraryMinorVersion( )
{
#if FRAME_SPEC_CURRENT == 6
  return FrameCPP::Version_6::LIBRARY_MINOR_VERSION;
#else
#error "Unsupported frame spec: FRAME_SPEC_CURRENT"
#endif
}

//-----------------------------------------------------------------------------
//
//: Reverses the order of one byte.
//
// This is a template specialization which does nothing.
//
//!param: unsigned char* dest - Pointer to the start of the data.
//!param: unsignet int n - Number of data points.
//
//!exc: None.   
//   
template<>
void FrameCPP::Version_3_4_5::reverse< 1 >( void* dest, unsigned int n ) 
{
    return;
}


//-----------------------------------------------------------------------------
//
//: Reverses the order of two bytes.
//
// This is a template specialization.
//
//!param: unsigned char* dest - Pointer to the start of the data.
//!param: unsignet int n - Number of data points.
//
//!exc: None.   
//      
template<>
void FrameCPP::Version_3_4_5::reverse< 2 >( void* dest, unsigned int n ) 
{
    unsigned char* d = reinterpret_cast< unsigned char* >( dest );

    unsigned char tmp;
    for ( unsigned int i = n; i != 0; --i )
    {
        tmp = *d;
        *d = *( d + 1 );
        *( d + 1 ) = tmp;
        d += 2;
    }
}


//-----------------------------------------------------------------------------
//
//: Reverses the order of four bytes.
//
// This is a template specialization.
//
//!param: unsigned char* dest - Pointer to the start of the data.
//!param: unsignet int n - Number of data points.
//
//!exc: None.   
//      
template<>
void FrameCPP::Version_3_4_5::reverse< 4 >( void* dest, unsigned int n ) 
{
    unsigned char* d = reinterpret_cast< unsigned char* >( dest );

    unsigned char tmp;
    for ( unsigned int i = n; i != 0; --i )
    {
        tmp = *d;
        *d = *( d + 3 );
        *( d + 3 ) = tmp;
        tmp = *( d + 1 );
        *( d + 1 ) = *( d + 2 );
        *( d + 2 ) = tmp;
        d += 4;
    }
}


//-----------------------------------------------------------------------------
//
//: Reverses the order of eight bytes.
//
// This is a template specialization.
//
//!param: unsigned char* dest - Pointer to the start of the data.
//!param: unsignet int n - Number of data points.
//
//!exc: None.   
//      
template<>
void FrameCPP::Version_3_4_5::reverse< 8 >( void* dest, unsigned int n ) 
{
    unsigned char* start( reinterpret_cast< unsigned char* >( dest ) );
    unsigned char* end( reinterpret_cast< unsigned char* >( dest ) + 7 );

    for ( unsigned int i = n; i != 0; --i )
    {
        unsigned char tmp = *start;
        *(start++) = *end;
        *(end--) = tmp;
        tmp = *start;
        *(start++) = *end;
        *(end--) = tmp;
        tmp = *start;
        *(start++) = *end;
        *(end--) = tmp;
        tmp = *start;
        *start = *end;
        *end = tmp;
        start += 5;
        end += 11;
    }
}
