#ifndef FrameCppUtilHH
#define FrameCppUtilHH

#include "general/types.hh"

namespace FrameCPP
{

  enum CODE {
    BAD_FRAME_FILE
  };
  //: Get the version of frameCPP
  std::string GetVersion( );

  //: Get the CVS date for this version of frameCPP
  std::string GetCVSDate( );

  //: Get the date when frameCPP was built.
  std::string GetBuildDate( );

  //: Return the data format version
  INT_2U GetDataFormatVersion( );

  //: Return the minor version of the frame library
  INT_2U GetFrameLibraryMinorVersion( );

namespace Version_3_4_5 {

    //-----------------------------------------------------------------------------
    //
    //: Byte swap data which size is SIZE.
    //
    // Template function.
    //
    //!param: unsigned char* s - Pointer to start of the data.
    //!param: unsignet int n - Number of data points.
    //
    //!exc: None.
    template< unsigned int SIZE > void reverse( void* s, unsigned int n );
    
    // Template specialization declarations
    //-----------------------------------------------------------------------------
    //
    //: Reverses the order of one byte.
    //
    // This is a template specialization which does nothing.
    //
    //!param: unsigned char* dest - Pointer to the start of the data.
    //!param: unsignet int n - Number of data points.
    //
    //!exc: None.   
    template<> void reverse< 1 >( void* s, unsigned int n );

    //-----------------------------------------------------------------------------
    //
    //: Reverses the order of two bytes.
    //
    // This is a template specialization.
    //
    //!param: unsigned char* dest - Pointer to the start of the data.
    //!param: unsignet int n - Number of data points.
    //
    //!exc: None.   
    template<> void reverse< 2 >( void* s, unsigned int n );


    //-----------------------------------------------------------------------------
    //
    //: Reverses the order of four bytes.
    //
    // This is a template specialization.
    //
    //!param: unsigned char* dest - Pointer to the start of the data.
    //!param: unsignet int n - Number of data points.
    //
    //!exc: None.   
    template<> void reverse< 4 >( void* s, unsigned int n );


    //-----------------------------------------------------------------------------
    //
    //: Reverses the order of eight bytes.
    //
    // This is a template specialization.
    //
    //!param: unsigned char* dest - Pointer to the start of the data.
    //!param: unsignet int n - Number of data points.
    //
    //!exc: None.   
    template<> void reverse< 8 >( void* s, unsigned int n );

} // namespace - Version_3_4_5

} // namespace - FrameCPP


#endif // FrameCppUtilHH
