#if HAVE_CONFIG_H
#include "framecpp/config.h"
#endif /* HAVE_CONFIG_H */

#include<malloc.h>

#include <iostream>
#include <fstream>
#include<vector>

#include "general/gpstime.hh"
#include "general/unittest.h"

#include "framecpp/FrameCPP.hh"

// #undef ENABLE_DEPRICATED

#include "framecpp/util.hh"
#include "framecpp/FrameH.hh"
#include "framecpp/FrHistory.hh"
#include "framecpp/FrRawData.hh"
#include "framecpp/FrAdcData.hh"
#include "framecpp/FrDetector.hh"
#include "framecpp/FrStatData.hh"
#include "framecpp/Dimension.hh"

#include "framecpp/Version6/OFrameStream.hh"

#include "test.hh"

using namespace FrameCPP::Version;
using namespace std;

#if FRAME_SPEC_CURRENT == 6
#define VER_STR "6"
#else
#error Version not supported
#endif

// The name of the program executable
char* programname = 0;

/*
	short	int	long
  uint	1	2	3
   int	-1	-2	-3

  -----

float		1.00
double  	2.00
string 		"TesT"
complex8 	3.00, 4.00
complex16 	5.00, 6.00
char		'a'
uchar		'b'
  -----
*/

FrameH* makeFrame()
{
    FrameH* fr = new FrameH( "TesT", -2, 2, GPSTime( 2, 2 ), 1, 2.00, 2 );

    // FrDetector (both detectors are the same)
    //	
    FrDetector d("TesT", "aa",
		 2.0, 2.0, 1.0,
		 1.00, 1.00,
		 1.00, 1.00,
		 1.00, 1.00,
		 -2);
    fr->RefDetectSim().append( d );
    fr->RefDetectProc().append( d );

    // FrHistory (append same record twice)
    FrHistory h1( "TesT", 2, "TesT" );
    fr->RefHistory().append( h1 );
    fr->RefHistory().append( h1 );

    // FrRawData
    FrRawData rawData( "TesT" );
    fr->SetRawData( rawData );

    // FrAdcData
    FrAdcData a( "TesT", 2, 2, 2, 2.00, 1.00, 1.00, "TesT", 2.00,
		 2.0, 1, 1.0 );
    a.AppendComment( "TesT" );
    fr->GetRawData()->RefFirstAdc().append( a );
    
    // character vector
    {
      CHAR d[ 4 ] = { 'a', 'a', 'a', 'a' };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect dvect( "TesT", 1, ddims, d, "TesT" );
#if WORKING_COMPRESS
      dvect.Compress( FrVect::GZIP, FrVect::DEFAULT_GZIP_LEVEL );
#endif
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // unsigned character vector
    {
      CHAR_U d[ 4 ] = { 'b', 'b', 'b', 'b' };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // short vector
    {
      INT_2S d[ 4 ] = { -1, -1, -1, -1 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // unsigned short vector
    {
      INT_2U d[ 4 ] = { 1, 1, 1, 1 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // int vector
    {
      INT_4S d[ 4 ] = { -2, -2, -2, -2 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // unsigned int vector
    {
      INT_4U d[ 4 ] = { 2, 2, 2, 2 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // int vector
    {
      INT_8S d[ 4 ] = { -3, -3, -3, -3 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // unsigned int vector
    {
      INT_8U d[ 4 ] = { 3, 3, 3, 3 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // real vector
    {
      REAL_4 d[ 4 ] = { 1.00, 1.00, 1.00, 1.00 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // real vector
    {
      REAL_8 d[ 4 ] = { 2.00, 2.00, 2.00, 2.00 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    {
      COMPLEX_8 cdata[ 4 ];
      cdata[0] = COMPLEX_8(3.00, 4.00);
      cdata[1] = COMPLEX_8(3.00, 4.00);
      Dimension cdims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect cvect( "TesT", 1, cdims, cdata, "TesT" );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( cvect );
    }

    {
      COMPLEX_16 cdata[ 4 ];
      cdata[0] = COMPLEX_16(5.00, 6.00);
      cdata[1] = COMPLEX_16(5.00, 6.00);
      Dimension cdims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect cvect( "TesT", 1, cdims, cdata, "TesT" );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( cvect );
    }

    // Log Message (a couple of them)
    FrMsg m( "TesT", "TesT", 2, GPSTime( 2, 2 ) );
    fr->GetRawData()->RefLogMsg().append( m );

    // FrProcData
    FrProcData p( "TesT", 2.00, GPSTime( 2, 2 ), 2.00, 1.00 );
    p.AppendComment("TesT");
    {
      INT_2U d[ 512 ];
      Dimension ddims[ 1 ] = {
	Dimension( sizeof( d ) / sizeof( *d ), 2.00, "TesT", 2.00 )
      };

      for ( INT_4U x = 0; x < sizeof( d ) / sizeof( *d ); x++ )
      {
	d[ x ] = x;
      }
      FrVect data( "ProcTest", 1, ddims, d, "Test of Ramp data" );
#if WORKING_COMPRESS
      data.Compress( FrVect::GZIP, FrVect::DEFAULT_GZIP_LEVEL );
#endif
      p.RefData( ).append( data );
    }
	
    fr->RefProcData().append( p );

    // FrSimData
    FrSimData sd( "TesT", "TesT", 1.00, 2.00, 1.00 );
    fr->RefSimData().append( sd );

    // FrSerData
    FrSerData srd( "TesT", GPSTime( 2, 2 ), 1.00 );
    srd.SetData("TesT");
    fr->GetRawData()->RefFirstSer().append( srd );

    // FrStatData
    FrStatData statData("TesT", "TesT", "TesT", 2, 2, 2 );
    // real vector
    {
      REAL_8 d[ 4 ] = { 2.00, 2.00, 2.00, 2.00 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      FrVect dvect( "TesT", 1, ddims, d, "TesT" );
      statData.RefData().append( dvect );
    }

    statData.SetDetector(fr->RefDetectSim()[0]);
    // fr->RefDetectSim()[0]->RefStatData().append( statData );
    statData.SetDetector(fr->RefDetectProc()[0]);
    // fr->RefDetectProc()[0]->RefStatData().append( statData );

    // TrigData
    FrEvent::ParamList_type event_params;
    event_params.push_back( FrEvent::Param_type( "TesT", 2.0 ) );
    FrEvent event(
		  "TesT", "TesT", "TesT", GPSTime( 2, 2 ),
		  1.0, 1.0,
		  2, 1.00,
		  1.00, "TesT",
		  event_params
		  );
    fr->RefEvent().append(event );

    // FrSummary
    FrSummary summary( "TesT", "TesT", "TesT", GPSTime(2, 2) );
    fr->RefSummaryData().append( summary );

    // FrSimEvent
    FrSimEvent sed( "TesT", "TesT", GPSTime(2, 2), 1.00, 1.00, 1.00 );
    sed.AppendComment("TesT");
    fr->RefSimEvent().append( sed );


#if WORKING
    // FrTable
    FrTable table( "TesT", 1 );
    table.AppendComment("TesT");
    fr->RefTable().append( table );
    fr->RefSimEvent()[0]->RefTable().append( table );
#endif /* WORKING */


    // ???

    //     fr->RefTable()[0]->RefColumns().append( vect );

    // :TODO: :FIXME: new Table interface
    // Create table with 8 rows
    //     Table table( "slow", 8 );
    //     fr->RefTable().append( table );
    // Insert first column: verified to be 8 rows deep; vector name is taken first column name
    //     fr->RefTable()[0]->append( vect );
    // Insert first column
    //     fr->RefTable()[0]->insert( 0, vect );
    // Remove indexed column
    //     fr->RefTable()[0]->erase( 0 );
    // Remove range of columns
    //     fr->RefTable()[0]->erase( 0, 1 );

    return fr;
}

void error_watch (const string& msg)
{
  cerr << msg << endl;
}

void
usage( char* f )
{
  if ( ! strncasecmp( f, "-h", 2 ) 
       || ! strncasecmp( f, "--h", 3 ) )
    {
      exit(0);
    }
}

General::UnitTest	Test;

int
main(int argc, char* argv[])
  try
{
  Test.Init( argc, argv );
  /* Determine program name (strip filesystem path) */
  for (programname = argv [0] + strlen (argv [0]) - 1;
       programname != argv [0] && *programname != '/';
       programname--)
    ;

  if (*programname == '/') 
    programname++;

  //  usage help
  if (argc > 1)
    usage(argv[1]);

  //---------------------------------------------------------------------
  // 
  //---------------------------------------------------------------------

  Test.Message( ) << "Version: " << FrameCPP::GetVersion( ) << std::endl;
  Test.Message( ) << "CVS Date: " << FrameCPP::GetCVSDate( ) << std::endl;
  Test.Message( ) << "Build Date: " << FrameCPP::GetBuildDate( ) << std::endl;

  //---------------------------------------------------------------------
  // FrVect
  //---------------------------------------------------------------------
  {
    CHAR data[ 4 ] = { 'a', 'a', 'a', 'a' };
    Dimension datadims[ 1 ] = {
      Dimension( 2, 2.00, "TesT", 2.00 )
    };
    FrVect s( "TesT", 1, datadims, data, "TesT" );
    FrVect d( s );

    Test.Check( s.GetData( false ) != d.GetData( false ) )
      << "FrVect" << std::endl;
  }

  //---------------------------------------------------------------------
  // FrameH
  //---------------------------------------------------------------------

  {
    FrameH* frame = makeFrame();

    FrameH	frame2( *frame );

    Test.Check( frame->GetRawData( ) != frame2.GetRawData( ) )
      << "frame::RawData" << std::endl;
    Test.Check( &(frame->GetRawData( )->RefFirstAdc( )) !=
		&(frame2.GetRawData( )->RefFirstAdc( )) )
      << "frame::RawData::firstAdc" << std::endl;
    Test.Check( frame->GetRawData( )->RefFirstAdc( )[0] !=
		frame2.GetRawData( )->RefFirstAdc( )[0] )
      << "frame::RawData::firstAdc[0]" << std::endl;
    Test.Check( frame->RefProcData( )[0] != frame2.RefProcData( )[0] )
      << "frame::ProcData[0]" << std::endl;

    delete frame;
  }

  Test.Exit( );
} catch (...) {
  Test.Check( false ) << "Caught an exception" << std::endl;
  Test.Exit( );
}

