/* -*- mode: C++; c-basic-offset: 4; -*- */
#if HAVE_CONFIG_H
#include "framecpp/config.h"
#endif /* HAVE_CONFIG_H */

#include <stdlib.h>

#include <new>

#include <iostream>
#include <set>
#include <sstream>
#include <vector>

#include "general/gpstime.hh"
#include "general/autoarray.hh"
#include "general/fstream.hh"

#include "framecpp/FrameCPP.hh"

// #undef ENABLE_DEPRICATED

#include "framecpp/FrameH.hh"
#include "framecpp/FrHistory.hh"
#include "framecpp/FrRawData.hh"
#include "framecpp/FrAdcData.hh"
#include "framecpp/FrDetector.hh"
#include "framecpp/FrStatData.hh"
#include "framecpp/FrVect.hh"
#include "framecpp/Dimension.hh"

#include "framecpp/Version6/OFrameStream.hh"

#include "test.hh"

using std::string;
using std::ostringstream;
using General::ofstream;

#define LM_DEBUG 0

#if LM_DEBUG
#define LM_INFO( a ) std::cerr << a << "( Line: " << __LINE__ << " File: " << __FILE__ << ")" << std::endl;
#define LM_AT( ) LM_INFO( "" )
#else
#define	LM_INFO( a )
#define LM_AT( )
#endif

using namespace FrameCPP::Version;

#if FRAME_SPEC_CURRENT == 6
#define VER_STR "6"
#else
#error Version not supported
#endif

// The name of the program executable
char* programname = 0;
const char *stdTestFrameName = "Z-R_std_test_frame_ver" VER_STR "-600000000-1.gwf";

/*
                short	int	long
  unsigned	 1	 2	 3
    signed	-1	-2	-3

  -----

float		1.00
double  	2.00
string 		"TesT"
complex8 	3.00, 4.00
complex16 	5.00, 6.00
char		'a'
uchar		'b'
  -----
*/

const REAL_4 R4 = 1.0;
const REAL_8 R8 = 2.0;
const std::string S = "TesT";
const INT_2U I2U = 1;
const INT_4U I4U = 2;
const INT_8U I8U = 3;
const INT_2S I2S = -1;
const INT_4S I4S = -2;
const INT_8S I8S = -3;
const char C = 'a';
const unsigned char UC = 'b';
const COMPLEX_8 C8( 3.0, 4.0 );
const COMPLEX_16 C16( 5.0, 6.0 );

typedef std::set<std::string> channel_set_type;

namespace
{
    INT_4U
    ChannelNumber( )
    {
	static INT_4U	cn( 0 );
	return ++cn;
    }

    template< class C, class T >
    void
    append( C& Container, const T& Data, INT_2S Cnt )
    {
	if ( Cnt < 0 )
	{
	    Cnt = Cnt * -1;
	}
	for ( ; Cnt > 0; --Cnt )
	{
	    Container.push_back( Data );
	}
    }
}

#define METHOD(A) case FrameCPP::FrVect::A: return #A
#define DATA_TYPE(A) template< > \
std::string \
data_type< A >( ) \
{ \
  return #A; \
}

template< class T >
std::string
data_type( )
{
  return "";
}

DATA_TYPE(CHAR_U)
DATA_TYPE(CHAR)
DATA_TYPE(INT_2U)
DATA_TYPE(INT_2S)
DATA_TYPE(INT_4U)
DATA_TYPE(INT_4S)
DATA_TYPE(INT_8U)
DATA_TYPE(INT_8S)
DATA_TYPE(REAL_4)
DATA_TYPE(REAL_8)
DATA_TYPE(COMPLEX_8)
DATA_TYPE(COMPLEX_16)

std::string
method( const FrameCPP::FrVect::compression_scheme_type Method )
{
  switch( Method )
  {
    METHOD(RAW);
    METHOD(GZIP);
    METHOD(DIFF);
    METHOD(DIFF_GZIP);
    METHOD(ZERO_SUPPRESS_SHORT);
    METHOD(ZERO_SUPRESS_SHORT_GZIP_OTHER);
    METHOD(ZERO_SUPPRESS_INT_FLOAT);
  default:
    {
      std::ostringstream	msg;

      msg << "Unknown compression method: " << Method;
      throw std::runtime_error( msg.str( ).c_str( ) ); 
    }
    break;
  }
}

template< class T >
void
compress( FrameCPP::FrameH& Frame,
	  FrameCPP::FrVect::compression_scheme_type Method,
	  INT_2U	Level,
	  const T	Start,
	  const T	End,
	  const T	Inc,
	  INT_4U	Samples )
{
  std::ostringstream	adc_name;
  
  adc_name << "TesT_" << method( Method )
	   << "_" << Level
	   << "_" << data_type< T >( );

  General::AutoArray< T	> data( new T[ Samples ] );
  FrameCPP::Dimension	dim( Samples );

  data[ 0 ] = Start;
  for ( INT_4U x = 1; x < Samples; x++ )
  {
      data[ x ] = data[ x - 1 ] + Inc;
  }
    
  FrameCPP::FrVect	adc_data( adc_name.str( ), 1, &dim, data.get() );

  adc_data.Compress( Method, Level );

  // FrAdcData
  LM_AT( );
  FrAdcData adc( adc_name.str( ),
		 0,			// Channel grouping
		 ChannelNumber( ),	// Channel number
		 I4U,			// nBits
		 Samples,		// sampleRate
		 R8,			// bias
		 R4,			// slope
		 S,		// units
		 0.0,			// fShift
		 0.0,			// timeOffset
		 0,			// dataValid
		 0.0			// phase
		 );
  adc.RefData( ).append( adc_data );
  LM_AT( );
  Frame.GetRawData()->RefFirstAdc().append( adc );
}

template< class T >
inline void
compress( FrameCPP::FrameH& Frame,
	  FrameCPP::FrVect::compression_scheme_type Method,
	  INT_2U	Level,
	  const T	Start,
	  const T	End,
	  const T	Inc )
{
  compress( Frame, Method, Level,
	    Start, End, Inc,
	    INT_4U( ( End - Start ) / Inc ) );
}

void
compress( FrameCPP::FrameH& Frame,
	  FrameCPP::FrVect::compression_scheme_type Method,
	  INT_2U	Level )
{
  compress< CHAR_U >( Frame, Method, Level, 0, 255, 1, 256 );
  compress< CHAR >( Frame, Method, Level, -128, 127, 1, 256 );
  compress< INT_2U >( Frame, Method, Level, 0, 65535, 1, 65536 );
  compress< INT_2S >( Frame, Method, Level, -32768, 32767, 1, 65536 );
  compress< INT_4U >( Frame, Method, Level, 0, 65536, 1 );
  compress< INT_4S >( Frame, Method, Level, -32768, 32768, 1 );
  compress< INT_8U >( Frame, Method, Level, 0, 65536, 1 );
  compress< INT_8S >( Frame, Method, Level, -32768, 32768, 1 );
  compress< REAL_4 >( Frame, Method, Level, -32768.0, 32768.0, 1.0 );
  compress< REAL_8 >( Frame, Method, Level, -32768.0, 32768.0, 1.0 );
  compress< COMPLEX_8 >( Frame, Method, Level, 
			 COMPLEX_8( -32768.0, 0.0 ),
			 COMPLEX_8( 32768.0, 0.0 ),
			 COMPLEX_8( 1.0, 0.0 ),
			 65636 );
  compress< COMPLEX_16 >( Frame, Method, Level, 
			  COMPLEX_8( -32768.0, 0.0 ),
			  COMPLEX_8( 32768.0, 0.0 ),
			  COMPLEX_8( 1.0, 0.0 ),
			  65636 );
}

std::auto_ptr< FrameH > makeFrame( std::list< FrStatData >& StatData )
{
    LM_AT( );
    std::auto_ptr< FrameH > fr( new FrameH( S, I4S, I4U, GPSTime( I4U, I4U ), I2U, R8, I4U ) );

    // FrDetector (both detectors are the same)
    //	
    FrDetector d(S,	// name
		 "aa",		// prefix
		 R8,		// longitude
		 R8,		// latitude
		 R4,		// elevation
		 R4,		// armXazimuth
		 R4,		// armYazimuth
		 R4,		// armXaltitude
		 R4,		// armYaltitude
		 R4,		// armXmidpoint
		 R4,		// armYmidpoint
		 I4S );		// localTime

    LM_AT( );
    fr->RefDetectSim().append( d );
    LM_AT( );
    fr->RefDetectProc().append( d );

    // FrHistory (append same record twice)
    LM_AT( );
    FrHistory h1( S, I4U, S );
    LM_AT( );
    fr->RefHistory().append( h1 );

    // FrRawData
    LM_AT( );
    FrRawData rawData( S );
    LM_AT( );
    fr->SetRawData( rawData );

    // FrAdcData
    LM_AT( );
    FrAdcData a( S, I4U,
		 I4U,
		 I4U,
		 R8,
		 R4,
		 R4,
		 S,
		 R8,
		 R8,
		 I2U,
		 R4 );
    LM_AT( );
    a.AppendComment( S );
    LM_AT( );
    fr->GetRawData()->RefFirstAdc().append( a );
    
    // character vector
    {
      CHAR d[ 4 ] = { C, C, C, C };
      Dimension ddims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect dvect( S, I2U, ddims, d, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // unsigned character vector
    {
      CHAR_U d[ 4 ] = { UC, UC, UC, UC };
      Dimension ddims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect dvect( S, I2U, ddims, d, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // short vector
    {
      INT_2S d[ 4 ] = { I2S, I2S, I2S, I2S };
      Dimension ddims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect dvect( S, I2U, ddims, d, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // unsigned short vector
    {
      INT_2U d[ 4 ] = { I2U, I2U, I2U, I2U };
      Dimension ddims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect dvect( S, I2U, ddims, d, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // int vector
    {
      INT_4S d[ 4 ] = { I4S, I4S, I4S, I4S };
      Dimension ddims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect dvect( S, I2U, ddims, d, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // unsigned int vector
    {
      INT_4U d[ 4 ] = { I4U, I4U, I4U, I4U };
      Dimension ddims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect dvect( S, I2U, ddims, d, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // int vector
    {
      INT_8S d[ 4 ] = { I8S, I8S, I8S, I8S };
      Dimension ddims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect dvect( S, I2U, ddims, d, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // unsigned int vector
    {
      INT_8U d[ 4 ] = { I8U, I8U, I8U, I8U };
      Dimension ddims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect dvect( S, I2U, ddims, d, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // real vector
    {
      REAL_4 d[ 4 ] = { R4, R4, R4, R4 };
      Dimension ddims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect dvect( S, I2U, ddims, d, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    // real vector
    {
      REAL_8 d[ 4 ] = { R8, R8, R8, R8 };
      Dimension ddims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect dvect( S, I2U, ddims, d, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( dvect );
    }

    {
      COMPLEX_8 cdata[ 4 ];
      cdata[0] = C8;
      cdata[1] = C8;
      Dimension cdims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect cvect( S, I2U, cdims, cdata, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( cvect );
    }

    {
      COMPLEX_16 cdata[ 4 ];
      cdata[0] = C16;
      cdata[1] = C16;
      Dimension cdims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect cvect( S, 1, cdims, cdata, S );
      fr->GetRawData()->RefFirstAdc()[0]->RefData().append( cvect );
    }

    // Log Message (a couple of them)
    LM_AT( );
    FrMsg m( "Mesg1", S, I4U, GPSTime( I4U, I4U ) );
    LM_AT( );
    fr->GetRawData()->RefLogMsg().append( m );

    // FrProcData
    LM_AT( );
    FrProcData p( S,	// name
		  S,	// comment
		  I2U,	// type
		  I2U,	// subType
		  R8,	// timeOffset
		  R8,	// tRange
		  R8,	// fShift
		  R4,	// phase
		  R8,	// fRange
		  R8 );	// BW
    {
      INT_2U d[ 512 ];
      Dimension ddims[ 1 ] = {
	Dimension( sizeof( d ) / sizeof( *d ), R8, S, R8 )
      };

      for ( INT_4U x = 0; x < sizeof( d ) / sizeof( *d ); x++ )
      {
	d[ x ] = x;
      }
      FrVect data( S, 1, ddims, d, "Test of Ramp data" );
      p.RefData( ).append( data );
      FrProcData::AuxParamList_type& param( p.GetAuxParam( ) );
      append( param, FrProcData::AuxParam_type( S, R8 ), I2U );
    }
	
    LM_AT( );
    fr->RefProcData().append( p );

    // FrSimData
    LM_AT( );
    fr->RefSimData().append( FrSimData( S, S, R4, R8, R4 ) );
    LM_AT( );
    fr->RefSimData().append( FrSimData( "SimData2", S, R4, R8, R4 ) );

    // FrSerData
    LM_AT( );
    FrSerData srd( S, GPSTime( I4U, I4U ), R4 );
    LM_AT( );
    srd.SetData(S);
    LM_AT( );
    FrSerData srd2( "SerData2", GPSTime( I4U, I4U ), R4 );
    LM_AT( );
    fr->GetRawData()->RefFirstSer().append( srd );
    LM_AT( );
    fr->GetRawData()->RefFirstSer().append( srd2 );

    // FrStatData
    LM_AT( );
    FrStatData statData("StatData1", S, S, I4U, I4U, I4U );
    // real vector
    {
      REAL_8 d[ 4 ] = { R8, R8, R8, R8 };
      Dimension ddims[ 1 ] = {
	Dimension( I4U, R8, S, R8 )
      };
      FrVect dvect( "FrVect1", I2U, ddims, d, S );
      statData.RefData().append( dvect );
    }

    LM_AT( );
    statData.SetDetector(fr->RefDetectSim()[0]);
    StatData.push_back( statData );
    LM_AT( );
    statData.SetDetector(fr->RefDetectProc()[0]);
    StatData.push_back( statData );

    // TrigData
    LM_AT( );
    FrEvent::ParamList_type event_params;
    event_params.push_back( FrEvent::Param_type( S, R4 ) );
    FrEvent event(
		  S, S, S, GPSTime( I4U, I4U ),
		  R4, R4,
		  I4U, R4,
		  R4, S,
		  event_params
		  );
    {
      FrEvent::ParamList_type& param( event.GetParam( ) );
      append( param, FrEvent::Param_type( S, R4 ), I2U );
    }
    LM_AT( );
    fr->RefEvent().append(event );

    // FrSummary
    ;
    LM_AT( );
    fr->RefSummaryData().append( FrSummary( S, S, S, GPSTime(I4U, I4U) ) );
    LM_AT( );
    fr->RefSummaryData().append( FrSummary( "FrSummary2", S, S, GPSTime(I4U, I4U) ) );

    // FrSimEvent
    LM_AT( );
    FrSimEvent sed( S, S, GPSTime(I4U, I4U), R4, R4, R4 );
    LM_AT( );
    sed.AppendComment(S);
    {
	FrSimEvent::ParamList_type& param( sed.GetParam( ) );
	append( param, FrSimEvent::Param_type( S, R4 ), I2U );
    }
    LM_AT( );
    fr->RefSimEvent().append( sed );


#if WORKING
    // FrTable
    FrTable table( S, I2U );
    table.AppendComment(S);
    fr->RefTable().append( table );
    fr->RefSimEvent()[0]->RefTable().append( table );
#endif /* WORKING */


    // ???

    //     fr->RefTable()[0]->RefColumns().append( vect );

    // :TODO: :FIXME: new Table interface
    // Create table with 8 rows
    //     Table table( "slow", 8 );
    //     fr->RefTable().append( table );
    // Insert first column: verified to be 8 rows deep; vector name is taken first column name
    //     fr->RefTable()[0]->append( vect );
    // Insert first column
    //     fr->RefTable()[0]->insert( 0, vect );
    // Remove indexed column
    //     fr->RefTable()[0]->erase( 0 );
    // Remove range of columns
    //     fr->RefTable()[0]->erase( 0, 1 );

    //-------------------------------------------------------------------
    // RAW
    //-------------------------------------------------------------------
    compress( *fr, FrameCPP::FrVect::RAW, 0 );
    //-------------------------------------------------------------------
    // Test all compression levels for gzip.
    //-------------------------------------------------------------------
    for ( INT_2U level = 1; level <= 9; level++ )
    {
	compress( *fr, FrameCPP::FrVect::GZIP, level );
    }
    //-------------------------------------------------------------------
    // Test all compression levels for gzip_diff.
    //-------------------------------------------------------------------
    for ( INT_2U level = 1; level <= 9; level++ )
    {
	compress( *fr, FrameCPP::FrVect::DIFF_GZIP, level );
    }
    //-------------------------------------------------------------------
    // ZERO_SUPPRESS_SHORT
    //-------------------------------------------------------------------
    compress( *fr, FrameCPP::FrVect::ZERO_SUPPRESS_SHORT, 0 );
    //-------------------------------------------------------------------
    // ZERO_SUPPRESS_INT_FLOAT
    //-------------------------------------------------------------------
    compress( *fr, FrameCPP::FrVect::ZERO_SUPPRESS_INT_FLOAT, 0 );

    return fr;
}

void
usage( char* ProgramName, int Exit )
{
    std::cout << "usage: " << ProgramName
	      << " [-c<channel type>]"
	      << " [-d<duration>]"
	      << " [-h]"
	      << " [-i<iterations>]"
	      << " [-s<start time>]"
	      << " [-t<type specifier>]"
	      << std::endl
	      << "    With no options specified, create LIGO standard testing"
	      << std::endl
	      << "    frame file '" << stdTestFrameName << "'"
	      << std::endl
	      << std::endl
	      << "    If any options are specified, then a frame file with"
	      << std::endl
	      << "    one or more datatypes is generated where:"
	      << std::endl
	      << "        <channel type> is the type of channel" << std::endl
	      << "        <duration> is frame length in seconds" << std::endl
	      << "        <iterations> is the number of sets of channels to produce" << std::endl
	      << "        <start time> specifies the gps start time for the frame" << std::endl
	      << "        <type specifier> is the type component of the frame file name" << std::endl;
    exit( Exit );
}

template< class T >
string
ramped_name( INT_4U Iteration )
{
    ostringstream	name;

    name << "Z0:RAMPED_" << data_type< T >( ) << "_" << Iteration;
    return name.str( );
}

template< class T >
T*
ramp_data_create( INT_4U Samples, T Start, T Stop, T Inc )
{
    General::AutoArray< T > data( new T[ Samples ] );
    data[ 0 ] = Start;
    for ( INT_4U x = 1; x < Samples; x++ )
    {
	if ( data[x -1] == Stop )
	{
	    data[x] = Start;
	}
	else
	{
	    data[ x ] = data[ x - 1 ] + Inc;
	}
    }
    return data.release( );
}

template< class T >
std::auto_ptr< FrVect >
ramped_vector( INT_4U DeltaT, INT_4U SampleRate, INT_4U Iteration,
	       T Start, T Stop, T Inc )
{
    const INT_4U samples( DeltaT * SampleRate );

    FrameCPP::Dimension	dim( samples );
    General::AutoArray< T > data( ramp_data_create< T >( samples,
							 Start,
							 Stop,
							 Inc ) );

    std::auto_ptr< FrVect >
	vect( new FrVect( ramped_name< T >( Iteration ), 1, &dim,
			  data.release( ), "" /* unitY */,
			  false /* allocate */, true /* own */ ) );
    return vect;
}

template< class T >
std::auto_ptr< FrAdcData >
ramped_channel( INT_4U DeltaT, INT_4U* Channel, INT_4U Iteration,
		T Start, T Stop, T Inc,
		INT_4U SampleRate )
{
    //-------------------------------------------------------------------
    // Create the FrAdcData structure to hold the Adc metadata
    //-------------------------------------------------------------------
    std::auto_ptr< FrAdcData >
	adc ( new FrAdcData( ramped_name< T >( Iteration ),
			     0, /* Group */
			     *Channel, /* Channel Number */
			     32, /* NBits */
			     SampleRate, /* sampleRate */
			     0.0, /* bias */
			     1.0, /* slope */
			     "counts", /* units */
			     0.0, /* fShift */
			     0.0, /* timeOffset */
			     0, /* dataValid */
			     0.0 /* phase */ ) );
    //-------------------------------------------------------------------
    // Link the data to the metadata
    //-------------------------------------------------------------------
    adc->RefData( ).append( ramped_vector< T >( DeltaT,
						SampleRate,
						Iteration,
						Start, Stop, Inc
						).release( ),
			    false, /* allocate */
			    true /* own */ );
    //-------------------------------------------------------------------
    // Incriment the channel number
    //-------------------------------------------------------------------
    (*Channel)++;
    return adc;
}

void
test_frame( )
{
    std::list< FrStatData > stat_data;
    std::auto_ptr< FrameH > frame( makeFrame( stat_data ) );

    ofstream out( stdTestFrameName );
    OFrameStream fw( out );
    for ( std::list< FrStatData >::iterator
	      cur = stat_data.begin( ),
	      last = stat_data.end( );
	  cur != last;
	  ++cur )
    {
	fw.WriteFrStatData( *cur );
    }
    fw.WriteFrame( *frame );
    fw.close();
    out.close();
}

void
all_type_iteration( const INT_4U DeltaT,
		    const channel_set_type& ListOfChannels,
		    std::auto_ptr< FrameH >& Frame,
		    INT_4U* Channel,
		    INT_4U Iteration )
{
    FrRawData*	rd( Frame->GetRawData( ) );

    //-------------------------------------------------------------------
    // Make sure that we do not dereference a null pointer
    //-------------------------------------------------------------------
    if ( rd == (FrRawData*)NULL )
    {
	return;
    }
    //-------------------------------------------------------------------
    // Safe to use
    //-------------------------------------------------------------------
    FrRawData::firstAdc_type& adclist( rd->RefFirstAdc( ) );
    //-------------------------------------------------------------------
    // Add the ramped channels
    //-------------------------------------------------------------------
    channel_set_type::const_iterator	last_channel( ListOfChannels.end( ) );

    if ( ListOfChannels.find( "int_2u" ) != last_channel )
    {
	adclist.append( ramped_channel< INT_2U >( DeltaT, Channel, Iteration,
						  0, 512, 1,
						  512 ).release( ),
			false /* allocate */,
			true /* own */ );
    }
    if ( ListOfChannels.find( "int_2s" ) != last_channel )
    {
	adclist.append( ramped_channel< INT_2S >( DeltaT, Channel, Iteration,
						  -512, 512, 1,
						  512 ).release( ),
			false /* allocate */,
			true /* own */ );
    }
    if ( ListOfChannels.find( "int_4u" ) != last_channel )
    {
	adclist.append( ramped_channel< INT_4U >( DeltaT, Channel, Iteration,
						  0, 1024, 1,
						  1024 ).release( ),
			false /* allocate */,
			true /* own */ );
    }
    if ( ListOfChannels.find( "int_4s" ) != last_channel )
    {
	adclist.append( ramped_channel< INT_4S >( DeltaT, Channel, Iteration,
						  -1024, 1024, 1,
						  1024 ).release( ),
			false /* allocate */,
			true /* own */ );
    }
    if ( ListOfChannels.find( "int_8u" ) != last_channel )
    {
	adclist.append( ramped_channel< INT_8U >( DeltaT, Channel, Iteration,
						  0, 2048, 1,
						  1024 ).release( ),
			false /* allocate */,
			true /* own */ );
    }
    if ( ListOfChannels.find( "int_8s" ) != last_channel )
    {
	adclist.append( ramped_channel< INT_8S >( DeltaT, Channel, Iteration,
						  -2048, 2048, 1,
						  1024 ).release( ),
			false /* allocate */,
			true /* own */ );
    }

    if ( ListOfChannels.find( "real_4" ) != last_channel )
    {
	adclist.append( ramped_channel< REAL_4 >( DeltaT, Channel, Iteration,
						  -1024, 1024, 0.5,
						  1024 ).release( ),
			false /* allocate */,
			true /* own */ );
    }
    if ( ListOfChannels.find( "real_8" ) != last_channel )
    {
	adclist.append( ramped_channel< REAL_8 >( DeltaT, Channel, Iteration,
						  -1024, 1024, 0.5,
						  1024 ).release( ),
			false /* allocate */,
			true /* own */ );
    }

    if ( ListOfChannels.find( "complex_8" ) != last_channel )
    {
	adclist.append( ramped_channel< COMPLEX_8 >( DeltaT, Channel, Iteration,
						     COMPLEX_8( -1024, 0),
						     COMPLEX_8( 1024, 0),
						     COMPLEX_8( 1, 0 ),
						     1024 ).release( ),
			false /* allocate */,
			true /* own */ );
    }
    if ( ListOfChannels.find( "complex_16" ) != last_channel )
    {
	adclist.append( ramped_channel< COMPLEX_16 >( DeltaT, Channel, Iteration,
						      COMPLEX_16( -1024, 0),
						      COMPLEX_16( 1024, 0),
						      COMPLEX_16( 1, 0 ),
						      1024 ).release( ),
			false /* allocate */,
			true /* own */ );
    }

}

void
all_type_frame( const string& Type,
		const channel_set_type& ChannelSet,
		const GPSTime& Start,
		const INT_4U DeltaT,
		const INT_4U Iterations )
{
    //-------------------------------------------------------------------
    // Create filename
    //-------------------------------------------------------------------
    std::ostringstream	framename;

    framename << "Z-" << Type << "-"
	      << Start.GetSeconds( ) << "-" << DeltaT
	      << ".gwf";

    //-------------------------------------------------------------------
    // Open the file and frame stream
    //-------------------------------------------------------------------
    ofstream	out( framename.str( ).c_str( ) );
    OFrameStream fw( out );

    //-------------------------------------------------------------------
    // Generation of the frame
    //-------------------------------------------------------------------

    std::auto_ptr< FrameH >
	frame( new FrameH( Type, -1, 0,
			   Start, Start.GetLeapSeconds( ),
			   REAL_8( DeltaT ) ) );

    frame->SetRawData( new FrRawData( "RawData" ),
		       false, /* allocate */
		       true /* own */ );

    //-------------------------------------------------------------------
    // Loop over the number of iterations for creation of the data sets
    //-------------------------------------------------------------------

    for ( INT_4U i = 1; i <= Iterations; ++i )
    {
	INT_4U	channel = 0;
	all_type_iteration( DeltaT, ChannelSet, frame, &channel, i );
    }

    //-------------------------------------------------------------------
    // Writing of the frame
    //-------------------------------------------------------------------

    fw.WriteFrame( *(frame.get( )) );

    //-------------------------------------------------------------------
    // Close the frame and file stream
    //-------------------------------------------------------------------
    fw.close();
    out.close( );
}

int
main(int argc, char* argv[])
  try
{
    bool	create_test_frame = true;
    int		args_parsed;

    INT_4U		deltat = 1;
    INT_4U		iterations = 1;
    GPSTime		start( 600000000, 0 );
    string		type( "ilwd_test_frame" );
    channel_set_type	channel_set;

    /* Determine program name (strip filesystem path) */
    for (programname = argv [0] + strlen (argv [0]) - 1;
	 programname != argv [0] && *programname != '/';
	 programname--)
	;
    
    if (*programname == '/')
    {
	programname++;
    }

    for ( args_parsed = 1; args_parsed < argc; ++args_parsed )
    {
	char* cur_arg = argv[ args_parsed ];
	
	if ( strlen( cur_arg ) > 2 )
	{
	    if ( strncmp( cur_arg, "-c", 2 ) == 0 )
	    {
		std::string	opt( &cur_arg[2] );

		// Convert to lower case
		for( std::string::iterator
			 cur = opt.begin( ),
			 last = opt.end( );
		     cur != last;
		     ++cur )
		{
		    *cur = tolower( *cur );
		}
		channel_set.insert( opt );
		create_test_frame = false;
	    }
	    else if ( strncmp( cur_arg, "-d", 2 ) == 0 )
	    {
		// Specify the duration in seconds for the output data.
		deltat = atol( &(cur_arg[2]) );
		create_test_frame = false;
	    }
	    else if ( ! strncasecmp( cur_arg, "-h", 2 ) 
		      || ! strncasecmp( cur_arg, "--h", 3 ) )
	    {
		usage( argv[0], 0 );
	    }
	    else if ( strncmp( cur_arg, "-i", 2 ) == 0 )
	    {
		// Specify the number of iterations
		iterations = atol( &(cur_arg[2]) );
		create_test_frame = false;
	    }
	    else if ( strncmp( cur_arg, "-s", 2 ) == 0 )
	    {
		// Specify the gps start time in whole seconds
		start = GPSTime( atol( &(cur_arg[2]) ), 0 );
		create_test_frame = false;
	    }
	    else if ( strncmp( cur_arg, "-t", 2 ) == 0 )
	    {
		// Specify the gps start time in whole seconds
		type = &(cur_arg[2]);
		create_test_frame = false;
	    }
	    else
	    {
		break;
	    }
	}
	else
	{
	    break;
	}
    }
    //  usage help
    if (args_parsed != argc )
    {
	usage( argv[0], 1 );
    }

    if ( create_test_frame )
    {
	test_frame( );
    }
    else
    {
	if ( channel_set.size( ) == 0 )
	{
	    channel_set.insert( "int_2u" );
	    channel_set.insert( "int_2s" );
	    channel_set.insert( "int_4u" );
	    channel_set.insert( "int_4s" );
	    channel_set.insert( "int_8u" );
	    channel_set.insert( "int_8s" );
	    channel_set.insert( "real_4" );
	    channel_set.insert( "real_8" );
	    channel_set.insert( "complex_8" );
	    channel_set.insert( "complex_16" );
	}
	all_type_frame( type, channel_set, start, deltat, iterations );
    }

    return 0;
} catch ( const std::exception& e ) {
    std::cerr << "FATAL: Caught exception: " << e.what( ) << std::endl;
    return 1;
} catch (...) {
    std::cerr << "FATAL: Caught unknown exception: " << std::endl;
    return 1;
}

