#if HAVE_CONFIG_H
#include "framecpp/config.h"
#endif /* HAVE_CONFIG_H */

#include <algorithm>
#include <memory>
#include <sstream>

#include "general/autoarray.hh"
#include "general/unittest.h"

#include "framecpp/Dimension.hh"
#include "framecpp/FrAdcData.hh"
#include "framecpp/FrVect.hh"

General::UnitTest	Test;

using namespace FrameCPP;

using FrameCPP::Version::FR_ADC_DATA_DEFAULT_BIAS;
using FrameCPP::Version::FR_ADC_DATA_DEFAULT_SLOPE;
using FrameCPP::Version::FR_ADC_DATA_DEFAULT_FSHIFT;
using FrameCPP::Version::FR_ADC_DATA_DEFAULT_TIME_OFFSET;
using FrameCPP::Version::FR_ADC_DATA_DEFAULT_DATA_VALID;
using FrameCPP::Version::FR_ADC_DATA_DEFAULT_PHASE;
using FrameCPP::Version::FR_ADC_DATA_DEFAULT_UNITS;

static const INT_2U SAMPLES = 256;

namespace
{
  template <class T>
  class Riser
  {
  public:
    Riser( )
      : m_counter( 0 )
    {
    }

    T operator()( )
    {
      return m_counter++;
    }

  private:
    T	m_counter;
  };
}

void
fr_adc_data( )
{
  //---------------------------------------------------------------------
  // Create a Single Dimensional Vector
  //---------------------------------------------------------------------
  General::AutoArray< INT_2U >	data( new INT_2U[ SAMPLES ] );
  Dimension			dim( SAMPLES );
  Riser< INT_2U >		riser;	

  std::generate( &(data[0]), &(data[ SAMPLES ]), riser );
  std::auto_ptr< FrVect > v( new FrVect( "fr_vect", 1, &dim,
					 data.release( ), "unitY",
					 false, true ) );

  //---------------------------------------------------------------------
  // Create the adc
  //---------------------------------------------------------------------
  std::auto_ptr< FrAdcData >
    a( new FrAdcData( "adc_data", INT_4U( 0 ), INT_4U( 0 ), INT_4U( 0 ),
		      REAL_8( SAMPLES ),
		      FR_ADC_DATA_DEFAULT_BIAS,
		      FR_ADC_DATA_DEFAULT_SLOPE,
		      FR_ADC_DATA_DEFAULT_UNITS( ),
		      FR_ADC_DATA_DEFAULT_FSHIFT,
		      FR_ADC_DATA_DEFAULT_TIME_OFFSET,
		      FR_ADC_DATA_DEFAULT_DATA_VALID,
		      FR_ADC_DATA_DEFAULT_PHASE ) );
  a->RefData( ).append( v.release( ), false, true );

  //---------------------------------------------------------------------
  // Check the creation of sub FrAdcData
  //---------------------------------------------------------------------
  std::auto_ptr< FrAdcData > sub_adc( a->SubFrAdcData( 0, 1 ) );
  Test.Check( *a == *sub_adc )
    << "Creation of subadc of equal length"
    << std::endl;
}

void
fr_vect( )
{
  //---------------------------------------------------------------------
  // Create a Single Dimensional Vector
  //---------------------------------------------------------------------
  General::AutoArray< INT_2U >	data( new INT_2U[ SAMPLES ] );
  Dimension			dim( SAMPLES );
  Riser< INT_2U >		riser;	

  std::generate( &(data[0]), &(data[ SAMPLES ]), riser );
  std::auto_ptr< FrVect > v( new FrVect( "fr_vect", 1, &dim,
					 data.release( ), "unitY",
					 false, true ) );
  //---------------------------------------------------------------------
  // Check the creation of sub vectors
  //---------------------------------------------------------------------
  std::auto_ptr< FrVect > sub_vect( v->SubFrVect( 0, SAMPLES ) );
  Test.Check( ( *v == *sub_vect ) )
    << "Creation of subvector of equal length"
    << std::endl;

}

int
main(int argc, char* argv[])
try
{
  //---------------------------------------------------------------------
  // Prepare for testing
  //---------------------------------------------------------------------
  Test.Init( argc, argv );

  //---------------------------------------------------------------------
  // Testing of basic data types
  //---------------------------------------------------------------------
  fr_vect( );		// FrVect
  fr_adc_data( );	// FrAdcData

  //---------------------------------------------------------------------
  // Exit with the proper exit status
  //---------------------------------------------------------------------
  Test.Exit( );
} catch (...) {
  Test.Check( false ) << "Caught an exception" << std::endl;
  Test.Exit( );
}
