#ifndef FRAMEF_HH
#define FRAMEF_HH

#include "fferrors.hh"
#include <string>
#include <iostream>

/**  Class frameF reads in a frame file.
  *  @memo Fast frame file reader.
  *  @author John Zweizig
  *  @version 1.2; Modified July 27, 2000 
  */
class FrameF {
public:
  /**  Construct a Frame file reader and attach it to an existing stream. 
    *  The stream is specified.
    *  @memo File constructor.
    */
  FrameF(std::istream& in);

  /**  Destructor.
   */
  ~FrameF(void);

  //------------------------------------  Accessors
  bool   isOK(void) const;
  short  getID(void) const;
  short  getInstance(void) const;
  int    getLength(void) const;
  int    getPatch(void) const;
  int    getVersion(void) const;

  //------------------------------------  Mutators
  void   ReadHeader(void) throw (BadFile);
  bool   NxStruct(void) throw (BadFile);
  short  getShort(void) throw (BadFile);
  float  getFloat(void) throw (BadFile);
  int    getInt(void) throw (BadFile);
  std::string getString(void) throw (BadFile);
  double getDouble(void) throw (BadFile);
  void   resetHeader(void);
  void   Seek(int off, std::ios::seekdir mode);
  void   Skip(int off);

private:
  std::istream& mIn;
  bool     mSwap;
  bool     mHeaderOK;
  char     mData[40];
  struct {
    int   length;
    short ID;
    short Instance;
  } mSHdr;
  int      mOffset;
};

inline int 
FrameF::getVersion() const {
    return mData[5];
}
inline int 
FrameF::getPatch() const {
    return mData[5];
}

inline bool
FrameF::isOK(void) const {
    return !mIn.bad();
}

inline short
FrameF::getID(void) const {
    return mSHdr.ID;
}

inline short 
FrameF::getInstance(void) const {
    return mSHdr.Instance;
}

inline int
FrameF::getLength(void) const {
    return mSHdr.length;
}
inline void
FrameF::resetHeader(void) {
    mHeaderOK = false;
}
#endif
