/*
  frdump -- dump frame file contents
*/
#include "framecpp/config.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <fstream>
#include <iostream>
#include <stdexcept>

#include "framecpp/dump.hh"	// Version_3_4_5
#include "framecpp/FrameH.hh"
#include "framecpp/FrameReader.hh"

using namespace FrameCPP;
using namespace std;

// The name of the program executable
char* programname = 0;

void
fatal( string msg ) {
  cerr << "FATAL: " << msg << endl;
  exit (1);
}

void
error_watch (const string& msg)
{
  cerr << "error: " << msg << endl;
}


void
usage( char* f )
{
  if ( ! strncasecmp( f, "-h", 2 ) 
       || ! strncasecmp( f, "--h", 3 ) )
    {
      cout << "Dump frame file contents" << endl;
      cout << "Usage: " << (string)programname << " INFILE" << endl;
      exit(0);
    }
}

int
main( int argc, char* argv[] )
try
{  
  /* Determine program name (strip filesystem path) */
  for (programname = argv [0] + strlen (argv [0]) - 1;
       programname != argv [0] && *programname != '/';
       programname--)
    ;

  if (*programname == '/')
  {
    programname++;
  }

  if ( argc > 1 )
  {
    usage( argv[1] );
  }
  
  int ifd;
  if (argc == 2)
  {
    ifd = open( argv[ 1 ], O_RDONLY );
  } else {
    usage("-h");
    return 0;
  }
  if ( ifd < 0 )
  {
    fatal( "could not open `" + (string)argv[1] + "' for input" );
  }
  
#if __GNUC__ >= 3
  std::ifstream in;
  {
    FILE *cfile = fdopen(ifd,"r");
    std::ifstream::__filebuf_type* inbuf
        = new std::ifstream::__filebuf_type (cfile, std::ios_base::in);
    std::ofstream::__streambuf_type* oldinbuf
        = ((std::ofstream::__ios_type* )&in)->rdbuf (inbuf);
    delete oldinbuf;
  }
#else
  ifstream in( ifd );
#endif
  
  FrameReader fr( in );
  // fr.setErrorWatch( error_watch );

  try {
    while( fr.isEndOfFile( ) == false )
    {
      FrameH* frame = fr.readFrame();
      if (!frame)
	break;
      dump(cout, *frame);
      delete frame;
    }
  } catch ( std::exception& e ) {
     fatal( e.what() );
  }

  in.close();
  return 0;
}
catch( const std::exception& e )
{
  std::cerr << "FATAL: Caught exception: " << e.what( ) << std::endl;
  exit( 1 );
}
catch( ... )
{
  std::cerr << "FATAL: Caught unknown exception: " << std::endl;
  exit( 1 );
}
