#if HAVE_CONFIG_H
#include "framecpp/config.h"
#endif /* HAVE_CONFIG_H */

#include<malloc.h>

#include <iostream>
#include <fstream>
#include<vector>

#include "framecpp/Version3_4_5/frame.hh"
#include "framecpp/Version3_4_5/history.hh"
#include "framecpp/Version3_4_5/rawdata.hh"
#include "framecpp/Version3_4_5/adcdata.hh"
#include "framecpp/Version3_4_5/detector.hh"
#include "framecpp/Version3_4_5/framewritertoc.hh"
#include "framecpp/Version3_4_5/frameappendwritertoc.hh"
#include "framecpp/Version3_4_5/framewriter.hh"
#include "framecpp/Version3_4_5/tocreader.hh"
#include "framecpp/Version3_4_5/framereader.hh"
#include "framecpp/Version3_4_5/dump.hh"
#include "framecpp/Version3_4_5/dimension.hh"

#include "test.hh"

using namespace FrameCPP::Version_3_4_5;
using namespace std;

// The name of the program executable
char* programname = 0;
const char *stdTestFrameName = "std_test_frame";

/*
	short	int	long
  uint	1	2	3
   int	-1	-2	-3

  -----

float		1.00
double  	2.00
string 		"TesT"
complex8 	3.00, 4.00
complex16 	5.00, 6.00
char		'a'
uchar		'b'
  -----
*/

Frame* makeFrame()
{
    Frame* fr = new Frame(
        "TesT", -2, 2, Time( 2, 2 ), 1, 2.00, 2 );

    // Detector (both detectors are the same)
    //	
    Detector d("TesT",
			 2.0, 2.0, 1.0,
			 1.00, 1.00,
			 1.00, 1.00,
			 1.00, 1.00,
			 -2, 2,"TesT");
    fr->refDetectorSim().append( d );
    fr->refDetectorProc().append( d );

    // History (append same record twice)
    History h1( "TesT", 2, "TesT" );
    fr->refHistory().append( h1 );
    fr->refHistory().append( h1 );

    // RawData
    RawData rawData( "TesT" );
    fr->setRawData( rawData );

    // AdcData
    AdcData a( "TesT", 2, 2, 2, 2.00, 1.00, 1.00, "TesT", 2.00,
			 Time(-2, 2), 1, 1.0 );
    a.appendComment( "TesT" );
    fr->getRawData()->refAdc().append( a );
    
    // character vector
    {
      CHAR d[ 4 ] = { 'a', 'a', 'a', 'a' };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( dvect );
    }

    // unsigned character vector
    {
      CHAR_U d[ 4 ] = { 'b', 'b', 'b', 'b' };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( dvect );
    }

    // short vector
    {
      INT_2S d[ 4 ] = { -1, -1, -1, -1 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( dvect );
    }

    // unsigned short vector
    {
      INT_2U d[ 4 ] = { 1, 1, 1, 1 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( dvect );
    }

    // int vector
    {
      INT_4S d[ 4 ] = { -2, -2, -2, -2 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( dvect );
    }

    // unsigned int vector
    {
      INT_4U d[ 4 ] = { 2, 2, 2, 2 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( dvect );
    }

    // int vector
    {
      INT_8S d[ 4 ] = { -3, -3, -3, -3 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( dvect );
    }

    // unsigned int vector
    {
      INT_8U d[ 4 ] = { 3, 3, 3, 3 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( dvect );
    }

    // real vector
    {
      REAL_4 d[ 4 ] = { 1.00, 1.00, 1.00, 1.00 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( dvect );
    }

    // real vector
    {
      REAL_8 d[ 4 ] = { 2.00, 2.00, 2.00, 2.00 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect dvect( "TesT", 1, ddims, d, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( dvect );
    }

    {
      COMPLEX_8 cdata[ 4 ];
      cdata[0] = COMPLEX_8(3.00, 4.00);
      cdata[1] = COMPLEX_8(3.00, 4.00);
      Dimension cdims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect cvect( "TesT", 1, cdims, cdata, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( cvect );
    }

    {
      COMPLEX_16 cdata[ 4 ];
      cdata[0] = COMPLEX_16(5.00, 6.00);
      cdata[1] = COMPLEX_16(5.00, 6.00);
      Dimension cdims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect cvect( "TesT", 1, cdims, cdata, "TesT" );
      fr->getRawData()->refAdc()[0]->refData().append( cvect );
    }

    // Log Message (a couple of them)
    Msg m( "TesT", "TesT", 2, Time( 2, 2 ) );
    fr->getRawData()->refMsg().append( m );
    fr->getRawData()->refMsg().append( m );

    // ProcData
    ProcData p( "TesT", 2.00, Time( 2, 2 ), 2.00, 1.00 );
    p.appendComment("TesT");
    fr->refProcData().append( p );
    fr->refProcData().append( p );

    // SimData
    SimData sd( "TesT", "TesT", 1.00, 2.00, 1.00 );
    fr->refSimData().append( sd );
    fr->refSimData().append( sd );

    // SerData
    SerData srd( "TesT", Time( 2, 2 ), 1.00 );
    srd.setData("TesT");
    fr->getRawData()->refSerial().append( srd );
    fr->getRawData()->refSerial().append( srd );

    // StatData
    StatData statData("TesT", "TesT", "TesT", 2, 2, 2 );
    // real vector
    {
      REAL_8 d[ 4 ] = { 2.00, 2.00, 2.00, 2.00 };
      Dimension ddims[ 1 ] = {
	Dimension( 2, 2.00, "TesT", 2.00 )
      };
      Vect dvect( "TesT", 1, ddims, d, "TesT" );
      statData.refData().append( dvect );
    }

    statData.setDetector(fr->refDetectorSim()[0]);
    fr->refDetectorSim()[0]->refStatData().append( statData );
    fr->refDetectorSim()[0]->refStatData().append( statData );
    statData.setDetector(fr->refDetectorProc()[0]);
    fr->refDetectorProc()[0]->refStatData().append( statData );
    fr->refDetectorProc()[0]->refStatData().append( statData );

    // TrigData
    Event event(
        "TesT", "TesT", "TesT", Time( 2, 2 ),
	1.0, 1.0,
	2, 1.00,
        1.00, "TesT",
	std::vector<std::pair<std::string, REAL_4> >()
	);
    fr->refEvent().append(event );
    fr->refEvent().append( event );

    // Summary
    Summary summary( "TesT", "TesT", "TesT", Time(2, 2) );
    fr->refSummary().append( summary );
    fr->refSummary().append( summary );

    // SimEvent
    SimEvent sed( "TesT", "TesT", Time(2, 2), 1.00, 1.00, 1.00 );
    sed.appendComment("TesT");
    fr->refSimEvent().append( sed );


    // Table
    Table table( "TesT", 1 );
    table.appendComment("TesT");
    fr->refTable().append( table );
    fr->refSimEvent()[0]->refTable().append( table );


    // ???

    //     fr->refTable()[0]->refColumns().append( vect );

    // :TODO: :FIXME: new Table interface
    // Create table with 8 rows
    //     Table table( "slow", 8 );
    //     fr->refTable().append( table );
    // Insert first column: verified to be 8 rows deep; vector name is taken first column name
    //     fr->refTable()[0]->append( vect );
    // Insert first column
    //     fr->refTable()[0]->insert( 0, vect );
    // Remove indexed column
    //     fr->refTable()[0]->erase( 0 );
    // Remove range of columns
    //     fr->refTable()[0]->erase( 0, 1 );

    return fr;
}

void error_watch (const string& msg)
{
  cerr << msg << endl;
}

void
usage( char* f )
{
  if ( ! strncasecmp( f, "-h", 2 ) 
       || ! strncasecmp( f, "--h", 3 ) )
    {
      cout << "Create LIGO standard testing frame file `"
	   << stdTestFrameName << "'" << endl;
      exit(0);
    }
}

int
main(int argc, char* argv[])
  try
{
  /* Determine program name (strip filesystem path) */
  for (programname = argv [0] + strlen (argv [0]) - 1;
       programname != argv [0] && *programname != '/';
       programname--)
    ;

  if (*programname == '/') 
    programname++;

  //  usage help
  if (argc > 1)
    usage(argv[1]);

  Frame* frame = makeFrame();

  ofstream out( stdTestFrameName );
  FrameWriterTOC fw( out );
  fw.writeFrame( *frame );
  fw.close();
  out.close();
  return 0;
} catch (...) {
  return 1;
}

