/*
  frvalidate - validate frame structure in a file
*/

#include "framecpp/config.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string>
#include <fstream>
#include <iostream>

using namespace std;

// The name of the program executable
char* programname = 0;

void
fatal( string msg ) {
  cout << msg << endl;
  exit (1);
}

void
error_watch (const string& msg)
{
  cout << "error: " << msg << endl;
}


void
usage( char* f )
{
  if ( ! strncasecmp( f, "-h", 2 ) 
       || ! strncasecmp( f, "--h", 3 ) )
    {
      cout << "Validates frame structure in a file" << endl;
      cout << "Usage: " << (string)programname << " INFILE" << endl;
      cout << "As filter: " << (string)programname << " < INFILE" << endl;
      exit(0);
    }
}

int
main( int argc, char* argv[] )
{  
  /* Determine program name (strip filesystem path) */
  for (programname = argv [0] + strlen (argv [0]) - 1;
       programname != argv [0] && *programname != '/';
       programname--)
    ;

  if (*programname == '/') 
    programname++;

  if ( argc > 1 )
    usage( argv[1] );
  
  int ifd;
  if ( argc < 2 ) {
    ifd = 0;
  } else {
    ifd = open( argv[ 1 ], O_RDONLY );
  }

  if ( ifd < 0 )
    {
      fatal( "could not open `" + (string)argv[1] + "' for input" );
    }

#if __GNUC__ >= 3
  std::ifstream in;
  {
    FILE *cfile = fdopen(ifd,"r");
    std::ofstream::__filebuf_type* outbuf
        = new std::ofstream::__filebuf_type (cfile, std::ios_base::out);
    std::ofstream::__streambuf_type* oldoutbuf
        = ((std::ofstream::__ios_type* )&in)->rdbuf (outbuf);
    delete oldoutbuf;
  }
#else
  ifstream in( ifd );
#endif

  unsigned long fpos=0; // frame file index position

  //  Skip header
  char header[40];
  in.read(header,fpos+=40);
  if (!in.good())
	fatal( "failed to read in 40 byte file header" );

  // Read frame structures
  //
  for (;;) {
    unsigned int length;   // length of structure in bytes
    unsigned short classId;  // File Structure class
    unsigned short instance; // Instance count for this class type

    in.read((char*)&length,sizeof(length)); 
    in.read((char*)&classId,sizeof(classId)); 
    in.read((char*)&instance,sizeof(instance)); 
    if (!in.good())
	fatal("failed to read structure header");

    cout << fpos << '\t' << length << '\t' << classId << '\t' << instance;
    if (classId == 4) {
	unsigned short strlen;
	in.read((char*)&strlen, 2);
	char str[strlen];
	in.read(str,strlen);
	cout << '\t' << str;
    }
    cout << endl;
    if ( length < 8 ) {
       cout << "(bad length)" << endl;
       return 1;
    }
    in.seekg(fpos+=length);
    if (!in.good())
	fatal("failed to seek structure length");
  }
  return 0;
}
