#ifndef GENERAL__ERROR_LOG_H
#define GENERAL__ERROR_LOG_H

#include <iostream>

#include "general/mutexlock.hh"
#include "general/types.hh"

namespace General
{
  class ErrorLog
  {
  public:
    enum state {
      ASSERT_FAILURE,
      FATAL,
      DEBUG,
      WARNING,
      INFO
    };

    ErrorLog( std::ostream& Stream, bool OpenState );
    ~ErrorLog( );

    static bool AbortOnAssertFailure( );

    static void AbortOnAssertFailure( bool Value );

    bool IsOpen( ) const;

    bool IsOpen( bool State );

    void MaskBitClear( state State );

    void MaskBitSet( state State );

    void operator()( state State,
		     const char* Filename, const int Line,
		     const std::string& Message,
		     bool EOL = true );

  private:
    static bool			m_abort_on_assert_failure;

    std::ostream&		m_stream;
    MutexLock::lock_type	m_lock;
    bool			m_is_open;
    INT_4U			m_mask;
  };

  inline bool ErrorLog::
  AbortOnAssertFailure( )
  {
    return m_abort_on_assert_failure;
  }
  
  inline void ErrorLog::
  AbortOnAssertFailure( bool Value )
  {
    m_abort_on_assert_failure = Value;
  }

  inline bool ErrorLog::
  IsOpen( ) const
  {
    return m_is_open;
  }

  inline bool ErrorLog::
  IsOpen( bool State )
  {
    bool retval = m_is_open;
    m_is_open = State;
    return retval;
  }

  inline void ErrorLog::
  MaskBitClear( state State )
  {
    m_mask &= ~(INT_4U(1<<State));
  }

  inline void ErrorLog::
  MaskBitSet( state State )
  {
    m_mask |= (1<<State);
  }

  extern ErrorLog StdErrLog;

  extern ErrorLog JournalLog;
}

#endif /* GENERAL__ERROR_LOG_H */
