/* -*- mode: c++; c-basic-offset: 2; -*- */

#ifndef GENERAL__THREAD_HH
#define GENERAL__THREAD_HH

#include <pthread.h>
#include <signal.h>

namespace General
{
  class Thread
  {
  public:
    typedef pthread_t		thread_type;
    typedef pthread_attr_t	attr_type;

    static thread_type NULL_THREAD;

    Thread( );
    virtual ~Thread( );

    void Cancel( ) const;

    void Detach( ) const;

    void Join( ) const;

    bool IsAlive( ) const;

    void Kill( int Signal ) const;

    void Start( );

    bool operator==( const Thread& Source );
    bool operator==( const thread_type Source );

  protected:
    virtual void run( ) = 0;

  private:
    mutable thread_type		m_tid;

    static Thread* start_routine( Thread* ThreadSource );
  };

  inline void Thread::
  Cancel( ) const
  {
    if ( m_tid != NULL_THREAD )
    {
      pthread_cancel( m_tid );
    }
  }

  inline void Thread::
  Detach( ) const
  {
    // Detach the current thread (makes it non-joinable).
    pthread_detach( m_tid );
  }

  inline bool Thread::
  IsAlive( ) const
  {
    if ( pthread_kill( m_tid, 0 ) == 0 )
    {
      return true;
    }
    return false;
  }

  inline void Thread::
  Join( ) const
  {
    pthread_join( m_tid, NULL );
    m_tid = NULL_THREAD;
  }

  inline void Thread::
  Kill( int Signal ) const
  {
    if ( m_tid != NULL_THREAD )
    {
      pthread_kill( m_tid, Signal );
    }
  }

  inline void Thread::
  Start( )
  {
    pthread_create( &m_tid, NULL, (void*(*)(void*))start_routine, this );
  }

  inline bool Thread::
  operator==( const Thread::thread_type Source )
  {
    return( pthread_equal( m_tid, Source ) );
  }

  inline bool Thread::
  operator==( const Thread& Source )
  {
    return( *this == m_tid );
  }
}

#endif /* GENERAL__THREAD_HH */
