#include "general/config.h"

#include "errorinfo.hh"
   
#include <cstring>

using std::strcmp;
using std::strcpy;
using std::strlen;

   
//-----------------------------------------------------------------------------
//
//: Constructor.
//
//!param: int library
//!param: int code
//!param: const std::string& msg
//!param: const std::string& info
//!param: const char* file
//!param: size_t line
//
ErrorInfo::ErrorInfo( int library, int code, const std::string& msg, 
   const std::string& info, const char* file, size_t line )
   : mLibrary( library ), mCode( code ), mMsg( msg ), mInfo( info ),
     mFile( 0 ), mLine( line )
{
   if( file )
   {
      const size_t file_len( strlen( file ) + 1 );
      mFile = new char[ file_len ];
   
      strcpy( mFile, file );
   }
}


//-----------------------------------------------------------------------------
//
//: Copy Constructor
//
//!param: const ErrorInfo& e
//
ErrorInfo::ErrorInfo( const ErrorInfo& e )
   : mLibrary( e.mLibrary ), mCode( e.mCode ), mMsg( e.mMsg ),
     mInfo( e.mInfo ), mFile( 0 ), mLine( e.mLine )
{
   if( e.mFile )
   {
      const size_t file_len( strlen( e.mFile ) + 1 );
      mFile = new char[ file_len ];
   
      strcpy( mFile, e.mFile );   
   }
}
   
   
//-----------------------------------------------------------------------------
//
//: Destructor.
//
ErrorInfo::~ErrorInfo() 
{
   delete[] mFile;
   mFile = 0;
}


//-----------------------------------------------------------------------------
//
//: Equal Comparison
//
//!param: const ErrorInfo& e
//
//!return: bool
//
bool ErrorInfo::operator==( const ErrorInfo& e ) const
{
    return ( mLibrary == e.mLibrary && mCode == e.mCode &&
             mLine == e.mLine && mMsg == e.mMsg &&
             mInfo == e.mInfo && strcmp( mFile, e.mFile ) == 0 );
 }
   

//-----------------------------------------------------------------------------
//
//: Assignment Operator
//
//!param: const ErrorInfo& e
//
//!return: const ErrorInfo&
//
const ErrorInfo& ErrorInfo::operator=( const ErrorInfo& e )
{
    if ( this != &e )
    {
        mLibrary = e.mLibrary;
        mCode = e.mCode;
        mMsg = e.mMsg;
        mInfo = e.mInfo;
   
        delete[] mFile;
        mFile = 0;
     
        if( e.mFile )
        {
           const size_t file_len( strlen( e.mFile ) + 1 );
           mFile = new char[ file_len ];
    
           strcpy( mFile, e.mFile );   
        }
    
        mLine = e.mLine;
    }

    return *this;
}


