#include "general/config.h"

#include "ldasexception.hh"
   
//-----------------------------------------------------------------------------
//
//: Default Constructor.
//
// Creates an exception with no error information.
//
LdasException::LdasException()
   : mErrors()
{}


//-----------------------------------------------------------------------------
//
//: Constructor.
//
//!param: int library
//!param: int code
//!param: const std::string& msg
//!param: const std::string& info
//!param: const char* file
//!param: size_t line
//
LdasException::LdasException(
   int library, int code, const std::string& msg, const std::string& info,
   const char* file, size_t line )
   : mErrors()
{
    mErrors.push_back( ErrorInfo( library, code, msg, info, file, line ) );
}


//-----------------------------------------------------------------------------
//
//: Constructor
//
// Creates an exception holding the passed ErrorInfo object.
//
//!param: const ErrorInfo& e
//
LdasException::LdasException( const ErrorInfo& e )
   : mErrors()
{
    mErrors.push_back( e );
}


//-----------------------------------------------------------------------------
//
//: Copy Constructor
//
//!param: const LdasException& e
//
LdasException::LdasException( const LdasException& e )
   : mErrors( e.mErrors )
{}


//-----------------------------------------------------------------------------
//
//: Destructor
//
LdasException::~LdasException()
{}


//-----------------------------------------------------------------------------
//
//: Assignment Operator
//
//!param: const LdasException& e
//
//!return: LdasException&
//
LdasException& LdasException::operator=( const LdasException& e )
{
    if ( this != &e )
    {
        mErrors = e.mErrors;
    }

    return *this;
}

   
//-----------------------------------------------------------------------------
//
//: Get Error Info
//
//!param: size_t i
//
//!return: const ErrorInfo&
//   
//!exc: range_error   
//
const ErrorInfo& LdasException::getError( size_t i ) const
{
    if ( i >= mErrors.size() )
    {
        throw std::range_error( "index out of range" );
    }

    return mErrors[ i ];
}

    
//-----------------------------------------------------------------------------
//
//: Array Operator
//
//!param: size_t i
//
//!return: const ErrorInfo&
//
//!exc: range_error
//
const ErrorInfo& LdasException::operator[]( size_t i ) const
{
    return getError( i );
}
   

//-----------------------------------------------------------------------------
//
//: Add Error
//
// Add another error info object to the class.
//
//!param: const ErrorInfo& e
//
void LdasException::addError( const ErrorInfo& e )
{
    mErrors.push_back( e );
    return;
}
   
    
//-----------------------------------------------------------------------------
//
//: Add Error
//
// Add another error info object to the class.
//
//!param: int library
//!param: int code
//!param: const std::string& msg
//!param: const strin& info
//!param: const char* file
//!param: size_t line
//
void LdasException::addError( int library, int code,
   const std::string& msg, const std::string& info,
   const char* file, size_t line )
{
    addError( ErrorInfo( library, code, msg, info, file, line ) );
    return;
}
   
