#ifndef GeneralUtilHH
#define GeneralUtilHH

#include <assert.h>
#include <pthread.h>
#include <string>
#include <strings.h>

#include "general/config.h"
#include "general/hash_map.hh"

//-----------------------------------------------------------------------------
//
//: Reverse Byte Order
//
//!param: void* dest
//!param: size_t n
//
template< size_t SIZE > void reverse( void* s, size_t n );


// Template specialization declarations
template<> void reverse< 1 >( void* s, size_t n );
template<> void reverse< 2 >( void* s, size_t n );
template<> void reverse< 4 >( void* s, size_t n );
template<> void reverse< 8 >( void* s, size_t n );

template<> inline void
reverse< 36U >( void* s, size_t n )
{
  assert( 0 );
}

std::string slower( const std::string &s );
void string2lower( std::string& s );

namespace General
{
  int cmp_nocase( const std::string& S1, const std::string& S2 );

#if !defined(hash_map)

  //: Hash proto
  template <class T> struct ic_hash { };

//-----------------------------------------------------------------------------
//
//: Case-Insensitive String Hash Specialization
//
// A specialization of the STL hash functional to perform a case-insensitive
// hash of a C++ string.
//
template<>
struct ic_hash< std::string >
{
  //: Overloaded call operator.
  //  
  //!param: const std::string& s - A reference to the string representing the   +
  //!param:    search key.
  //
  //!return: size_t - Index into the hash.
  // 
  size_t operator()( const std::string& s ) const
  {
    std::string tmp( slower( s ));
    HASH_NAMESPACE::hash< const char* > h;
    return h( tmp.c_str() );
  }
};

#endif	/* hash_map */

  void
  Rethrow( const std::string& Prefix, const std::exception& Exception );
} // namespace General

//-----------------------------------------------------------------------------
//
//: Case-Insensitive Compare Functional
//
// A Function Object used to perform a case-insensitive compare of two string
// objects.
//
class CaseInsensitiveCmp
{
public:

    //: Overloaded call operator.
    //  
    //!param: const std::string& s1 - A reference to the first string object.
    //!param: const std::string& s2 - A reference to the second string object.
    //
    //!return: bool - True if strings are equal, false otherwise.
    //     
    bool operator()( const std::string& s1, const std::string& s2 ) const
        {
            return General::cmp_nocase( s1, s2 ) == 0;
        }
};

   
const bool is_space( const char c );   
const bool not_space( const char c );      
   

#endif // GeneralUtilHH
