#include "general/config.h"

#include <signal.h>
#include <time.h>
#include <unistd.h>

#include <list>

#include "general/unittest.h"

#include "general/ErrorLog.hh"
#include "general/TaskTimer.hh"
#include "general/Timeout.hh"

General::UnitTest	Test;
using General::Timeout;
using General::Timer;

#define WAITER_SIGNAL SIGUSR1

struct sleeper_info_type {
  pthread_t		s_thread;
  int			s_sleep;
  int			s_wakeup;
};

typedef std::list< sleeper_info_type* >	sleepers_type;

sleepers_type	Sleepers;

void
wakeup( int SignalNumber )
{
  signal( SignalNumber, wakeup );
}

void*
sleeper( void* Data )
{
  //---------------------------------------------------------------------
  // Setup the timer and wait for the results
  //---------------------------------------------------------------------
  sleeper_info_type*	info = reinterpret_cast<sleeper_info_type*>( Data );
  Timer			nsleep( info->s_sleep, WAITER_SIGNAL );
  {
    Timeout( nsleep, info->s_wakeup );
  }

  //---------------------------------------------------------------------
  // Check the results
  //---------------------------------------------------------------------
  bool		status;
  std::string	alarm;
  if ( info->s_sleep < info->s_wakeup )
  {
    alarm = "before";
    status = ! nsleep.Error( );
  }
  else
  {
    alarm = "after";
    status = nsleep.Error( );
  }
  Test.Check( status )
    << " Task completes " << alarm << " alarm rings: (task: " << info->s_sleep
    << " timeout: " << info->s_wakeup << ")"
    << std::endl;
  return (void*)NULL;
}


//-----------------------------------------------------------------------
// Wrapper function to start the thread
//-----------------------------------------------------------------------
void
sleeper_test( int Sleep, int Wakeup )
{
  Test.Message( )
    << "Setting up sleeper_test: Task: " << Sleep
    << " Timeout: " << Wakeup
    << " Signal: " << WAITER_SIGNAL
    << std::endl;
  sleeper_info_type*	retval = new sleeper_info_type;

  retval->s_sleep = Sleep;
  retval->s_wakeup = Wakeup;
  Sleepers.push_back( retval );
  pthread_create( &(retval->s_thread), NULL, sleeper, retval );
}

int
main( int ArgC, char** ArgV )
{
  //---------------------------------------------------------------------
  // Initialize the test structure
  //---------------------------------------------------------------------
  Test.Init(ArgC, ArgV);

  signal( WAITER_SIGNAL, wakeup );
  //---------------------------------------------------------------------
  // Set to true for verbose output, false for no output from TimerThread
  //   class.
  //---------------------------------------------------------------------
  General::StdErrLog.IsOpen( false );

  //---------------------------------------------------------------------
  // Setup and run the differnet threads
  //---------------------------------------------------------------------

  sleeper_test( 5, 10 );
  sleeper_test( 10, 5 );
  sleeper_test( 20, 18 );
  sleeper_test( 10, 20 );
  sleep( 30 );
  sleeper_test( 5, 10 );
  sleeper_test( 10, 5 );
  sleeper_test( 20, 18 );
  sleeper_test( 10, 20 );

  //---------------------------------------------------------------------
  // Wait for all threads to terminate
  //---------------------------------------------------------------------
  while( Sleepers.size( ) > 0 )
  {
    //-------------------------------------------------------------------
    // Get rid of any threads that did not really start
    //-------------------------------------------------------------------
    if ( Sleepers.front( )->s_thread == (pthread_t)NULL )
    {
      delete Sleepers.front( );
      Sleepers.pop_front( );
      continue;
    }
    //-------------------------------------------------------------------
    // Join threads that started for real
    //-------------------------------------------------------------------
    pthread_join( Sleepers.front( )->s_thread, NULL );
    delete Sleepers.front( );
    Sleepers.pop_front( );
  }
  //---------------------------------------------------------------------
  // Exit with the appropriate exit status
  //---------------------------------------------------------------------
  Test.Exit();
  return 1;
}


